--
-- PostgreSQL database dump
--

SET client_encoding = 'UNICODE';
SET check_function_bodies = false;

SET SESSION AUTHORIZATION 'postgres';

SET search_path = public, pg_catalog;

--
-- TOC entry 153 (OID 22485423)
-- Name: plpgsql_call_handler(); Type: FUNC PROCEDURAL LANGUAGE; Schema: public; Owner: postgres
--

CREATE FUNCTION plpgsql_call_handler() RETURNS language_handler
    AS '$libdir/plpgsql', 'plpgsql_call_handler'
    LANGUAGE c;


SET SESSION AUTHORIZATION DEFAULT;

--
-- TOC entry 152 (OID 22485424)
-- Name: plpgsql; Type: PROCEDURAL LANGUAGE; Schema: public; Owner: 
--

CREATE TRUSTED PROCEDURAL LANGUAGE plpgsql HANDLER plpgsql_call_handler;


SET SESSION AUTHORIZATION 'postgres';

--
-- TOC entry 4 (OID 2200)
-- Name: public; Type: ACL; Schema: -; Owner: postgres
--

REVOKE ALL ON SCHEMA public FROM PUBLIC;
REVOKE ALL ON SCHEMA public FROM postgres;
GRANT ALL ON SCHEMA public TO PUBLIC;


SET SESSION AUTHORIZATION 'conflux';

--
-- TOC entry 5 (OID 22485425)
-- Name: item_id_seq; Type: SEQUENCE; Schema: public; Owner: conflux
--

CREATE SEQUENCE item_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- TOC entry 7 (OID 22485427)
-- Name: scrollback_id_seq; Type: SEQUENCE; Schema: public; Owner: conflux
--

CREATE SEQUENCE scrollback_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- TOC entry 25 (OID 22485429)
-- Name: access_rights; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE access_rights (
    item_id integer NOT NULL,
    group_id integer NOT NULL,
    right_id smallint NOT NULL,
    treepos bit varying NOT NULL
);


--
-- TOC entry 26 (OID 22485434)
-- Name: access_groups; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE access_groups (
    group_id integer NOT NULL,
    person_id integer NOT NULL
);


--
-- TOC entry 27 (OID 22485436)
-- Name: items; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE items (
    item_id integer DEFAULT nextval('item_id_seq'::text) NOT NULL,
    parent_id integer NOT NULL,
    item_name text NOT NULL,
    item_size integer NOT NULL,
    creator_id integer NOT NULL,
    creator_ip text NOT NULL,
    create_time timestamp with time zone DEFAULT now() NOT NULL,
    editor_id integer NOT NULL,
    editor_ip text NOT NULL,
    edit_time timestamp with time zone DEFAULT now() NOT NULL,
    item_type_id integer NOT NULL,
    is_internal boolean DEFAULT false NOT NULL,
    is_public boolean DEFAULT false NOT NULL,
    is_private boolean DEFAULT false NOT NULL,
    treepos bit varying NOT NULL,
    owner_id integer,
    index_this_item integer DEFAULT 0
);


--
-- TOC entry 9 (OID 22485448)
-- Name: item_type_id_seq; Type: SEQUENCE; Schema: public; Owner: conflux
--

CREATE SEQUENCE item_type_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- TOC entry 28 (OID 22485450)
-- Name: item_types; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE item_types (
    item_type_id integer DEFAULT nextval('item_type_id_seq'::text) NOT NULL,
    item_type_name character varying(128) NOT NULL,
    item_type_icon character varying(128)
);


--
-- TOC entry 29 (OID 22485453)
-- Name: sessions; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE sessions (
    session_id integer DEFAULT nextval('session_id_seq'::text) NOT NULL,
    session_key character(16) NOT NULL,
    user_id integer NOT NULL,
    user_ip character varying(16) NOT NULL,
    start_stamp timestamp with time zone DEFAULT now() NOT NULL,
    update_stamp timestamp with time zone DEFAULT now() NOT NULL,
    update_count integer DEFAULT 0 NOT NULL
);


--
-- TOC entry 11 (OID 22485459)
-- Name: session_id_seq; Type: SEQUENCE; Schema: public; Owner: conflux
--

CREATE SEQUENCE session_id_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- TOC entry 30 (OID 22485461)
-- Name: persons; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE persons (
    person_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    first_name text NOT NULL,
    middle_name text,
    last_name text NOT NULL,
    username character varying(64) NOT NULL,
    "password" character(13) NOT NULL,
    phone_work text,
    phone_home text,
    phone_cell text,
    phone_fax text,
    e_mail text,
    is_disabled integer DEFAULT 0 NOT NULL,
    picture text,
    email_notify integer DEFAULT 0 NOT NULL,
    is_deleted integer DEFAULT 0 NOT NULL,
    person_flags integer DEFAULT 0 NOT NULL
);


--
-- TOC entry 31 (OID 22485471)
-- Name: contacts; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE contacts (
    contact_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    first_name text NOT NULL,
    middle_name text,
    last_name text NOT NULL,
    company text,
    department text,
    jobtitle text,
    phone_work text,
    phone_home text,
    phone_cell text,
    phone_fax text,
    e_mail text,
    mailing_address text,
    description text,
    homepage text,
    contact_category text,
    birth_date date,
    contact_sex integer DEFAULT 0,
    e_mail2 text,
    e_mail3 text
);


--
-- TOC entry 32 (OID 22485478)
-- Name: organizations; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE organizations (
    organization_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    organization_name text NOT NULL,
    address text,
    city text,
    state text,
    zipcode text,
    country text,
    website text,
    email text,
    telephone text,
    fax text,
    superuser_id integer
);


--
-- TOC entry 33 (OID 22485484)
-- Name: user_groups; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE user_groups (
    group_id integer DEFAULT nextval('user_groups_seq'::text) NOT NULL,
    group_org_id integer,
    group_name text,
    top_org_id integer
);


--
-- TOC entry 13 (OID 22485490)
-- Name: user_groups_seq; Type: SEQUENCE; Schema: public; Owner: conflux
--

CREATE SEQUENCE user_groups_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- TOC entry 34 (OID 22485492)
-- Name: events; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE events (
    event_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    event_name character varying(256) NOT NULL,
    event_priority text,
    event_activity text,
    start_stamp timestamp with time zone,
    end_stamp timestamp with time zone,
    description text,
    comments text,
    remind_type integer DEFAULT 0,
    remind_time interval,
    related_persons integer[] DEFAULT '{}'::integer[] NOT NULL,
    seen_persons integer[] DEFAULT '{}'::integer[] NOT NULL,
    accepted_persons integer[] DEFAULT '{}'::integer[] NOT NULL,
    rejected_persons integer[] DEFAULT '{}'::integer[] NOT NULL,
    repeat_interval integer,
    dayofweek integer,
    dayofmonth integer,
    monthofyear integer,
    repeat_type integer,
    last_reminder timestamp with time zone,
    related_groups integer[] DEFAULT '{}'::integer[] NOT NULL,
    event_flags integer DEFAULT 0 NOT NULL
);


--
-- TOC entry 35 (OID 22485505)
-- Name: emails; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE emails (
    email_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    email_from text NOT NULL,
    email_subject text NOT NULL,
    email_to text NOT NULL,
    email_date timestamp with time zone NOT NULL,
    email_savedate timestamp with time zone DEFAULT now() NOT NULL,
    email_size integer NOT NULL,
    email_flags integer DEFAULT 0 NOT NULL,
    email_message_id text,
    email_contents text NOT NULL,
    email_contacts text,
    email_from_addr text
);


--
-- TOC entry 36 (OID 22485513)
-- Name: efolders; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE efolders (
    efolder_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    efolder_name text NOT NULL,
    efolder_flags integer NOT NULL
);


--
-- TOC entry 37 (OID 22485519)
-- Name: eaccounts; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE eaccounts (
    eaccount_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    eaccount_name text NOT NULL,
    eaccount_user text NOT NULL,
    eaccount_password text NOT NULL,
    eaccount_flags integer NOT NULL
);


--
-- TOC entry 38 (OID 22485525)
-- Name: folders; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE folders (
    folder_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    folder_name character varying(128),
    comments text
);


--
-- TOC entry 39 (OID 22485531)
-- Name: documents; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE documents (
    document_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    filename text NOT NULL,
    title character varying(256) NOT NULL,
    description text,
    comments text,
    author text,
    mime_major text DEFAULT 'application'::text NOT NULL,
    mime_minor text DEFAULT 'binary'::text NOT NULL,
    final_version_id integer,
    doc_category text,
    locker_id integer,
    lock_due timestamp with time zone
);


--
-- TOC entry 40 (OID 22485539)
-- Name: todos; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE todos (
    todo_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    related_persons integer[] DEFAULT '{}'::integer[] NOT NULL,
    seen_persons integer[] DEFAULT '{}'::integer[] NOT NULL,
    accepted_persons integer[] DEFAULT '{}'::integer[] NOT NULL,
    rejected_persons integer[] DEFAULT '{}'::integer[] NOT NULL,
    title text NOT NULL,
    status text,
    priority text,
    category text,
    remind_type integer DEFAULT 0,
    remind_time interval,
    description text,
    comments text,
    percent_complete integer DEFAULT 0 NOT NULL,
    workflow_id integer,
    workflow_sequence_id integer,
    start_time date,
    due_time date
);


--
-- TOC entry 41 (OID 22485551)
-- Name: settings; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE settings (
    setting_key character varying(64) NOT NULL,
    value text NOT NULL,
    is_configurable integer DEFAULT 0 NOT NULL,
    treepos bit varying
);


--
-- TOC entry 42 (OID 22485557)
-- Name: emails_received; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE emails_received (
    email_account text,
    email_message_id text,
    email_pop3_uid text,
    parent_id integer NOT NULL
);


--
-- TOC entry 43 (OID 22485562)
-- Name: usernotes; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE usernotes (
    note_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    note_message text NOT NULL,
    note_color integer DEFAULT 0 NOT NULL
);


--
-- TOC entry 44 (OID 22485569)
-- Name: disccategories; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE disccategories (
    disccategory_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    disccategory_name text NOT NULL
);


--
-- TOC entry 15 (OID 22485575)
-- Name: systemnote_seq; Type: SEQUENCE; Schema: public; Owner: conflux
--

CREATE SEQUENCE systemnote_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- TOC entry 45 (OID 22485577)
-- Name: systemnotes; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE systemnotes (
    note_id integer DEFAULT nextval('systemnote_seq'::text) NOT NULL,
    note_message text NOT NULL,
    note_stamp date DEFAULT now()
);


--
-- TOC entry 46 (OID 22485584)
-- Name: discussions; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE discussions (
    discussion_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    disc_subject text NOT NULL,
    disc_message text,
    is_front_page boolean DEFAULT false NOT NULL
);


--
-- TOC entry 47 (OID 22485591)
-- Name: resources; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE resources (
    resource_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    resource_name text NOT NULL
);


--
-- TOC entry 48 (OID 22485597)
-- Name: event_resources; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE event_resources (
    event_id integer,
    resource_id integer
);


--
-- TOC entry 49 (OID 22485599)
-- Name: item_rels; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE item_rels (
    left_item_id integer,
    right_item_id integer,
    relation_flags integer DEFAULT 0 NOT NULL
);


--
-- TOC entry 50 (OID 22485602)
-- Name: bookmarks; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE bookmarks (
    bookmark_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    bookmark_url text NOT NULL,
    bookmark_description text,
    is_category integer
);


--
-- TOC entry 51 (OID 22485608)
-- Name: workflows; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE workflows (
    workflow_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    workflow_title text NOT NULL,
    workflow_description text,
    workflow_flags integer DEFAULT 0 NOT NULL,
    related_persons integer[] DEFAULT '{}'::integer[] NOT NULL,
    percent_complete integer DEFAULT 0 NOT NULL,
    workflow_start_time date,
    workflow_due_time date
);


--
-- TOC entry 52 (OID 22485617)
-- Name: calendar_groups; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE calendar_groups (
    parent_id integer NOT NULL,
    group_name text NOT NULL,
    members integer[] DEFAULT '{}'::integer[] NOT NULL,
    group_id integer DEFAULT nextval('calendar_groups_seq'::text) NOT NULL
);


--
-- TOC entry 17 (OID 22485624)
-- Name: calendar_groups_seq; Type: SEQUENCE; Schema: public; Owner: conflux
--

CREATE SEQUENCE calendar_groups_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- TOC entry 53 (OID 22485626)
-- Name: email_filters; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE email_filters (
    filter_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    condition_id integer NOT NULL,
    keywords text NOT NULL,
    action_id integer NOT NULL,
    folder_id integer NOT NULL,
    use_case boolean DEFAULT false NOT NULL,
    priority integer DEFAULT 1 NOT NULL
);


--
-- TOC entry 154 (OID 22485634)
-- Name: items_treepos_trig(); Type: FUNCTION; Schema: public; Owner: conflux
--

CREATE FUNCTION items_treepos_trig() RETURNS "trigger"
    AS '
  DECLARE
    parent_rec RECORD;
  BEGIN
    SELECT INTO parent_rec treepos FROM items WHERE item_id = NEW.parent_id;
    NEW.treepos := parent_rec.treepos || "bit"(NEW.item_id);
    RETURN NEW;
  END;
'
    LANGUAGE plpgsql;


--
-- TOC entry 54 (OID 22485635)
-- Name: clients; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE clients (
    client_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    client_name text NOT NULL,
    client_nr text,
    telephone text,
    fax text,
    e_mail text,
    registry_code text,
    client_type text,
    client_field text,
    client_rating integer,
    client_ownership text,
    homepage text,
    vat_nr text,
    country text,
    mail_address text,
    legal_address text,
    description text,
    bank_reqs text
);


--
-- TOC entry 55 (OID 22485641)
-- Name: opportunities; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE opportunities (
    opp_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    opp_name text NOT NULL,
    client_id integer NOT NULL,
    opp_stage integer NOT NULL,
    opp_next_stage text,
    description text,
    req_descr text,
    amount numeric(10,2),
    close_date date NOT NULL,
    opp_lost text,
    lead_source text,
    client_type text
);


--
-- TOC entry 56 (OID 22485647)
-- Name: products; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE products (
    product_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    product_name text NOT NULL,
    product_group text,
    product_unit text,
    product_price numeric(10,2),
    description text
);


--
-- TOC entry 57 (OID 22485653)
-- Name: opp_products; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE opp_products (
    opp_id integer NOT NULL,
    product_id integer NOT NULL,
    amount numeric(10,2),
    price numeric(10,2),
    key_requirement text
);


--
-- TOC entry 58 (OID 22485658)
-- Name: fti_search; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE fti_search (
    word text NOT NULL,
    item_id integer NOT NULL,
    fti_type boolean DEFAULT false NOT NULL
);


--
-- TOC entry 19 (OID 22485664)
-- Name: log_batch_seq; Type: SEQUENCE; Schema: public; Owner: conflux
--

CREATE SEQUENCE log_batch_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- TOC entry 59 (OID 22485666)
-- Name: item_log; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE item_log (
    item_id integer NOT NULL,
    batch_id integer NOT NULL,
    person_id integer NOT NULL,
    change_time timestamp with time zone DEFAULT now() NOT NULL,
    field_name text NOT NULL,
    old_value text NOT NULL,
    new_value text NOT NULL,
    log_flags integer DEFAULT 0 NOT NULL
);


--
-- TOC entry 60 (OID 22485673)
-- Name: items_scrollback; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE items_scrollback (
    item_id integer NOT NULL,
    parent_id integer NOT NULL,
    item_name text NOT NULL,
    item_type_id integer NOT NULL,
    deleter_id integer NOT NULL,
    delete_time timestamp with time zone DEFAULT now() NOT NULL,
    conflux_tid integer NOT NULL,
    scrollback_flags integer DEFAULT 0 NOT NULL
);


--
-- TOC entry 61 (OID 22485680)
-- Name: quotas; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE quotas (
    quota_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    quota_soft_size integer NOT NULL,
    quota_hard_size integer NOT NULL,
    quota_used bigint DEFAULT 0 NOT NULL
);


--
-- TOC entry 62 (OID 22485684)
-- Name: im; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE im (
    msg_id integer DEFAULT nextval('im_seq'::text) NOT NULL,
    source_id integer,
    dest_id integer,
    message text,
    date timestamp with time zone
);


--
-- TOC entry 21 (OID 22485690)
-- Name: im_seq; Type: SEQUENCE; Schema: public; Owner: conflux
--

CREATE SEQUENCE im_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- TOC entry 63 (OID 22485692)
-- Name: dictionaries; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE dictionaries (
    person_id integer NOT NULL,
    "language" text NOT NULL,
    word text NOT NULL
);


--
-- TOC entry 64 (OID 22485697)
-- Name: contact_groups; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE contact_groups (
    group_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    group_name text NOT NULL,
    group_description text,
    contacts integer[] DEFAULT '{}'::integer[] NOT NULL
);


--
-- TOC entry 65 (OID 22485704)
-- Name: virt_folders; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE virt_folders (
    virt_folder_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    folder_name character varying(128),
    comments text
);


--
-- TOC entry 66 (OID 22485710)
-- Name: messages; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE messages (
    message_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    message_source integer NOT NULL,
    sender_org_id integer,
    sender_org_name text,
    sender_nr text,
    receiver_org_id integer,
    receiver_org_name text,
    receiver_person_id integer,
    receiver_person_name text,
    responsible_person integer,
    message_contents text,
    message_type text,
    delivery_method text,
    priority text,
    date_dispatch timestamp with time zone,
    date_arrival timestamp with time zone,
    date_deadline timestamp with time zone,
    date_answered timestamp with time zone,
    message_title text NOT NULL
);


--
-- TOC entry 23 (OID 22485716)
-- Name: dus_job_seq; Type: SEQUENCE; Schema: public; Owner: conflux
--

CREATE SEQUENCE dus_job_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;


--
-- TOC entry 67 (OID 22485718)
-- Name: dus_jobs; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE dus_jobs (
    job_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    job_number integer DEFAULT nextval('dus_job_seq'::text) NOT NULL,
    job_status integer NOT NULL,
    description text,
    received timestamp with time zone NOT NULL,
    due_translation timestamp with time zone NOT NULL,
    due_editing timestamp with time zone NOT NULL,
    due_checking timestamp with time zone NOT NULL,
    client_id integer NOT NULL,
    category text NOT NULL,
    person_translation integer NOT NULL,
    person_editing integer NOT NULL,
    person_checking integer NOT NULL,
    job_flags integer DEFAULT 0 NOT NULL
);


--
-- TOC entry 68 (OID 22485726)
-- Name: dus_job_rows; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE dus_job_rows (
    job_id integer NOT NULL,
    service_id integer NOT NULL,
    price numeric(12,2) NOT NULL,
    price_mod_percent integer NOT NULL,
    unit_count numeric(12,2) NOT NULL,
    jobrow_flags integer DEFAULT 0 NOT NULL
);


--
-- TOC entry 69 (OID 22485729)
-- Name: dus_services; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE dus_services (
    service_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    language_set character(7) NOT NULL,
    description text NOT NULL,
    unit_name text NOT NULL,
    unit_price numeric(12,2) DEFAULT 0 NOT NULL,
    unit_count integer DEFAULT 1 NOT NULL,
    service_flags integer DEFAULT 0 NOT NULL,
    service_article text NOT NULL
);


--
-- TOC entry 70 (OID 22485738)
-- Name: dus_translations; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE dus_translations (
    translation_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    filename text NOT NULL
);


--
-- TOC entry 71 (OID 22485744)
-- Name: dus_languages; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE dus_languages (
    language_set character(7) NOT NULL,
    description text NOT NULL
);


--
-- TOC entry 72 (OID 22485749)
-- Name: dus_translators; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE dus_translators (
    person_id integer NOT NULL,
    language_set character(7) NOT NULL
);


--
-- TOC entry 73 (OID 22485751)
-- Name: rpc_sessions; Type: TABLE; Schema: public; Owner: conflux
--

CREATE TABLE rpc_sessions (
    session_id integer NOT NULL,
    user_id integer NOT NULL,
    login_time timestamp with time zone NOT NULL,
    timezone_offset integer
);


--
-- Data for TOC entry 155 (OID 22485429)
-- Name: access_rights; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY access_rights (item_id, group_id, right_id, treepos) FROM stdin;
3	1	2	0000000000000000000000000000000000000000000000000000000000000011
\.


--
-- Data for TOC entry 156 (OID 22485434)
-- Name: access_groups; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY access_groups (group_id, person_id) FROM stdin;
\.


--
-- Data for TOC entry 157 (OID 22485436)
-- Name: items; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY items (item_id, parent_id, item_name, item_size, creator_id, creator_ip, create_time, editor_id, editor_ip, edit_time, item_type_id, is_internal, is_public, is_private, treepos, owner_id, index_this_item) FROM stdin;
0	0	Root Organization	0	0	127.0.0.1	2003-04-15 03:31:30.587225+03	0	127.0.0.1	2003-04-15 03:31:30.587225+03	2	f	f	f	00000000000000000000000000000000	\N	\N
1	3	System Administrator	0	0	127.0.0.1	2003-04-15 03:31:30.587225+03	1	213.168.15.186	2005-02-09 20:14:14.722005+02	1	f	f	f	000000000000000000000000000000000000000000000000000000000000001100000000000000000000000000000001	\N	0
3	0	System Organization	0	0	127.0.0.1	2003-04-15 03:31:30.587225+03	1	213.168.15.186	2005-02-06 17:58:09.985822+02	2	f	f	f	0000000000000000000000000000000000000000000000000000000000000011	\N	0
\.


--
-- Data for TOC entry 158 (OID 22485450)
-- Name: item_types; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY item_types (item_type_id, item_type_name, item_type_icon) FROM stdin;
16	Workflows	workflow.gif
1	Persons	person.gif
2	Organizations	toporg.gif
3	Folders	fld.gif
4	Documents	document.gif
5	Events	event.gif
6	Todos	todo.gif
7	Contacts	contact.gif
8	eFolders	efolder.gif
9	e-mails	email.gif
10	eAccounts	eaccount.gif
12	UserNotes	usernote.gif
11	Discussions	discussion.gif
14	Calendar resources	resource.gif
13	Discussion categories	disccategory.gif
15	Bookmarks	bookmark.gif
17	E-filter	efilter.gif
18	Client	client.gif
19	Opportunity	opportunity.gif
20	Product	product.gif
21	Quota	quota.gif
25	Messages	message.gif
23	Contact groups	contactgroups.gif
26	Translation	translation.gif
24	Virtual folders	fld.gif
27	Service	services.gif
28	Translation	document.gif
\.


--
-- Data for TOC entry 159 (OID 22485453)
-- Name: sessions; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY sessions (session_id, session_key, user_id, user_ip, start_stamp, update_stamp, update_count) FROM stdin;
\.


--
-- Data for TOC entry 160 (OID 22485461)
-- Name: persons; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY persons (person_id, first_name, middle_name, last_name, username, "password", phone_work, phone_home, phone_cell, phone_fax, e_mail, is_disabled, picture, email_notify, is_deleted, person_flags) FROM stdin;
1	System		Administrator	superuser	A63jx4ac46ytU						0		1	0	3
\.


--
-- Data for TOC entry 161 (OID 22485471)
-- Name: contacts; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY contacts (contact_id, first_name, middle_name, last_name, company, department, jobtitle, phone_work, phone_home, phone_cell, phone_fax, e_mail, mailing_address, description, homepage, contact_category, birth_date, contact_sex, e_mail2, e_mail3) FROM stdin;
\.


--
-- Data for TOC entry 162 (OID 22485478)
-- Name: organizations; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY organizations (organization_id, organization_name, address, city, state, zipcode, country, website, email, telephone, fax, superuser_id) FROM stdin;
0	Root organization	\N	\N	\N	\N	\N	\N	\N	\N	\N	\N
3	System Organization										1
\.


--
-- Data for TOC entry 163 (OID 22485484)
-- Name: user_groups; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY user_groups (group_id, group_org_id, group_name, top_org_id) FROM stdin;
1	3	Default group	\N
\.


--
-- Data for TOC entry 164 (OID 22485492)
-- Name: events; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY events (event_id, event_name, event_priority, event_activity, start_stamp, end_stamp, description, comments, remind_type, remind_time, related_persons, seen_persons, accepted_persons, rejected_persons, repeat_interval, dayofweek, dayofmonth, monthofyear, repeat_type, last_reminder, related_groups, event_flags) FROM stdin;
\.


--
-- Data for TOC entry 165 (OID 22485505)
-- Name: emails; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY emails (email_id, email_from, email_subject, email_to, email_date, email_savedate, email_size, email_flags, email_message_id, email_contents, email_contacts, email_from_addr) FROM stdin;
\.


--
-- Data for TOC entry 166 (OID 22485513)
-- Name: efolders; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY efolders (efolder_id, efolder_name, efolder_flags) FROM stdin;
\.


--
-- Data for TOC entry 167 (OID 22485519)
-- Name: eaccounts; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY eaccounts (eaccount_id, eaccount_name, eaccount_user, eaccount_password, eaccount_flags) FROM stdin;
\.


--
-- Data for TOC entry 168 (OID 22485525)
-- Name: folders; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY folders (folder_id, folder_name, comments) FROM stdin;
\.


--
-- Data for TOC entry 169 (OID 22485531)
-- Name: documents; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY documents (document_id, filename, title, description, comments, author, mime_major, mime_minor, final_version_id, doc_category, locker_id, lock_due) FROM stdin;
\.


--
-- Data for TOC entry 170 (OID 22485539)
-- Name: todos; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY todos (todo_id, related_persons, seen_persons, accepted_persons, rejected_persons, title, status, priority, category, remind_type, remind_time, description, comments, percent_complete, workflow_id, workflow_sequence_id, start_time, due_time) FROM stdin;
\.


--
-- Data for TOC entry 171 (OID 22485551)
-- Name: settings; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY settings (setting_key, value, is_configurable, treepos) FROM stdin;
todo_statuses	KGxwMQpTJ05vdCBzdGFydGVkJwphUydTdGFydGVkJwpwMgphUydEb25lJwpwMwphUydOb3QgRG9u\nZScKYVMnQ291bGQgbm90IGRvJwphLg==\n	1	\N
user_language	Uydlbl9VUycKcDAKLg==\n	1	\N
todo_priorities	KGxwMQpWSXJyZWxldmFudAphVk5vcm1hbAphVkltcG9ydGFudAphVkNyaXRpY2FsCmEu\n	1	\N
user_dateformat	VmVuX1VTCnAxCi4=\n	1	\N
todo_categories	KGxwMQpWV29yayByZWxhdGVkCmFWUGVyc29uYWwKYS4=\n	1	\N
contact_categories	KGxwMQpWQnVzaW5lc3MKYVZQZXJzb25hbAphVk90aGVyCmEu\n	1	\N
user_timezone	VjI3OjEzCnAxCi4==\n	1	\N
doc_categories	KGxwMQpWQ29udHJhY3QKcDIKYVZEb2N1bWVudApwMwphVkZpbGUKcDQKYVZNZW1vCnA1CmFWUHJv\ndG9jb2wKcDYKYVZSZXBvcnQKcDcKYS4=\n	1	\N
cal_priorities	KGxwMQpWVHJpdmlhbApwMgphVk5vcm1hbApwMwphVkltcG9ydGFudApwNAphVkNyaXRpY2FsCnA1\nCmEu\n	1	\N
dictionary_languages	KGxwMQpWZW5fVVMKcDIKYVZldF9FRQpwMwphLg==\n	1	\N
\.


--
-- Data for TOC entry 172 (OID 22485557)
-- Name: emails_received; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY emails_received (email_account, email_message_id, email_pop3_uid, parent_id) FROM stdin;
\.


--
-- Data for TOC entry 173 (OID 22485562)
-- Name: usernotes; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY usernotes (note_id, note_message, note_color) FROM stdin;
\.


--
-- Data for TOC entry 174 (OID 22485569)
-- Name: disccategories; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY disccategories (disccategory_id, disccategory_name) FROM stdin;
\.


--
-- Data for TOC entry 175 (OID 22485577)
-- Name: systemnotes; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY systemnotes (note_id, note_message, note_stamp) FROM stdin;
\.


--
-- Data for TOC entry 176 (OID 22485584)
-- Name: discussions; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY discussions (discussion_id, disc_subject, disc_message, is_front_page) FROM stdin;
\.


--
-- Data for TOC entry 177 (OID 22485591)
-- Name: resources; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY resources (resource_id, resource_name) FROM stdin;
\.


--
-- Data for TOC entry 178 (OID 22485597)
-- Name: event_resources; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY event_resources (event_id, resource_id) FROM stdin;
\.


--
-- Data for TOC entry 179 (OID 22485599)
-- Name: item_rels; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY item_rels (left_item_id, right_item_id, relation_flags) FROM stdin;
\.


--
-- Data for TOC entry 180 (OID 22485602)
-- Name: bookmarks; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY bookmarks (bookmark_id, bookmark_url, bookmark_description, is_category) FROM stdin;
\.


--
-- Data for TOC entry 181 (OID 22485608)
-- Name: workflows; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY workflows (workflow_id, workflow_title, workflow_description, workflow_flags, related_persons, percent_complete, workflow_start_time, workflow_due_time) FROM stdin;
\.


--
-- Data for TOC entry 182 (OID 22485617)
-- Name: calendar_groups; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY calendar_groups (parent_id, group_name, members, group_id) FROM stdin;
\.


--
-- Data for TOC entry 183 (OID 22485626)
-- Name: email_filters; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY email_filters (filter_id, condition_id, keywords, action_id, folder_id, use_case, priority) FROM stdin;
\.


--
-- Data for TOC entry 184 (OID 22485635)
-- Name: clients; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY clients (client_id, client_name, client_nr, telephone, fax, e_mail, registry_code, client_type, client_field, client_rating, client_ownership, homepage, vat_nr, country, mail_address, legal_address, description, bank_reqs) FROM stdin;
\.


--
-- Data for TOC entry 185 (OID 22485641)
-- Name: opportunities; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY opportunities (opp_id, opp_name, client_id, opp_stage, opp_next_stage, description, req_descr, amount, close_date, opp_lost, lead_source, client_type) FROM stdin;
\.


--
-- Data for TOC entry 186 (OID 22485647)
-- Name: products; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY products (product_id, product_name, product_group, product_unit, product_price, description) FROM stdin;
\.


--
-- Data for TOC entry 187 (OID 22485653)
-- Name: opp_products; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY opp_products (opp_id, product_id, amount, price, key_requirement) FROM stdin;
\.


--
-- Data for TOC entry 188 (OID 22485658)
-- Name: fti_search; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY fti_search (word, item_id, fti_type) FROM stdin;
\.


--
-- Data for TOC entry 189 (OID 22485666)
-- Name: item_log; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY item_log (item_id, batch_id, person_id, change_time, field_name, old_value, new_value, log_flags) FROM stdin;
\.


--
-- Data for TOC entry 190 (OID 22485673)
-- Name: items_scrollback; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY items_scrollback (item_id, parent_id, item_name, item_type_id, deleter_id, delete_time, conflux_tid, scrollback_flags) FROM stdin;
\.


--
-- Data for TOC entry 191 (OID 22485680)
-- Name: quotas; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY quotas (quota_id, quota_soft_size, quota_hard_size, quota_used) FROM stdin;
\.


--
-- Data for TOC entry 192 (OID 22485684)
-- Name: im; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY im (msg_id, source_id, dest_id, message, date) FROM stdin;
\.


--
-- Data for TOC entry 193 (OID 22485692)
-- Name: dictionaries; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY dictionaries (person_id, "language", word) FROM stdin;
\.


--
-- Data for TOC entry 194 (OID 22485697)
-- Name: contact_groups; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY contact_groups (group_id, group_name, group_description, contacts) FROM stdin;
\.


--
-- Data for TOC entry 195 (OID 22485704)
-- Name: virt_folders; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY virt_folders (virt_folder_id, folder_name, comments) FROM stdin;
\.


--
-- Data for TOC entry 196 (OID 22485710)
-- Name: messages; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY messages (message_id, message_source, sender_org_id, sender_org_name, sender_nr, receiver_org_id, receiver_org_name, receiver_person_id, receiver_person_name, responsible_person, message_contents, message_type, delivery_method, priority, date_dispatch, date_arrival, date_deadline, date_answered, message_title) FROM stdin;
\.


--
-- Data for TOC entry 197 (OID 22485718)
-- Name: dus_jobs; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY dus_jobs (job_id, job_number, job_status, description, received, due_translation, due_editing, due_checking, client_id, category, person_translation, person_editing, person_checking, job_flags) FROM stdin;
\.


--
-- Data for TOC entry 198 (OID 22485726)
-- Name: dus_job_rows; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY dus_job_rows (job_id, service_id, price, price_mod_percent, unit_count, jobrow_flags) FROM stdin;
\.


--
-- Data for TOC entry 199 (OID 22485729)
-- Name: dus_services; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY dus_services (service_id, language_set, description, unit_name, unit_price, unit_count, service_flags, service_article) FROM stdin;
\.


--
-- Data for TOC entry 200 (OID 22485738)
-- Name: dus_translations; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY dus_translations (translation_id, filename) FROM stdin;
\.


--
-- Data for TOC entry 201 (OID 22485744)
-- Name: dus_languages; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY dus_languages (language_set, description) FROM stdin;
\.


--
-- Data for TOC entry 202 (OID 22485749)
-- Name: dus_translators; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY dus_translators (person_id, language_set) FROM stdin;
\.


--
-- Data for TOC entry 203 (OID 22485751)
-- Name: rpc_sessions; Type: TABLE DATA; Schema: public; Owner: conflux
--

COPY rpc_sessions (session_id, user_id, login_time, timezone_offset) FROM stdin;
\.


--
-- TOC entry 76 (OID 23012305)
-- Name: access_rights_itemid_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX access_rights_itemid_ndx ON access_rights USING btree (item_id);


--
-- TOC entry 91 (OID 23012306)
-- Name: session_userid_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX session_userid_ndx ON sessions USING btree (user_id);


--
-- TOC entry 92 (OID 23012307)
-- Name: session_userip_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX session_userip_ndx ON sessions USING btree (user_ip);


--
-- TOC entry 90 (OID 23012308)
-- Name: session_sessionkey_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX session_sessionkey_ndx ON sessions USING btree (session_key);


--
-- TOC entry 103 (OID 23012309)
-- Name: events_start_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX events_start_ndx ON events USING btree (start_stamp);


--
-- TOC entry 101 (OID 23012310)
-- Name: events_end_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX events_end_ndx ON events USING btree (end_stamp);


--
-- TOC entry 75 (OID 23012311)
-- Name: access_rights_groupid_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX access_rights_groupid_ndx ON access_rights USING btree (group_id);


--
-- TOC entry 77 (OID 23012312)
-- Name: access_rights_rightid_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX access_rights_rightid_ndx ON access_rights USING btree (right_id);


--
-- TOC entry 80 (OID 23012313)
-- Name: access_groups_personid_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX access_groups_personid_ndx ON access_groups USING btree (person_id);


--
-- TOC entry 79 (OID 23012314)
-- Name: access_groups_groupid_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX access_groups_groupid_ndx ON access_groups USING btree (group_id);


--
-- TOC entry 85 (OID 23012315)
-- Name: items_name_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX items_name_ndx ON items USING btree (item_name);


--
-- TOC entry 84 (OID 23012316)
-- Name: items_itemtypeid_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX items_itemtypeid_ndx ON items USING btree (item_type_id);


--
-- TOC entry 99 (OID 23012317)
-- Name: usergroups_grouporgid_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX usergroups_grouporgid_ndx ON user_groups USING btree (group_org_id);


--
-- TOC entry 81 (OID 23012318)
-- Name: items_creatorid_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX items_creatorid_ndx ON items USING btree (creator_id);


--
-- TOC entry 86 (OID 23012319)
-- Name: items_parentid_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX items_parentid_ndx ON items USING btree (parent_id);


--
-- TOC entry 82 (OID 23012320)
-- Name: items_isprivate_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX items_isprivate_ndx ON items USING btree (is_private);


--
-- TOC entry 83 (OID 23012321)
-- Name: items_ispublic_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX items_ispublic_ndx ON items USING btree (is_public);


--
-- TOC entry 117 (OID 23012322)
-- Name: evnt_res_ev_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX evnt_res_ev_ndx ON event_resources USING btree (event_id);


--
-- TOC entry 118 (OID 23012323)
-- Name: evnt_res_res_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX evnt_res_res_ndx ON event_resources USING btree (resource_id);


--
-- TOC entry 119 (OID 23012324)
-- Name: item_rels_left; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX item_rels_left ON item_rels USING btree (left_item_id);


--
-- TOC entry 120 (OID 23012325)
-- Name: item_rels_right; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX item_rels_right ON item_rels USING btree (right_item_id);


--
-- TOC entry 123 (OID 23012326)
-- Name: cal_groups_uniq; Type: INDEX; Schema: public; Owner: conflux
--

CREATE UNIQUE INDEX cal_groups_uniq ON calendar_groups USING btree (parent_id, group_name);


--
-- TOC entry 88 (OID 23012327)
-- Name: items_treepos_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE UNIQUE INDEX items_treepos_ndx ON items USING btree (treepos);


--
-- TOC entry 111 (OID 23012328)
-- Name: settings_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE UNIQUE INDEX settings_ndx ON settings USING btree (setting_key, treepos);


--
-- TOC entry 78 (OID 23012329)
-- Name: access_rights_treepos_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX access_rights_treepos_ndx ON access_rights USING btree (treepos);


--
-- TOC entry 126 (OID 23012330)
-- Name: opp_client_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX opp_client_ndx ON opportunities USING btree (client_id);


--
-- TOC entry 100 (OID 23012331)
-- Name: usergroups_toporgid_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX usergroups_toporgid_ndx ON user_groups USING btree (top_org_id);


--
-- TOC entry 129 (OID 23012332)
-- Name: opp_product_opp_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX opp_product_opp_ndx ON opp_products USING btree (opp_id);


--
-- TOC entry 130 (OID 23012333)
-- Name: opp_product_prod_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX opp_product_prod_ndx ON opp_products USING btree (product_id);


--
-- TOC entry 131 (OID 23012334)
-- Name: fti_item_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX fti_item_ndx ON fti_search USING btree (item_id, fti_type);


--
-- TOC entry 132 (OID 23012335)
-- Name: fti_word_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX fti_word_ndx ON fti_search USING btree (word, fti_type);


--
-- TOC entry 134 (OID 23012336)
-- Name: item_log_item_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX item_log_item_ndx ON item_log USING btree (item_id);


--
-- TOC entry 135 (OID 23012337)
-- Name: item_log_person_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX item_log_person_ndx ON item_log USING btree (person_id);


--
-- TOC entry 133 (OID 23012338)
-- Name: item_log_field_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX item_log_field_ndx ON item_log USING btree (field_name);


--
-- TOC entry 104 (OID 23012339)
-- Name: emails_addr_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX emails_addr_ndx ON emails USING btree (email_from_addr);


--
-- TOC entry 137 (OID 23012340)
-- Name: scrollback_itemid_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE UNIQUE INDEX scrollback_itemid_ndx ON items_scrollback USING btree (conflux_tid, item_id);


--
-- TOC entry 136 (OID 23012341)
-- Name: scrollback_item_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX scrollback_item_ndx ON items_scrollback USING btree (item_id);


--
-- TOC entry 94 (OID 23012342)
-- Name: persons_deleted_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX persons_deleted_ndx ON persons USING btree (is_deleted);


--
-- TOC entry 145 (OID 23012343)
-- Name: dus_jobrows_ndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX dus_jobrows_ndx ON dus_job_rows USING btree (job_id, service_id);


--
-- TOC entry 150 (OID 23012344)
-- Name: rpc_sessions_idndx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX rpc_sessions_idndx ON rpc_sessions USING btree (session_id);


--
-- TOC entry 151 (OID 23012345)
-- Name: rpc_sessions_timendx; Type: INDEX; Schema: public; Owner: conflux
--

CREATE INDEX rpc_sessions_timendx ON rpc_sessions USING btree (login_time);


--
-- TOC entry 87 (OID 23012346)
-- Name: items_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY items
    ADD CONSTRAINT items_pkey PRIMARY KEY (item_id);


--
-- TOC entry 102 (OID 23012348)
-- Name: events_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY events
    ADD CONSTRAINT events_pkey PRIMARY KEY (event_id);


--
-- TOC entry 109 (OID 23012350)
-- Name: documents_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY documents
    ADD CONSTRAINT documents_pkey PRIMARY KEY (document_id);


--
-- TOC entry 105 (OID 23012352)
-- Name: emails_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY emails
    ADD CONSTRAINT emails_pkey PRIMARY KEY (email_id);


--
-- TOC entry 106 (OID 23012354)
-- Name: efolders_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY efolders
    ADD CONSTRAINT efolders_pkey PRIMARY KEY (efolder_id);


--
-- TOC entry 107 (OID 23012356)
-- Name: eaccounts_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY eaccounts
    ADD CONSTRAINT eaccounts_pkey PRIMARY KEY (eaccount_id);


--
-- TOC entry 108 (OID 23012358)
-- Name: folders_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY folders
    ADD CONSTRAINT folders_pkey PRIMARY KEY (folder_id);


--
-- TOC entry 110 (OID 23012360)
-- Name: todos_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY todos
    ADD CONSTRAINT todos_pkey PRIMARY KEY (todo_id);


--
-- TOC entry 89 (OID 23012362)
-- Name: item_types_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY item_types
    ADD CONSTRAINT item_types_pkey PRIMARY KEY (item_type_id);


--
-- TOC entry 93 (OID 23012364)
-- Name: sessions_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY sessions
    ADD CONSTRAINT sessions_pkey PRIMARY KEY (session_id);


--
-- TOC entry 95 (OID 23012366)
-- Name: persons_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY persons
    ADD CONSTRAINT persons_pkey PRIMARY KEY (person_id);


--
-- TOC entry 96 (OID 23012368)
-- Name: contacts_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY contacts
    ADD CONSTRAINT contacts_pkey PRIMARY KEY (contact_id);


--
-- TOC entry 97 (OID 23012370)
-- Name: organizations_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY organizations
    ADD CONSTRAINT organizations_pkey PRIMARY KEY (organization_id);


--
-- TOC entry 98 (OID 23012372)
-- Name: user_groups_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY user_groups
    ADD CONSTRAINT user_groups_pkey PRIMARY KEY (group_id);


--
-- TOC entry 112 (OID 23012374)
-- Name: usernotes_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY usernotes
    ADD CONSTRAINT usernotes_pkey PRIMARY KEY (note_id);


--
-- TOC entry 113 (OID 23012376)
-- Name: disccategories_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY disccategories
    ADD CONSTRAINT disccategories_pkey PRIMARY KEY (disccategory_id);


--
-- TOC entry 114 (OID 23012378)
-- Name: systemnotes_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY systemnotes
    ADD CONSTRAINT systemnotes_pkey PRIMARY KEY (note_id);


--
-- TOC entry 115 (OID 23012380)
-- Name: discussions_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY discussions
    ADD CONSTRAINT discussions_pkey PRIMARY KEY (discussion_id);


--
-- TOC entry 116 (OID 23012382)
-- Name: resources_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY resources
    ADD CONSTRAINT resources_pkey PRIMARY KEY (resource_id);


--
-- TOC entry 121 (OID 23012384)
-- Name: bookmarks_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY bookmarks
    ADD CONSTRAINT bookmarks_pkey PRIMARY KEY (bookmark_id);


--
-- TOC entry 122 (OID 23012386)
-- Name: workflows_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY workflows
    ADD CONSTRAINT workflows_pkey PRIMARY KEY (workflow_id);


--
-- TOC entry 124 (OID 23012388)
-- Name: email_filters_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY email_filters
    ADD CONSTRAINT email_filters_pkey PRIMARY KEY (filter_id);


--
-- TOC entry 125 (OID 23012390)
-- Name: clients_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY clients
    ADD CONSTRAINT clients_pkey PRIMARY KEY (client_id);


--
-- TOC entry 127 (OID 23012392)
-- Name: opportunities_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY opportunities
    ADD CONSTRAINT opportunities_pkey PRIMARY KEY (opp_id);


--
-- TOC entry 128 (OID 23012394)
-- Name: products_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY products
    ADD CONSTRAINT products_pkey PRIMARY KEY (product_id);


--
-- TOC entry 138 (OID 23012396)
-- Name: quotas_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY quotas
    ADD CONSTRAINT quotas_pkey PRIMARY KEY (quota_id);


--
-- TOC entry 139 (OID 23012398)
-- Name: im_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY im
    ADD CONSTRAINT im_pkey PRIMARY KEY (msg_id);


--
-- TOC entry 142 (OID 23012400)
-- Name: virt_folders_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY virt_folders
    ADD CONSTRAINT virt_folders_pkey PRIMARY KEY (virt_folder_id);


--
-- TOC entry 141 (OID 23012402)
-- Name: contact_groups_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY contact_groups
    ADD CONSTRAINT contact_groups_pkey PRIMARY KEY (group_id);


--
-- TOC entry 140 (OID 23012404)
-- Name: dictionaries_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY dictionaries
    ADD CONSTRAINT dictionaries_pkey PRIMARY KEY (person_id, "language", word);


--
-- TOC entry 143 (OID 23012406)
-- Name: messages_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY messages
    ADD CONSTRAINT messages_pkey PRIMARY KEY (message_id);


--
-- TOC entry 147 (OID 23012408)
-- Name: dus_translations_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY dus_translations
    ADD CONSTRAINT dus_translations_pkey PRIMARY KEY (translation_id);


--
-- TOC entry 144 (OID 23012410)
-- Name: dus_jobs_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY dus_jobs
    ADD CONSTRAINT dus_jobs_pkey PRIMARY KEY (job_id);


--
-- TOC entry 146 (OID 23012412)
-- Name: dus_services_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY dus_services
    ADD CONSTRAINT dus_services_pkey PRIMARY KEY (service_id);


--
-- TOC entry 148 (OID 23012414)
-- Name: dus_languages_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY dus_languages
    ADD CONSTRAINT dus_languages_pkey PRIMARY KEY (language_set);


--
-- TOC entry 149 (OID 23012416)
-- Name: dus_translators_pkey; Type: CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY dus_translators
    ADD CONSTRAINT dus_translators_pkey PRIMARY KEY (person_id, language_set);


--
-- TOC entry 216 (OID 23012418)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY events
    ADD CONSTRAINT "$1" FOREIGN KEY (event_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 222 (OID 23012422)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY todos
    ADD CONSTRAINT "$1" FOREIGN KEY (todo_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 212 (OID 23012426)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY contacts
    ADD CONSTRAINT "$1" FOREIGN KEY (contact_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 219 (OID 23012430)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY eaccounts
    ADD CONSTRAINT "$1" FOREIGN KEY (eaccount_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 218 (OID 23012434)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY efolders
    ADD CONSTRAINT "$1" FOREIGN KEY (efolder_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 217 (OID 23012438)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY emails
    ADD CONSTRAINT "$1" FOREIGN KEY (email_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 224 (OID 23012442)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY emails_received
    ADD CONSTRAINT "$1" FOREIGN KEY (parent_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 213 (OID 23012446)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY organizations
    ADD CONSTRAINT "$1" FOREIGN KEY (organization_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 211 (OID 23012450)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY persons
    ADD CONSTRAINT "$1" FOREIGN KEY (person_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 214 (OID 23012454)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY user_groups
    ADD CONSTRAINT "$1" FOREIGN KEY (group_org_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 205 (OID 23012458)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY access_rights
    ADD CONSTRAINT "$1" FOREIGN KEY (item_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 206 (OID 23012462)
-- Name: $2; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY access_rights
    ADD CONSTRAINT "$2" FOREIGN KEY (group_id) REFERENCES user_groups(group_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 208 (OID 23012466)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY access_groups
    ADD CONSTRAINT "$1" FOREIGN KEY (group_id) REFERENCES user_groups(group_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 209 (OID 23012470)
-- Name: $2; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY access_groups
    ADD CONSTRAINT "$2" FOREIGN KEY (person_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 221 (OID 23012474)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY documents
    ADD CONSTRAINT "$1" FOREIGN KEY (document_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 220 (OID 23012478)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY folders
    ADD CONSTRAINT "$1" FOREIGN KEY (folder_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 210 (OID 23012482)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY sessions
    ADD CONSTRAINT "$1" FOREIGN KEY (user_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 225 (OID 23012486)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY usernotes
    ADD CONSTRAINT "$1" FOREIGN KEY (note_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 226 (OID 23012490)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY disccategories
    ADD CONSTRAINT "$1" FOREIGN KEY (disccategory_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 227 (OID 23012494)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY discussions
    ADD CONSTRAINT "$1" FOREIGN KEY (discussion_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 228 (OID 23012498)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY resources
    ADD CONSTRAINT "$1" FOREIGN KEY (resource_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 229 (OID 23012502)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY event_resources
    ADD CONSTRAINT "$1" FOREIGN KEY (event_id) REFERENCES events(event_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 230 (OID 23012506)
-- Name: $2; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY event_resources
    ADD CONSTRAINT "$2" FOREIGN KEY (resource_id) REFERENCES resources(resource_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 231 (OID 23012510)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY item_rels
    ADD CONSTRAINT "$1" FOREIGN KEY (left_item_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 232 (OID 23012514)
-- Name: $2; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY item_rels
    ADD CONSTRAINT "$2" FOREIGN KEY (right_item_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 233 (OID 23012518)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY bookmarks
    ADD CONSTRAINT "$1" FOREIGN KEY (bookmark_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 234 (OID 23012522)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY workflows
    ADD CONSTRAINT "$1" FOREIGN KEY (workflow_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 235 (OID 23012526)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY email_filters
    ADD CONSTRAINT "$1" FOREIGN KEY (filter_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 236 (OID 23012530)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY clients
    ADD CONSTRAINT "$1" FOREIGN KEY (client_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 237 (OID 23012534)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY opportunities
    ADD CONSTRAINT "$1" FOREIGN KEY (opp_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 238 (OID 23012538)
-- Name: $2; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY opportunities
    ADD CONSTRAINT "$2" FOREIGN KEY (client_id) REFERENCES clients(client_id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- TOC entry 215 (OID 23012542)
-- Name: $2; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY user_groups
    ADD CONSTRAINT "$2" FOREIGN KEY (top_org_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 239 (OID 23012546)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY products
    ADD CONSTRAINT "$1" FOREIGN KEY (product_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 240 (OID 23012550)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY opp_products
    ADD CONSTRAINT "$1" FOREIGN KEY (opp_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 241 (OID 23012554)
-- Name: $2; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY opp_products
    ADD CONSTRAINT "$2" FOREIGN KEY (product_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 207 (OID 23012558)
-- Name: $3; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY access_rights
    ADD CONSTRAINT "$3" FOREIGN KEY (treepos) REFERENCES items(treepos) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- TOC entry 242 (OID 23012562)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY quotas
    ADD CONSTRAINT "$1" FOREIGN KEY (quota_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 244 (OID 23012566)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY virt_folders
    ADD CONSTRAINT "$1" FOREIGN KEY (virt_folder_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 243 (OID 23012570)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY contact_groups
    ADD CONSTRAINT "$1" FOREIGN KEY (group_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 245 (OID 23012574)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY messages
    ADD CONSTRAINT "$1" FOREIGN KEY (message_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 246 (OID 23012578)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY dus_jobs
    ADD CONSTRAINT "$1" FOREIGN KEY (job_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 251 (OID 23012582)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY dus_services
    ADD CONSTRAINT "$1" FOREIGN KEY (service_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 252 (OID 23012586)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY dus_translations
    ADD CONSTRAINT "$1" FOREIGN KEY (translation_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;


--
-- TOC entry 247 (OID 23012590)
-- Name: $2; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY dus_jobs
    ADD CONSTRAINT "$2" FOREIGN KEY (client_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- TOC entry 248 (OID 23012594)
-- Name: $3; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY dus_jobs
    ADD CONSTRAINT "$3" FOREIGN KEY (person_translation) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- TOC entry 249 (OID 23012598)
-- Name: $4; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY dus_jobs
    ADD CONSTRAINT "$4" FOREIGN KEY (person_editing) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- TOC entry 250 (OID 23012602)
-- Name: $5; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY dus_jobs
    ADD CONSTRAINT "$5" FOREIGN KEY (person_checking) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE RESTRICT;


--
-- TOC entry 223 (OID 23012606)
-- Name: $1; Type: FK CONSTRAINT; Schema: public; Owner: conflux
--

ALTER TABLE ONLY settings
    ADD CONSTRAINT "$1" FOREIGN KEY (treepos) REFERENCES items(treepos) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- TOC entry 253 (OID 23012610)
-- Name: items_treepos_trig; Type: TRIGGER; Schema: public; Owner: conflux
--

CREATE TRIGGER items_treepos_trig
    BEFORE INSERT OR UPDATE ON items
    FOR EACH ROW
    EXECUTE PROCEDURE items_treepos_trig();


--
-- TOC entry 6 (OID 22485425)
-- Name: item_id_seq; Type: SEQUENCE SET; Schema: public; Owner: conflux
--

SELECT pg_catalog.setval('item_id_seq', 100, true);


--
-- TOC entry 8 (OID 22485427)
-- Name: scrollback_id_seq; Type: SEQUENCE SET; Schema: public; Owner: conflux
--

SELECT pg_catalog.setval('scrollback_id_seq', 100, true);


--
-- TOC entry 10 (OID 22485448)
-- Name: item_type_id_seq; Type: SEQUENCE SET; Schema: public; Owner: conflux
--

SELECT pg_catalog.setval('item_type_id_seq', 28, true);


--
-- TOC entry 12 (OID 22485459)
-- Name: session_id_seq; Type: SEQUENCE SET; Schema: public; Owner: conflux
--

SELECT pg_catalog.setval('session_id_seq', 100, true);


--
-- TOC entry 14 (OID 22485490)
-- Name: user_groups_seq; Type: SEQUENCE SET; Schema: public; Owner: conflux
--

SELECT pg_catalog.setval('user_groups_seq', 100, true);


--
-- TOC entry 16 (OID 22485575)
-- Name: systemnote_seq; Type: SEQUENCE SET; Schema: public; Owner: conflux
--

SELECT pg_catalog.setval('systemnote_seq', 100, true);


--
-- TOC entry 18 (OID 22485624)
-- Name: calendar_groups_seq; Type: SEQUENCE SET; Schema: public; Owner: conflux
--

SELECT pg_catalog.setval('calendar_groups_seq', 100, true);


--
-- TOC entry 20 (OID 22485664)
-- Name: log_batch_seq; Type: SEQUENCE SET; Schema: public; Owner: conflux
--

SELECT pg_catalog.setval('log_batch_seq', 100, true);


--
-- TOC entry 22 (OID 22485690)
-- Name: im_seq; Type: SEQUENCE SET; Schema: public; Owner: conflux
--

SELECT pg_catalog.setval('im_seq', 100, true);


--
-- TOC entry 24 (OID 22485716)
-- Name: dus_job_seq; Type: SEQUENCE SET; Schema: public; Owner: conflux
--

SELECT pg_catalog.setval('dus_job_seq', 100, true);


SET SESSION AUTHORIZATION 'postgres';

--
-- TOC entry 3 (OID 2200)
-- Name: SCHEMA public; Type: COMMENT; Schema: -; Owner: postgres
--

COMMENT ON SCHEMA public IS 'Standard public schema';


