#!/usr/bin/python
#
# Copyright (C) 2003-2005 Inversion Software Ltd. <inversion@inversion.ee>
# http://www.conflux.ee/
#
# This file is part of Conflux Lite
#
# Conflux Lite is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2,
# or (at your option) any later version.
#
# Conflux Lite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Conflux Lite; if not, write to the Free Software
# Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
######################################################################

import sys, signal, os, time, tempfile, base64, email, cStringIO, mx.DateTime, cPickle

PID_FILE = "/tmp/confluxd.pid"
tempfile.tempdir = "/tmp/"
REQUEST = None

# Logs
def log_message(message, error=0):
    if error:
        sys.stdout.write("%s: ERROR: %s\n" % (time.strftime("%Y-%m-%d %H:%M:%S"), message))
    else:
        sys.stdout.write("%s: LOG: %s\n" % (time.strftime("%Y-%m-%d %H:%M:%S"), message))
    sys.stdout.flush()

# Initialize signals
def exit_fault(signum):
    global REQUEST
    if REQUEST is not None:
        REQUEST.close()
    os.unlink(PID_FILE)
    log_message('Conflux Daemon caught signal %d, exiting.' % signum, error=1)
    raise
    sys.exit()

# Set up pid file
def deamonize(stdout='/dev/null', stderr=None, stdin='/dev/null',
              pidfile=None, startmsg = 'started with pid %s' ):
    try:
        pid = os.fork()
        if pid > 0: sys.exit(0) # Exit first parent.
    except OSError, e: 
        sys.stderr.write("fork #1 failed: (%d) %s\n" % (e.errno, e.strerror))
        sys.exit(1)

    os.chdir("/")
    os.umask(0) 
    os.setsid() 

    try: 
        pid = os.fork()
        if pid > 0: sys.exit(0) # Exit second parent.
    except OSError, e:
        sys.stderr.write("fork #2 failed: (%d) %s\n" % (e.errno, e.strerror))
        sys.exit(1)

    if not stderr: stderr = stdout
    si = file(stdin, 'r')
    so = file(stdout, 'a+')
    se = file(stderr, 'a+', 0)
    pid = str(os.getpid())
    sys.stderr.write("\n%s\n" % startmsg % pid)
    sys.stderr.flush()
    if pidfile:
        tempname = tempfile.mktemp()
        fp = open(tempname,'w+b')
        fp.write(pid)
        fp.close()
        try:
            os.link(tempname, pidfile)
        except OSError: # pid-file already exists, do not start.
            log_message('Pidfile %s exists, exiting. If you are sure that\nthe daemon is currently not running, remove the file and start\nthe daemon again.' % PID_FILE, 1)
            os.unlink(tempname)
            sys.exit(1)
        os.unlink(tempname)

    os.dup2(si.fileno(), sys.stdin.fileno())
    os.dup2(so.fileno(), sys.stdout.fileno())
    os.dup2(se.fileno(), sys.stderr.fileno())
    log_message("Daemon started.", 0)


def startstop(stdout='/dev/null', stderr=None, stdin='/dev/null',
              pidfile=PID_FILE, startmsg = 'started with pid %s' ):
    if len(sys.argv) > 1:
        action = sys.argv[1]
        try:
            pf  = file(pidfile,'r')
            pid = int(pf.read().strip())
            pf.close()
        except IOError:
            pid = None
        if 'stop' == action or 'restart' == action:
            if not pid:
                mess = "Could not stop, pid file '%s' missing.\n"
                sys.stderr.write(mess % pidfile)
                sys.exit(1)
            try:
               while 1:
                   os.kill(pid,signal.SIGTERM)
                   time.sleep(1)
            except OSError, err:
               err = str(err)
               if err.find("No such process") > 0:
                   os.remove(pidfile)
                   if 'stop' == action:
                       sys.exit(0)
                   action = 'start'
                   pid = None
               else:
                   print str(err)
                   sys.exit(1)
        if 'start' == action:
            if pid:
                mess = "Start aborted since pid file '%s' exists.\n"
                sys.stderr.write(mess % pidfile)
                sys.exit(1)
            deamonize(stdout,stderr,stdin,pidfile,startmsg)
            return
    print "usage: %s start|stop|restart" % sys.argv[0]
    sys.exit(2)


def main():
    # Set up module import path  FIXME
    sys.path.append('/var/lib/conflux/lib/')

    # Real daemon code
    try:
        import _Conf, _Baselib, _RootObject, _InitTypes, _mail, _calendar, _Utils, _Efolder

        INDEXABLE_TYPES = [_InitTypes.ITEM_TYPE_DOCUMENT, _InitTypes.ITEM_TYPE_EMAIL]


        REQUEST = _Baselib.Request(None)
        fp = None

        TIMES = 0
        while 1: # Main cycle
            TIMES += 1
            # E-mail reminder
            if TIMES % 15 == 0:
                now = _Utils.utc_now_mx()
                data = _calendar.DayHandler(REQUEST)
                data.load(REQUEST, all_users=1)

                offset = None
                if REQUEST.tz is not None:
                    offset = mx.DateTime.DateTimeDelta(0, 0, _Conf.TIMEZONES_D[REQUEST.tz][4])

                for timerange in data.active_week[0].events:
                    if not timerange[2]:
                        continue
                    for event in timerange[2]:
                        if not event['remind_type'] or not event['remind_time']:
                            continue
                        remind_space = event['start_stamp'] - event['remind_time'].delta - mx.DateTime.DateTimeDelta(0,0,3)
                        if offset:
                            remind_space = remind_space - offset # Negative because we already have timezoned data
                        if now >= remind_space and (not event['last_reminder'] or event['last_reminder'].day_of_year <> now.day_of_year):
                            try:
                                _mail.send_reminder(event, REQUEST)
                                REQUEST.db.execute("""UPDATE events set last_reminder=now() WHERE event_id=%d""" % event['event_id'])
                                if event['remind_type'] == 1:
                                    log_message('Sent reminder to %s from event %d' % (event['e_mail'].decode('UTF8'), event['event_id']))
                                else:
                                    log_message('Sent reminder to %s from event %d' % (event['phone_cell'].decode('UTF8'), event['event_id']))
                            except:
                                log_message('Failed sending a reminder from event %d' % event['event_id'], 1)
                REQUEST.db.commit()

            # E-mail autodelete
            if TIMES % 15 == 0:
                REQUEST.db.execute("""SELECT i.item_id, b.value
FROM
 items i,
 persons p,
 settings a,
 settings b
WHERE
 i.item_id=p.person_id AND
 i.treepos=a.treepos AND
 i.treepos=b.treepos AND
 a.setting_key='email_deleteold' AND
 b.setting_key='email_deletedays' AND
 a.value='STEKLg==\n'""")
                users = REQUEST.db.fetchall()
                for row in users:
                    days = cPickle.loads(base64.decodestring(row[1]))
                    REQUEST.db.execute("""SELECT e.efolder_id FROM items i, efolders e WHERE i.item_id=e.efolder_id AND i.creator_id=%d AND (e.efolder_flags & %d) <> 0""" % (row[0], _Efolder.EFOLDER_DELETED))
                    folders = map(lambda x: x[0], REQUEST.db.fetchall())
                    for folder in folders:
                        REQUEST.db.execute("""SELECT e.email_id FROM items i, emails e WHERE e.email_id=i.item_id AND i.parent_id=%d AND i.create_time < now()-'%d days'::interval""" % (folder, days))
                        for email in map(lambda x: x[0], REQUEST.db.fetchall()):
                            try:
                                email_obj = _RootObject.dispatch_item(REQUEST, email)
                                email_obj.delete()
                                log_message('Autodeleted e-mail %d.' % email)
                                REQUEST.db.commit()
                            except:
                                log_message('Autodeleting e-mail %d failed.' % email)
                                REQUEST.db.rollback()

            if TIMES % 14400 == 0 and (time.localtime()[3] <= 3 or time.localtime()[3] >= 20):
                log_message('Running "vacuum analyze;".')
                REQUEST.db.execute("commit; VACUUM ANALYZE; begin;")

            # Take a nap
            REQUEST.db.commit()
            time.sleep(1)
    except:
        exit_fault(-1)

    # Clean up
    REQUEST.close()
    os.unlink(PID_FILE)

if __name__ == "__main__":
    startstop(stdout='/tmp/confluxd.log')
    main()
