# Copyright (C) 2003-2005 Inversion Software Ltd. <inversion@inversion.ee>
# http://www.conflux.ee/
#
# This file is part of Conflux Lite
#
# Conflux Lite is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2,
# or (at your option) any later version.
#
# Conflux Lite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Conflux Lite; if not, write to the Free Software
# Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
######################################################################

import _Conf, _Utils, datetime, mx.DateTime

def iso2userdate(date):
    return "%02d/%02d/%04d" % (date.month, date.day, date.year)

def iso2usertime(REQUEST, date, is_utc=1):
    if REQUEST.tz is not None and is_utc:
        date_utc = _Utils.mx2dt(date)
        offset = datetime.timedelta(minutes=_Conf.TIMEZONES_D[REQUEST.tz][4])
        date = date_utc + offset
    if date.hour >= 12:
        hour = date.hour - 12
        if hour == 0: hour = 12
        return "%02d:%02dPM" % (hour, date.minute)
    else:        
        hour = date.hour
        if hour == 0: hour = 12
        return "%02d:%02dAM" % (hour, date.minute)

def iso2userdatetime(REQUEST, date, is_utc=1):
    if REQUEST.tz is not None and is_utc:
        date_utc = _Utils.mx2dt(date)
        offset = datetime.timedelta(minutes=_Conf.TIMEZONES_D[REQUEST.tz][4])
        date = date_utc + offset
    if date.hour >= 12:
        hour = date.hour - 12
        if hour == 0: hour = 12
        return "%02d/%02d/%04d %02d:%02dPM" % (date.month, date.day, date.year, hour, date.minute)
    else:
        hour = date.hour
        if hour == 0: hour = 12
        return "%02d/%02d/%04d %02d:%02dAM" % (date.month, date.day, date.year, hour, date.minute)

def user2isodate(REQUEST, date):
    month, day, year = map(int, date.strip().split('/'))
    try:
        tmp = mx.DateTime.Date(year, month, day)
    except:
        raise RuntimeError, REQUEST.TLS('tls_invdateformat')
    return "%04d-%02d-%02d" % (year, month, day)

def user2isodatetime(REQUEST, date, is_utc=1):
    date, time = date.strip().split(' ')
    month, day, year = map(int, date.strip().split('/'))
    hour, minute = time.strip().split(':')
    hour = int(hour)
    if len(minute) == 4 and minute[2:].lower() == 'pm':
       if hour <> 12:
           hour = hour + 12
       minute = int(minute[:2])
    elif len(minute) == 4:
       if hour == 12:
           hour = 0
       minute = int(minute[:2])
    else:
       minute = int(minute)
    try:
        tmp = mx.DateTime.Date(year, month, day, hour, minute)
    except:
        raise RuntimeError, REQUEST.TLS('tls_invdateformat')
    date_user = datetime.datetime(year, month, day, hour, minute)
    if REQUEST.tz is not None and is_utc:
        offset = datetime.timedelta(minutes=_Conf.TIMEZONES_D[REQUEST.tz][4])
        date_user = date_user - offset
    return "%04d-%02d-%02d %02d:%02d" % (date_user.year, date_user.month, date_user.day, date_user.hour, date_user.minute)

def date2usertext(REQUEST, day, month):
    month_name = REQUEST.TLS('month_name_%02d' % month)
    return "%s %02d" % (month_name, day)

def datecheck_js():
    return u"""
function datecheck_js(datevar)
{
    // Check for size first
    if (datevar.length == 0)
        return true;
    if (datevar.length > 10)
        return '';

    // Check for separators
    var firstsep = datevar.indexOf("/");
    if (firstsep == -1 || firstsep == 0)
        return ''
    
    var secondsep = datevar.indexOf("/", firstsep+1);
    if (secondsep == -1 || firstsep+1 == secondsep)
        return ''

    // Check values
    var month = parseFloat(datevar.substring(0,firstsep));
    var day = parseFloat(datevar.substring(firstsep+1,secondsep));
    var year = parseFloat(datevar.substring(secondsep+1,secondsep+5));

    if (year < 1 || year > 9999)
        return '';

    if (month < 1 || month > 12)
        return '';

    if (day < 1 || day > 31)
        return '';

    if (day < 10 && month < 10)
        return '0'+month+'/0'+day+'/'+year;
    else if (day < 10)
        return month+'/0'+day+'/'+year;
    else if (month < 10)
        return '0'+month+'/'+day+'/'+year;
    else
        return month+'/'+day+'/'+year;
}
"""

def timecheck_js():
    return u"""
function timecheck_js(datevar)
{

    datevar = datevar.replace(' ', '');

    // Check for size first
    if (datevar.length == 0)
        return true;
    if (datevar.length > 7)
        return '';
    
    // Check for separators
    var firstsep = datevar.indexOf(":");
    if (firstsep == -1 || firstsep == 0)
        return '' 
        
    // Check values
    var hour = parseFloat(datevar.substring(0,firstsep));
    var minute = parseFloat(datevar.substring(firstsep+1,firstsep+3));
    var halfday = datevar.substring(firstsep+3,firstsep+5).toUpperCase();
    
    if (minute < 0 || minute > 59)
        return '';
    
    if (hour < 0 || hour > 12)
        return '';

    if (halfday.length != 2 || (halfday != 'AM' && halfday != 'PM'))
        return '';

    if (hour < 10 && minute < 10)
        return '0'+hour+':0'+minute+halfday;
    else if (minute < 10)
        return hour+':0'+minute+halfday;
    else if (hour < 10)
        return '0'+hour+':'+minute+halfday;
    else
        return hour+':'+minute+halfday;
}
"""
