# Copyright (C) 2003-2005 Inversion Software Ltd. <inversion@inversion.ee>
# http://www.conflux.ee/
#
# This file is part of Conflux Lite
#
# Conflux Lite is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2,
# or (at your option) any later version.
#
# Conflux Lite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Conflux Lite; if not, write to the Free Software
# Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
######################################################################

ACCESS_NONE   = 0
ACCESS_READ   = 1
ACCESS_WRITE  = 2
ACCESS_MANAGE = 3

import _Conf

###
# SQL
###

SELECT_ONE = u"""\
SELECT
 ar.treepos,
 max(CASE
  WHEN (i.is_public = 't' AND ar.right_id < %(ACCESS_READ)d::int2) THEN %(ACCESS_READ)d
  WHEN (i.is_private = 't' AND i.creator_id <> %(person_id)d) THEN %(ACCESS_NONE)d
  ELSE ar.right_id
 END) AS right_id
FROM
 access_rights ar 
JOIN
 access_groups ag ON ag.group_id=ar.group_id
JOIN
 items i ON i.treepos >= ar.treepos and i.treepos < (ar.treepos || B'1')
WHERE 
 i.item_id=%(item_id)d AND
 ag.person_id=%(person_id)d
GROUP BY ar.treepos
ORDER BY ar.treepos DESC
LIMIT 1"""

SELECT_ONE_GROUP = u"""\
SELECT
 CASE
  WHEN (i.is_public = 't' AND ar.right_id < %(ACCESS_READ)d::int2) THEN %(ACCESS_READ)d
  WHEN (i.is_private = 't') THEN %(ACCESS_NONE)d
  ELSE ar.right_id
 END AS right_id
FROM
 access_rights ar 
JOIN
 items i ON i.treepos >= ar.treepos and i.treepos < (ar.treepos || B'1')
WHERE 
 i.item_id=%(item_id)d AND
 ar.group_id=%(group_id)d
ORDER BY ar.treepos DESC
LIMIT 1"""

SELECT_ALL_QUERY = u"""\
(i.item_type_id IN ( %(item_type_ids)s ) AND
(i.is_private = 'f' OR (i.is_private = 't' AND i.creator_id=%(person_id)d)) AND
(
 (i.is_public = 't' OR 
  %(table_field)s IN 
(
   SELECT
    i.item_id
   FROM
    access_rights ar 
   JOIN
    access_groups ag ON ag.group_id=ar.group_id
   JOIN
    items i ON i.treepos >= ar.treepos and i.treepos < (ar.treepos || B'1')
   WHERE 
    i.item_type_id IN ( %(item_type_ids)s ) AND
    ag.person_id=%(person_id)d AND
    ar.right_id >= %(right_id)d::int2
))))"""

SELECT_ALL_GROUP_QUERY = u"""\
(i.item_type_id IN ( %(item_type_ids)s ) AND
(i.is_private = 'f') AND
(
 (i.is_public = 't' OR 
  %(table_field)s IN 
(
   SELECT
    i.item_id
   FROM
    access_rights ar 
   JOIN
    items i ON i.treepos >= ar.treepos and i.treepos < (ar.treepos || B'1')
   WHERE 
    i.item_type_id IN ( %(item_type_ids)s ) AND
    ar.group_id=%(group_id)d AND
    ar.right_id >= %(right_id)d::int2
))))"""

###
# Functions
###

def get_item_right(REQUEST, item_id, person_id, debug=0):
    """
    Returns access right by given person to given item
    """

    ACCESS_NONE   = 0
    ACCESS_READ   = 1
    ACCESS_WRITE  = 2
    ACCESS_MANAGE = 3

    if debug:
        return SELECT_ONE % locals()

    if person_id == _Conf.SUPERUSER:
        return ACCESS_MANAGE
    elif item_id == _Conf.ROOTOBJECT:
        return ACCESS_READ # for dav
    elif hasattr(REQUEST, 'disable_permissions'):
        return REQUEST.disable_permissions

    REQUEST.db.execute((SELECT_ONE % locals()).encode('UTF8'))
    try:
        return REQUEST.db.dictfetchall()[0]['right_id']
    except IndexError:
        return ACCESS_NONE # Nothing found, no right.

def get_item_right_group(REQUEST, item_id, group_id, debug=0):
    """
    Returns access right by given person to given item
    """

    ACCESS_NONE   = 0
    ACCESS_READ   = 1
    ACCESS_WRITE  = 2
    ACCESS_MANAGE = 3

    if debug:
        return SELECT_ONE_GROUP % locals()

    REQUEST.db.execute((SELECT_ONE_GROUP % locals()).encode('UTF8'))
    try:
        return REQUEST.db.dictfetchall()[0]['right_id']
    except IndexError:
        return ACCESS_NONE # Nothing found, no right.

def select_param(person_id, right_id, item_types, table_field="i.item_id"):
    """
    Returns query parameters for selects
    """

    ACCESS_NONE   = 0
    ACCESS_READ   = 1
    ACCESS_WRITE  = 2
    ACCESS_MANAGE = 3

    if person_id == _Conf.SUPERUSER:
        return u""

    item_type_ids = ",".join(map(str, item_types))
    return SELECT_ALL_QUERY % locals()

def select_param_group(group_id, right_id, item_types, table_field="i.item_id"):
    """
    Returns query parameters for selects
    """

    ACCESS_NONE   = 0
    ACCESS_READ   = 1
    ACCESS_WRITE  = 2
    ACCESS_MANAGE = 3

    item_type_ids = ",".join(map(str, item_types))
    return SELECT_ALL_GROUP_QUERY % locals()

