# Copyright (C) 2003-2005 Inversion Software Ltd. <inversion@inversion.ee>
# http://www.conflux.ee/
#
# This file is part of Conflux Lite
#
# Conflux Lite is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2,
# or (at your option) any later version.
#
# Conflux Lite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Conflux Lite; if not, write to the Free Software
# Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
######################################################################

import Cookie
import urllib
import base64
import time
import cCore
try:
    import cPickle
    pickle = cPickle
except ImportError:
    import pickle

import _Conf
import _DBSpool
import _Exceptions
import _Utils
import _Person, _InitTypes, _Organization
import translator

class Session:
    # Select one, no older than 15 minutes, user must exist.
    # Requires sql quoted strings sql_session_key and sql_session_host
    SQL_SELECT = """\
SELECT
 s.*,
 p.person_id,
 p.first_name,
 p.last_name,
 p.username,
 p.password,
 p.is_disabled,
 p.is_deleted
FROM
 sessions s,
 persons p,
 items i -- for checking if person is real
WHERE
-- Take out idle for now
-- ( s.update_stamp+'15 minutes'::interval ) > now() AND
 i.item_id=p.person_id     AND
 p.person_id=s.user_id     AND
 s.session_key=%(sql_session_key)s   AND
-- s.user_ip=%(sql_session_host)s
 p.username=%(sql_session_username)s AND
p.is_deleted=0
ORDER BY s.update_stamp DESC
LIMIT 1"""
    # Update one after reload.
    # Requires decimal session_id
    SQL_UPDATE = """UPDATE sessions SET update_count=update_count+1, update_stamp=now() WHERE session_id=%(session_id)d"""
    # Creates new session record
    # Requires sql quoted strings sql_session_key and sql_session_host
    #  and decimal person_id
    SQL_CREATE = """INSERT INTO sessions ( session_key, user_ip, user_id ) VALUES ( %(sql_session_key)s, %(sql_session_host)s, %(person_id)d )"""
    # Loads person data for a new session
    SQL_PERSON = """SELECT p.person_id, p.username, p.password, p.first_name, p.last_name, p.is_disabled, p.is_deleted FROM persons p, items i WHERE i.item_id=p.person_id AND p.is_deleted=0 AND username=%(sql_username)s"""

    def __init__(self, REQUEST, create=1, emulate_session=0):
        self.userdata = None
        if emulate_session: self._emulate(REQUEST)
        elif create: self._create(REQUEST)
        else: self._load(REQUEST)

    def __str__(self):
        if self.userdata and self.userdata.has_key("session_key"):
            return "Session: %s" % self.userdata["session_key"]
        else:
            return "Uninitialized session"

    def _create(self, REQUEST):
        sql_username = _Utils.sql_quote(REQUEST.form['RealmAuthUser'].value.decode('UTF8'))
        REQUEST.db.execute((Session.SQL_PERSON % locals()).encode('utf8'))
        data = REQUEST.db.dictfetchall()
        if not data:
            raise _Exceptions.PleaseAuthenticate, REQUEST.TLS('baselib_authenticate_wrongpass')

        data = data[0]

        if data['is_disabled']:
            raise _Exceptions.PleaseAuthenticate, REQUEST.TLS('baselib_authenticate_disabled')
        crypted_password = _Utils.crypt(REQUEST.form['RealmAuthPass'].value, data['password'][:2]) # Crypt does not support unicode strings, so no decode
        if crypted_password <> data['password']:
            raise _Exceptions.PleaseAuthenticate, REQUEST.TLS('baselib_authenticate_wrongpass')

        data['session_key'] = _Utils.new_session_key()

        sql_session_key = _Utils.sql_quote(data['session_key'])
        sql_session_host = _Utils.sql_quote(REQUEST.mod_req.connection.remote_ip)
        person_id = data['person_id']
        REQUEST.db.execute((Session.SQL_CREATE % locals()).encode('utf8'))
        REQUEST.db.execute((u"""SELECT session_id FROM sessions WHERE session_key=%s""" % sql_session_key).encode('UTF8'))
        data['session_id'] = REQUEST.db.fetchone()[0]
        if REQUEST.form.has_key('RealmAuthRemember'):
            REQUEST.setCookie("RealmAuthCookie", "%s:%s" % (data["session_key"], REQUEST.form['RealmAuthUser'].value), setForever=1)
        else:
            REQUEST.setCookie("RealmAuthCookie", "%s:%s" % (data["session_key"], REQUEST.form['RealmAuthUser'].value))

        self.userdata = data
        time.sleep(0.5)

    def _load(self, REQUEST):
        sql_session_key = _Utils.sql_quote(REQUEST.cookies['RealmAuthCookie'].value.split(':')[0])
        sql_session_host = _Utils.sql_quote(REQUEST.mod_req.connection.remote_ip)
        try:
            sql_session_username = _Utils.sql_quote(REQUEST.cookies['RealmAuthCookie'].value.split(':')[1])
        except IndexError: # Old-style auth cookie?
            raise _Exceptions.PleaseAuthenticate

        QUERY = Session.SQL_SELECT % locals()
        REQUEST.db.execute(QUERY.encode('UTF8'))
        data = REQUEST.db.dictfetchall()

        if not data:
            # Let's try once more just in case
            REQUEST.db.rollback()
            REQUEST.db.execute(QUERY.encode('UTF8'))
            data = REQUEST.db.dictfetchall()
            if not data:
                raise _Exceptions.PleaseAuthenticate # , REQUEST.TLS('baselib_authenticate_expired')

        if data[0]['is_disabled']:
            raise _Exceptions.PleaseAuthenticate, REQUEST.TLS('baselib_authenticate_disabled')
        self.userdata = data[0]

    def _emulate(self, REQUEST):
        self.userdata = {
'person_id':	_Conf.SUPERUSER,
'first_name':	'Shell',
'last_name':	'Superuser',
'username':	'superuser',
'user_ip':	'127.0.0.1',
'session_id':   -1 }

    def close(self, REQUEST):
        REQUEST.delCookie("RealmAuthCookie")

class Request:
    SQL_SU_NAME = """SELECT person_id, username, password, first_name, last_name FROM persons WHERE username=%(sql_username)s"""
    SQL_SU_ID = """SELECT person_id, username, password, first_name, last_name FROM persons WHERE person_id=%(sql_person_id)d"""

    def __init__(self, req):
        self._su_users = []

        self._translator = translator.Translator(REQUEST=self) # Dummy, only translates to default language.
                                                               # But otherwise we have problems translating
                                                               # session errors.
        try:
            self.db = _DBSpool.DBSpool.get_connection()
        except:
            reload(_DBSpool) # Try to reload to get a connection
            self.db = _DBSpool.DBSpool.get_connection()
        try:
            if req:
                self.from_apache = 1 # show that we come from apache, so we have the provided request etc
                self.mod_req = req
                self.form = self.mod_req.form
            else:
                self.from_apache = 0 # show that we come from shell, so we have to emulate certain stuff
                self.mod_req = None
                self.form = {}

            if self.form.has_key('qs') and self.form['qs'].value:
                self.qs = int(self.form['qs'].value)
            else:
                self.qs = 0

            # Inititate cookie
            if self.from_apache and self.mod_req.headers_in.has_key('Cookie'):
                self.cookies = Cookie.SimpleCookie()
                self.cookies.load(self.mod_req.headers_in['Cookie'])
            else:
                self.cookies = {}

            if not self.from_apache:
                self.emulate_session()
            elif self.cookies.has_key('RealmAuthCookie'):
                self.load_session()
            elif self.form.has_key('RealmAuthUser') and self.form.has_key('RealmAuthPass'):
                self.create_session()
            else:
                raise _Exceptions.PleaseAuthenticate # , "Please authenticate yourself:"
            self.USER = User(REQUEST=self)
            self._settings = SettingsManager()
            self._translator = translator.Translator(REQUEST=self)

            try:
                self.batch_size = self.get_setting('user_default_batch')
            except:
                self.batch_size = _Conf.DEFAULT_BATCH
            try:
                self.email_batch_size = self.get_setting('user_email_batch')
            except:
                self.email_batch_size = _Conf.DEFAULT_BATCH
            
            try:
                self.tz = int(self.get_setting('user_timezone').split(':')[0])
            except:
                self.tz = None

            if self.from_apache and self.mod_req.headers_in.has_key('User-Agent'): # FIXME
                if self.mod_req.headers_in['User-Agent'][:11] == 'Mozilla/4.0':
                    self.browser = 'IE'
                else:
                    self.browser = 'Mozilla'
            else:
                self.browser = 'IE'

            self.is_https = self.mod_req and self.mod_req.subprocess_env.has_key('HTTPS') and self.mod_req.subprocess_env['HTTPS'] == 'on'
            if self.from_apache and self.is_https:
                if self.mod_req.connection.local_addr[1] == 443:
                    self.SERVER_URL = "https://%s" % self.mod_req.server.server_hostname
                else:
                    self.SERVER_URL = "https://%s:%s" % (self.mod_req.server.server_hostname, self.mod_req.connection.local_addr[1])
            elif self.from_apache:
                if self.mod_req.connection.local_addr[1] == 80:
                    self.SERVER_URL = "http://%s" % self.mod_req.server.server_hostname
                else:
                    self.SERVER_URL = "http://%s:%s" % (self.mod_req.server.server_hostname, self.mod_req.connection.local_addr[1])
            else:
                self.SERVER_URL = "N/A"

        except:
            self.db.rollback()
            self.db.close()
            raise # re-raise

    def su_by_id(self, person_id):
        self.db.execute(self.SQL_SU_ID % {'sql_person_id':person_id})
        results = self.db.dictfetchone()
        if not results:
            raise _Exceptions.RequestError, self.TLS('request_sunouser')
        self._su_users.append((self.USER, self.session.userdata, self._translator))
        self.session.userdata = results
        if not self.from_apache:
            self.session.userdata['user_ip'] = '127.0.0.1'
        self.USER = User(REQUEST=self)
        self._translator = translator.Translator(REQUEST=self)

    def su_by_name(self, username):
        self.db.execute(self.SQL_SU_NAME % {'sql_username':_Utils.sql_cast_str(REQUEST=self, var=username)})
        results = self.db.dictfetchone()
        if not results:
            raise _Exceptions.RequestError, self.TLS('request_sunouser')
        self._su_users.append((self.USER, self.session.userdata, self._translator))
        self.session.userdata = results
        self.USER = User(REQUEST=self)
        self._translator = translator.Translator(REQUEST=self)

    def unsu(self):
        if not self._su_users:
            raise _Exceptions.RequestError, self.TLS('request_nounsu')
        self.USER, self.session.userdata, self._translator = self._su_users.pop()

        self.iso2userdate = self._translator._dateformat.iso2userdate
        self.iso2usertime = self._translator._dateformat.iso2usertime
        self.iso2userdatetime = self._translator._dateformat.iso2userdatetime
        
        self.user2isodate = self._translator._dateformat.user2isodate
        self.user2isodatetime = self._translator._dateformat.user2isodatetime
        
        self.datecheck_js = self._translator._dateformat.datecheck_js
        self.timecheck_js = self._translator._dateformat.timecheck_js


    def TLS(self, word):
        return self._translator.translate(word)

    def get_setting(self, key):
        return self._settings.get_setting(REQUEST=self, key=key)

    def del_setting(self, key):
        return self._settings.del_setting(REQUEST=self, key=key)

    def set_setting(self, key, value):
        self._settings.set_setting(REQUEST=self, key=key, value=value)

    def get_formstring(self, badkeys = []):
        output = []
        for key in self.form.keys():
            item = self.form[key]
            try:
                a = item.filename
                del a
                continue
            except:
                pass
            if key not in badkeys and key[:4] != "sel_":
                if isinstance(item, list):
                    for itemval in item:
                        output.append("%s=%s" % (key, urllib.quote(itemval))) # FIXME
                else:
                    output.append("%s=%s" % (key, urllib.quote(item.value))) # FIXME
        return "?" + "&".join(output) + ((output and "&") or "")

    def load_session(self):
        self.session = Session(REQUEST=self,create=0)

    def create_session(self):
        self.session = Session(REQUEST=self,create=1)

    def emulate_session(self):
        self.session = Session(REQUEST=self,emulate_session=1)

    def getCookie(self, key):
        return self.cookies[key].value

    def setCookie(self, key, value, setForever=0):
        if not self.from_apache:
            return

        if setForever:
            self.mod_req.headers_out.add("Set-Cookie", "%s=%s; Path=%s; Expires=Wed, 01-Jan-2020 00:00:00 GMT;" % (key, value, _Conf.BASE_URL)) # FIXME
        else:
            self.mod_req.headers_out.add("Set-Cookie", "%s=%s; Path=%s" % (key, value, _Conf.BASE_URL))

    def delCookie(self, key):
        if self.from_apache:
            self.mod_req.headers_out.add("Set-Cookie", "%s=; Path=%s; Expires=Mon, 01-Jan-1990 00:00:00 GMT;" % (key, _Conf.BASE_URL))

    def close(self):
        self.db.close()
        del self.session
        del self.cookies
        del self.form
        del self.mod_req
        del self.db
        del self.USER

class User:
    """Handly class for keeping user variables. Initialized after the session."""

    def __init__(self, REQUEST):
        self.__data__ = {}
        self.set('username', REQUEST.session.userdata['username'])
        self.set('person_id', REQUEST.session.userdata['person_id'])
        self.set('name', "%s %s" % (REQUEST.session.userdata['first_name'], REQUEST.session.userdata['last_name']))
        if REQUEST.from_apache:
            self.set('site', REQUEST.mod_req.connection.remote_ip)
        else:
            self.set('site', REQUEST.session.userdata['user_ip'])
        self._person = None
        self._parent = None

    def set(self, key, value):
        self.__data__[key] = value

    def get(self, key):
        return self.__data__[key] # FIXME, error handling

    def get_id(self):
        return self.__data__['person_id'] # FIXME, error handling

    def get_name(self):
        name = self.__data__['name']
        if isinstance(name, str):
            return name.decode('UTF8')
        return name

    def get_host(self):
        site = self.__data__['site']
        if isinstance(site, str):
            return site.decode('UTF8')
        return site

    def get_person(self, REQUEST):
        if self._person is None:
            self._person = _Person.Person(REQUEST, _InitTypes.INIT_TYPE_ID, self.get_id())
        return self._person

    def get_parent_id(self, REQUEST):
        return self.get_person(REQUEST).get_parent_id()

    def get_parent(self, REQUEST):
        if self._parent is None:
            self._parent = _Organization.Organization(REQUEST, _InitTypes.INIT_TYPE_ID, self.get_person(REQUEST).get_parent_id())
        return self._parent

class SettingsManager:
    """
Setting name length is limited to 64 characters!!!
"""

    SQL_GET_SETTING = u"""\
SELECT
 treepos,
 value
FROM
 settings
WHERE
 setting_key=%(key)s AND
(treepos is null OR
 treepos = %(top_treepos)s OR
 treepos = %(user_treepos)s )
ORDER BY treepos DESC"""
    SQL_GET_SETTINGS_USER = u"""\
SELECT
 setting_key,
 value
FROM
 settings
WHERE
 treepos = %(user_treepos)s"""
    SQL_GET_SETTINGS_ORG = u"""\
SELECT
 setting_key,
 value
FROM
 settings
WHERE
 treepos = %(top_treepos)s"""
    SQL_GET_SETTINGS_TOP = u"""\
SELECT
 setting_key,
 value,
 is_configurable
FROM
 settings
WHERE
 treepos is null"""


    SQL_SET_SETTING_ORG_DEL = u"""\
DELETE FROM
 settings
WHERE
 setting_key=%(key)s AND
 treepos = %(top_treepos)s"""
    SQL_SET_SETTING_ORG_INS = u"""\
INSERT INTO
 settings
(setting_key, value, treepos)
VALUES
(%(key)s, %(value)s, %(top_treepos)s)"""
    SQL_SET_SETTING_ORG_SEL = u"""\
SELECT count(*) FROM
 settings
WHERE setting_key=%(key)s and treepos=%(top_treepos)s"""
    SQL_SET_SETTING_ORG_UPD = u"""\
UPDATE
 settings
SET
value=%(value)s
WHERE
setting_key=%(key)s AND treepos=%(top_treepos)s
"""


    SQL_SET_SETTING_TOP_DEL = u"""\
DELETE FROM
 settings
WHERE
 setting_key=%(key)s AND
 treepos is null"""
    SQL_SET_SETTING_TOP_SEL = u"""\
SELECT count(*) FROM
 settings
WHERE
 setting_key=%(key)s AND
 treepos is null;"""
    SQL_SET_SETTING_TOP_INS = u"""\
INSERT INTO
 settings
(setting_key, value, treepos, is_configurable)
VALUES
(%(key)s, %(value)s, null, %(is_configurable)d)"""
    SQL_SET_SETTING_TOP_UPD = u"""\
UPDATE
 settings
SET
value=%(value)s,
is_configurable=%(is_configurable)d
WHERE setting_key=%(key) and treepos is null"""


    SQL_SET_SETTING_USER_DEL = u"""\
DELETE FROM
 settings
WHERE
 setting_key=%(key)s AND
 treepos = %(user_treepos)s"""
    SQL_SET_SETTING_USER_SEL = u"""\
SELECT count(*) FROM
 settings
WHERE
 setting_key=%(key)s AND
 treepos = %(user_treepos)s"""
    SQL_SET_SETTING_USER_INS = u"""\
INSERT INTO
 settings
(setting_key, value, treepos)
VALUES
(%(key)s, %(value)s, %(user_treepos)s)"""
    SQL_SET_SETTING_USER_UPD = u"""\
UPDATE
 settings
SET
value=%(value)s
WHERE
setting_key=%(key)s AND treepos=%(user_treepos)s"""


    SQL_DEL_SETTING_USER = u"""\
DELETE FROM
 settings
WHERE
 setting_key=%(key)s AND
 treepos = %(user_treepos)s"""

    def __init__(self):
        pass

    def get_setting(self, REQUEST, key):
        key = _Utils.sql_cast_str(REQUEST, key)
        user_treepos = _Utils.sql_cast_bit(REQUEST, REQUEST.USER.get_person(REQUEST).get('treepos'))
        top_treepos = _Utils.sql_cast_bit(REQUEST, REQUEST.USER.get_person(REQUEST).get_top_parent().get('treepos'))
        REQUEST.db.execute((self.SQL_GET_SETTING % locals()).encode('UTF8'))
        result = REQUEST.db.dictfetchall()
        if len(result) > 1 and result[0]['treepos'] is None: # Fix null sorting
            ndx = 1
        else:
            ndx = 0
        return pickle.loads(base64.decodestring(result[ndx]['value']))

    def set_setting(self, REQUEST, key, value):
        key = _Utils.sql_cast_str(REQUEST, key)
        value = _Utils.sql_cast_str(REQUEST, base64.encodestring(pickle.dumps(value)))
        user_treepos = _Utils.sql_cast_bit(REQUEST, REQUEST.USER.get_person(REQUEST).get('treepos'))
        REQUEST.db.execute((self.SQL_SET_SETTING_USER_DEL % locals()).encode('UTF8'))
        REQUEST.db.execute((self.SQL_SET_SETTING_USER_SEL % locals()).encode('UTF8'))
        if REQUEST.db.fetchone()[0]:
            REQUEST.db.execute((self.SQL_SET_SETTING_USER_UPD % locals()).encode('UTF8'))
        else:
            REQUEST.db.execute((self.SQL_SET_SETTING_USER_INS % locals()).encode('UTF8'))

    def del_setting(self, REQUEST, key):
        key = _Utils.sql_cast_str(REQUEST, key)
        user_treepos = _Utils.sql_cast_bit(REQUEST, REQUEST.USER.get_person(REQUEST).get('treepos'))
        REQUEST.db.execute((self.SQL_DEL_SETTING_USER % locals()).encode('UTF8'))

    def set_setting_org(self, REQUEST, key, value):
        key = _Utils.sql_cast_str(REQUEST, key)
        value = _Utils.sql_cast_str(REQUEST, base64.encodestring(pickle.dumps(value)))
        top_treepos = _Utils.sql_cast_bit(REQUEST, REQUEST.USER.get_person(REQUEST).get_top_parent().get('treepos'))
        REQUEST.db.execute((self.SQL_SET_SETTING_ORG_DEL % locals()).encode('UTF8'))
        REQUEST.db.execute((self.SQL_SET_SETTING_ORG_SEL % locals()).encode('UTF8'))
        if REQUEST.db.fetchone()[0]:
            REQUEST.db.execute((self.SQL_SET_SETTING_ORG_UPD % locals()).encode('UTF8'))
        else:
            REQUEST.db.execute((self.SQL_SET_SETTING_ORG_INS % locals()).encode('UTF8'))

    def set_setting_org_any(self, REQUEST, org_id, key, value):
        if REQUEST.USER.get_id() <> _Conf.SUPERUSER:
            return
        org = _Organization.Organization(REQUEST, _InitTypes.INIT_TYPE_ID, org_id)
        if org.get_parent_id() <> _Conf.ROOTOBJECT:
            return

        key = _Utils.sql_cast_str(REQUEST, key)
        value = _Utils.sql_cast_str(REQUEST, base64.encodestring(pickle.dumps(value)))
        top_treepos = _Utils.sql_cast_bit(REQUEST, org.get('treepos'))
        REQUEST.db.execute((self.SQL_SET_SETTING_ORG_DEL % locals()).encode('UTF8'))
        REQUEST.db.execute((self.SQL_SET_SETTING_ORG_SEL % locals()).encode('UTF8'))
        if REQUEST.db.fetchone()[0]:
            REQUEST.db.execute((self.SQL_SET_SETTING_ORG_UPD % locals()).encode('UTF8'))
        else:
            REQUEST.db.execute((self.SQL_SET_SETTING_ORG_INS % locals()).encode('UTF8'))

    def set_setting_top(self, REQUEST, key, value):
        is_configurable = 0
        try:
            values = self.get_settings_top(REQUEST)
            for value2 in values:
                if value2['setting_key'] == key and value2['is_configurable']:
                    is_configurable = 1
                    break
        except:
            pass
        key = _Utils.sql_cast_str(REQUEST, key)
        value = _Utils.sql_cast_str(REQUEST, base64.encodestring(pickle.dumps(value)))
        REQUEST.db.execute((self.SQL_SET_SETTING_TOP_DEL % locals()).encode('UTF8'))
        REQUEST.db.execute((self.SQL_SET_SETTING_TOP_SEL % locals()).encode('UTF8'))
        if REQUEST.db.fetchone()[0]:
            REQUEST.db.execute((self.SQL_SET_SETTING_TOP_UPD % locals()).encode('UTF8'))
        else:
            REQUEST.db.execute((self.SQL_SET_SETTING_TOP_INS % locals()).encode('UTF8'))

    def get_settings(self, REQUEST):
        user_treepos = _Utils.sql_cast_bit(REQUEST, REQUEST.USER.get_person(REQUEST).get('treepos'))
        REQUEST.db.execute((self.SQL_GET_SETTINGS_USER % locals()).encode('UTF8'))
        results = REQUEST.db.dictfetchall()
        for r in results:
            r['value'] = pickle.loads(base64.decodestring(r['value']))
        return results

    def get_settings_org(self, REQUEST):
        top_treepos = _Utils.sql_cast_bit(REQUEST, REQUEST.USER.get_person(REQUEST).get_top_parent().get('treepos'))
        REQUEST.db.execute((self.SQL_GET_SETTINGS_ORG % locals()).encode('UTF8'))
        results = REQUEST.db.dictfetchall()
        for r in results:
            r['value'] = pickle.loads(base64.decodestring(r['value']))
        return results

    def get_settings_top(self, REQUEST):
        REQUEST.db.execute(self.SQL_GET_SETTINGS_TOP.encode('UTF8'))
        results = REQUEST.db.dictfetchall()
        for r in results:
            r['value'] = pickle.loads(base64.decodestring(r['value']))
        return results

    def get_setting_org_any(self, REQUEST, org_id, key):
        if REQUEST.USER.get_id() <> _Conf.SUPERUSER:
            return
        org = _Organization.Organization(REQUEST, _InitTypes.INIT_TYPE_ID, org_id)
        if org.get_parent_id() <> _Conf.ROOTOBJECT:
            return

        key = _Utils.sql_cast_str(REQUEST, key)
        user_treepos = _Utils.sql_cast_bit(REQUEST, org.get('treepos'))
        top_treepos = _Utils.sql_cast_bit(REQUEST, org.get('treepos'))
        REQUEST.db.execute((self.SQL_GET_SETTING % locals()).encode('UTF8'))
        result = REQUEST.db.dictfetchall()
        if len(result) > 1 and result[0]['treepos'] is None: # Fix null sorting
            ndx = 1
        else:
            ndx = 0
        return pickle.loads(base64.decodestring(result[ndx]['value']))
