# Copyright (C) 2003-2005 Inversion Software Ltd. <inversion@inversion.ee>
# http://www.conflux.ee/
#
# This file is part of Conflux Lite
#
# Conflux Lite is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2,
# or (at your option) any later version.
#
# Conflux Lite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Conflux Lite; if not, write to the Free Software
# Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
######################################################################

import psycopg, mx.DateTime, time
import _Conf


from _DBSpool_utils import TimeParser

TIMEX = psycopg.new_type((704,1186), "INTERVAL", TimeParser)
psycopg.register_type(TIMEX)

class ConnectionSpool:
    """
        
    THIS MUST BE PUT IN SEPARATE MODULE FOR THE SPOOLING TO WORK.

    """
    def __init__(self):
        if not (_Conf.IS_MP_31 and _Conf.PLATFORM_WIN):
            if _Conf.DB_HOST:
                self.con_str = "host=%s port=%s dbname=%s user=%s password=%s" % (_Conf.DB_HOST, _Conf.DB_PORT, _Conf.DB_NAME, _Conf.DB_USER, _Conf.DB_PASS)
            else:
                self.con_str = "dbname=%s port=%s user=%s password=%s" % (_Conf.DB_NAME, _Conf.DB_PORT, _Conf.DB_USER, _Conf.DB_PASS)
            try:
                self.__connection__ = psycopg.connect(self.con_str)
            except psycopg.OperationalError:
                raise RuntimeError, "Unable to connect to the database." # FIXME
            
    def get_connection(self):
        if _Conf.IS_MP_31 and _Conf.PLATFORM_WIN:
            if _Conf.DB_HOST:
                self.con_str = "host=%s port=%s dbname=%s user=%s password=%s" % (_Conf.DB_HOST, _Conf.DB_PORT, _Conf.DB_NAME, _Conf.DB_USER, _Conf.DB_PASS)
            else:
                self.con_str = "dbname=%s port=%s user=%s password=%s" % (_Conf.DB_NAME, _Conf.DB_PORT, _Conf.DB_USER, _Conf.DB_PASS)
            try:
                con = psycopg.connect(self.con_str)
            except psycopg.OperationalError:
                raise RuntimeError, "Unable to connect to the database." # FIXME
            return CursorWrapper(con, newcon = 1)
	else:
            return CursorWrapper(self.__connection__)

class CursorWrapper:
    def __init__(self, con, newcon = 0):
        self.__history__ = []
        self.__con__ = con
        self.__cur__ = self.__con__.cursor()
	self.__newcon__ = newcon

        self.__execute      = self.__cur__.execute
        self.dictfetchall = self.__cur__.dictfetchall
        self.dictfetchone = self.__cur__.dictfetchone
        self.fetchall = self.__cur__.fetchall
        self.fetchone = self.__cur__.fetchone

        self.__commit       = self.__con__.commit
        self.__rollback     = self.__con__.rollback

        # Set timezone to UTC
        self.__cur__.execute("""SET TIME ZONE UTC""")
        self.__cur__.execute("""SELECT version()""")
        if self.__cur__.fetchone()[0][:14].lower() == 'postgresql 7.3':
            self.version = 730
        else:
            self.version = 740

    def close(self):
        if self.__newcon__:
             self.__cur__.close()
             self.__con__.close()
	else:
             self.__cur__.close()

    def execute(self, query):
        self.__history__.append(query)
        try:
            self.__execute(query)
        except psycopg.ProgrammingError, msg: # See if we have a serialize error, older psycopg
            if str(msg).find('serialize access') <> -1:
                tries = 1
                while 1:
                    self.__rollback()     # Real rollback, we don't want to clear the query cache
                    time.sleep(0.25)       # Sleep for half a second to get the other transaction out of the way
                    try:
                        for q in self.__history__:
                            self.__execute(q) # Real execute, we don't want to mess up the query cache
                        break
                    except:
                        pass # hack? FIXME
                    if tries > 40:
                        raise RuntimeError, msg
                    tries = tries + 1
            else:
                raise RuntimeError, msg # If not, just reraise and let whatever called us deal with it
        except psycopg.IntegrityError, msg: # See if we have a serialize error, newer psycopg
            if str(msg).find('serialize access') <> -1:
                tries = 1
                while 1:
                    self.__rollback()     # Real rollback, we don't want to clear the query cache
                    time.sleep(0.25)       # Sleep for half a second to get the other transaction out of the way
                    try:
                        for q in self.__history__:
                            self.__execute(q) # Real execute, we don't want to mess up the query cache
                        break
                    except:
                        pass # hack? FIXME
                    if tries > 40:
                        raise RuntimeError, msg
                    tries = tries + 1
            else:
                raise RuntimeError, msg # If not, just reraise and let whatever called us deal with it

    def commit(self):
        self.__commit()
        del self.__history__
        self.__history__ = []

    def rollback(self):
        self.__rollback()
        del self.__history__
        self.__history__ = []

DBSpool = ConnectionSpool()
