# Copyright (C) 2003-2005 Inversion Software Ltd. <inversion@inversion.ee>
# http://www.conflux.ee/
#
# This file is part of Conflux Lite
#
# Conflux Lite is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2,
# or (at your option) any later version.
#
# Conflux Lite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Conflux Lite; if not, write to the Free Software
# Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
######################################################################

import _RootObject, _Utils
import _InitTypes
import SQLTree
import _Conf

class Discussion(_RootObject.RootObject):
    table_name = 'discussions'
    field_name = 'discussion_id'
    item_type_id = _InitTypes.ITEM_TYPE_DISCUSSION
    is_container = 1
    object_fields = {
'discussion_id':	{ 'type': _Utils.sql_cast_int,	'simple_type': int,	'null': 0,	'can_set': 0,	'has_default': 1,       'can_insert': 0 },
'disc_subject':{ 'type': _Utils.sql_cast_str,		'simple_type': unicode,	'null': 0,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'disc_message':	{ 'type': _Utils.sql_cast_str_hidenull,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'is_front_page':	{ 'type': _Utils.sql_cast_bool,	'simple_type': int,	'null': 0,	'can_set': 1,	'has_default': 1,       'can_insert': 1 },
    }

    fti_keys = ['disc_subject','disc_message']

    def get_name(self):
        if self.get('disc_subject') is not None:
            return self.get('disc_subject')
        elif self.get_id() is None:
            return self.REQUEST.TLS('discobj_notsaved')
        else:
            return self.REQUEST.TLS('discobj_noname') % self.get_id()


def disctree(REQUEST):
    output = []
    a = SQLTree.SQLTree(REQUEST, "disctree1")
    a.set_param('item_types', [_InitTypes.ITEM_TYPE_ORGANIZATION, _InitTypes.ITEM_TYPE_DISCCATEGORY])
    a.set_param('linked_types', [_InitTypes.ITEM_TYPE_DISCCATEGORY])
    a.set_param('tree_url', _Conf.BASE_DISCUSSION_URL + 'disctree.py')
    a.set_param('target_url', _Conf.BASE_DISCUSSION_URL + 'discindex.py')
    a.set_param('target', 'topcontents')
    a.set_param('urlparams', 'parent_id=%(item_id)d')
    a.set_param('is_open_tree',1)
    a.set_param('disc_filter_childless',1)
    a.forbidden = ['discussion_id']
    a.maketree()
    output.append(a.getTree())
    return "\n".join(output)

def tree_bar(REQUEST):
    tree_output = disctree(REQUEST)
    BASE_ICON_URL = _Conf.BASE_ICON_URL
    BASE_DISCUSSION_URL = _Conf.BASE_DISCUSSION_URL
    tls_subjects = REQUEST.TLS('tls_subjects')
    bar = ""
    try:
        parent_id = int(REQUEST.get_setting('activediscFolder'))
    except:
        parent_id = 0
    if parent_id:
        REQUEST.db.execute("""SELECT disccategory_name FROM disccategories WHERE disccategory_id=%d""" % parent_id)
        cat_name = REQUEST.db.fetchone()
        if cat_name:
            cat_name = _Utils.html_quote(cat_name[0].decode('UTF8'))
            bar = bar + """ &raquo; %(cat_name)s """ % locals()
        else:
            del cat_name
            parent_id = 0
    if REQUEST.form.has_key('discussion_id') and REQUEST.form['discussion_id'].value:
        discussion_id = int(REQUEST.form['discussion_id'].value)
        REQUEST.db.execute("""SELECT item_name FROM items WHERE item_id=%d""" % discussion_id)
        disc_name = _Utils.html_quote(REQUEST.db.fetchone()[0].decode('UTF8'))
        bar = bar + """ &raquo; %(disc_name)s """ % locals()
    return u"""\
<script type="text/javascript">
function show_folders()
{
 var fld = document.getElementById('folders')
 if(fld.style.visibility=='visible')
    fld.style.visibility='hidden';
 else
    fld.style.visibility='visible';
}
</script>
<div style="cursor: pointer; cursor: hand;position:absolute; width:550px; height: 16px ;top:35px; left:23px; background: white; border: 1px solid #7BA5BD; z-index: 2; padding: 3px;" onclick="show_folders();">
<table border=0 width="100%%" cellpadding=0 cellspacing=0>
  <tr>
    <td><img src="%(BASE_ICON_URL)sspacer.gif" width=8 height=1 border=0 alt=""></td>
    <td width="100%%" align=left class="email_folderb">%(tls_subjects)s %(bar)s</td>
    <td nowrap><img src="%(BASE_ICON_URL)sspacer.gif" width=8 height=1 border=0 alt=""></td>
  </tr>
</table>
</div>
<div id="folders" style="position:absolute; width:550px; top:57px; left:23px; background: white; border-left: 1px solid #7BA5BD; border-right: 1px solid #AAAAAA; border-bottom: 1px solid #AAAAAA; padding: 3px; visibility: hidden; z-index: 3">
%(tree_output)s
<BR>
</div>
<BR>
<BR>
""" % locals()
