# Copyright (C) 2003-2005 Inversion Software Ltd. <inversion@inversion.ee>
# http://www.conflux.ee/
#
# This file is part of Conflux Lite
#
# Conflux Lite is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2,
# or (at your option) any later version.
#
# Conflux Lite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Conflux Lite; if not, write to the Free Software
# Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
######################################################################

import os, stat
import mx.DateTime

import _Utils, _Exceptions
import _InitTypes, _Conf, _Access
import _RootObject

BLOCK_SIZE = 65536

class Document(_RootObject.RootObject):
    table_name = 'documents'
    field_name = 'document_id'
    item_type_id = _InitTypes.ITEM_TYPE_DOCUMENT
    is_container = 0
    object_fields = {
'document_id':	{ 'type': _Utils.sql_cast_int,	'simple_type': int,	'null': 0,	'can_set': 0,	 'has_default': 1,       'can_insert': 0 },
'filename':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 0,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'title':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 0,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'description':	{ 'type': _Utils.sql_cast_str_hidenull,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'comments':	{ 'type': _Utils.sql_cast_str_hidenull,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'author':	{ 'type': _Utils.sql_cast_str_hidenull,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'mime_major':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 0,	'can_set': 1,	'has_default': 1,       'can_insert': 1 },
'mime_minor':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 0,	'can_set': 1,	'has_default': 1,       'can_insert': 1 },
'final_version_id':	{ 'type': _Utils.sql_cast_int,	'simple_type': int,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'doc_category':	{ 'type': _Utils.sql_cast_str_hidenull,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'locker_id':	{ 'type': _Utils.sql_cast_int,	'simple_type': int,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'lock_due':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,	'can_insert': 1	},
    }

    fti_keys = ['filename','title','description','comments','author', 'doc_category']

    def get_name(self):
        if self.get('title') is not None:
            return self.get('title')
        elif self.get_id() is None:
            return self.REQUEST.TLS('docobj_notsaved')
        else:
            return self.REQUEST.TLS('docobj_noname') % self.get_id()

    def _get_path(self):
        if self.get_id() == _Conf.ROOTOBJECT:
            return _Conf.BASE_DOCUMENT_PATH
        elif self.get_parent_id() == _Conf.ROOTOBJECT:
            return os.path.join(_Conf.BASE_DOCUMENT_PATH, _Utils.make_path(self.get('filename')))
        else:
            return os.path.join(self.get_parent()._get_path(), _Utils.make_path(self.get('filename')))

    def get_size(self):
        return self.get('item_size') or 0

    def AfterInsertHook(self):
        if self.REQUEST.form.has_key('filepointer'):
            filepointer = self.REQUEST.form['filepointer'].file
        else:
            filepointer = self.get('filepointer')
        if not filepointer:
            raise _Exceptions.DocumentError, self.REQUEST.TLS('docobj_nodocsave')

        if not self.get('filename'):
            raise _Exceptions.DocumentError, self.REQUEST.TLS('docobj_nodocsave')

        path = self.get_path()

        self.REQUEST.db.execute("""UPDATE items SET index_this_item=1 WHERE item_id=%d""" % self.get_id())

        fp = open(path.encode('UTF8'),'wb')

        fpwrite = fp.write
        fpread = filepointer.read

        data = fpread(BLOCK_SIZE)
        while data:
            fpwrite(data)
            data = fpread(BLOCK_SIZE)
        fp.close()

        self.__itemdata__['item_size'] = os.stat(path.encode('UTF8'))[stat.ST_SIZE]
        self.REQUEST.db.execute("UPDATE items SET item_size=%d WHERE item_id=%d" % (self.get_size(), self.get_id()))

    def AfterUpdateHook(self):
        if self.get('filepointer'):
            filepointer = self.get('filepointer')
            path = self.get_path()

            fp = open(path.encode('UTF8'),'wb')

            fpwrite = fp.write
            fpread = filepointer.read

            data = fpread(BLOCK_SIZE)
            while data:
                fpwrite(data)
                data = fpread(BLOCK_SIZE)
            fp.close()

            self.__itemdata__['item_size'] = os.stat(path.encode('UTF8'))[stat.ST_SIZE]
            self.REQUEST.db.execute("UPDATE items SET item_size=%d WHERE item_id=%d" % (self.get_size(), self.get_id()))

        self.rename_hook()

    def delete_hook(self):
        SQL = u"""SELECT d.document_id FROM items i, documents d WHERE d.document_id=i.item_id AND d.final_version_id=%d""" % self.get_id()
        self.REQUEST.db.execute(SQL.encode('UTF8'))
        versions = self.REQUEST.db.dictfetchall()
        for version in versions:
            ver = Document(self.REQUEST, _InitTypes.INIT_TYPE_ID, version['document_id'])
            ver.delete()

        oldpath = self.get_path()
        filename = "%d_%s" % (self.get_id(), os.path.split(oldpath)[1])
        newpath = os.path.join(self.make_backup_dir(), filename)
        if os.path.exists(oldpath.encode('UTF8')):
            os.rename(oldpath.encode('UTF8'), newpath.encode('UTF8'))
        else:
            fp = open(newpath.encode('UTF8'), 'wb') # FIXME
            fp.write("ERROR, MISSING FILE.\n")
            fp.close()
        self.set('oldpath', oldpath)

    def undelete_hook(self):
        delete_time = self.get('delete_time')
        newpath = self.get('oldpath')
        if delete_time is None or newpath is None:
            raise _Exceptions.DocumentError, self.REQUEST.TLS('docobj_undeletemissing')
        opj = os.path.join
        oldpath = opj(opj(opj(opj(_Conf.BACKUP_DOCUMENT_PATH, str(delete_time.year)), str(delete_time.month)), str(delete_time.day)), "%d_%s" % (self.get_id(), self.get('filename')))
        os.rename(oldpath.encode('UTF8'), newpath.encode('UTF8'))

    def rename_hook(self):
        if not self.__olddata__:
            self.REQUEST.db.execute("SELECT i.parent_id, d.filename FROM items i, documents d WHERE i.item_id=d.document_id AND item_id = %d" % self.get_id())
            r = self.REQUEST.db.dictfetchall()[0]
            old_filename = r['filename'].decode('UTF8')
            old_parent_id = r['parent_id']
        else:
            old_parent_id = self.__olddata__['parent_id']
            old_filename = self.__olddata__['filename'].decode('UTF8')
        if self.get_parent_id() == old_parent_id and self.get('filename') == old_filename:
            return

        oldpath = os.path.join(_RootObject.dispatch_item(self.REQUEST, old_parent_id).get_path(), _Utils.make_path(old_filename))
        os.rename(oldpath.encode('UTF8'), self.get_path().encode('UTF8'))

    def full_copyto(self, new_parent_id): # Overriding _RootObject method as we need to set a variable before save()
        old_path = self.get_path()
        new_obj = self.__class__(self.REQUEST, _InitTypes.INIT_TYPE_NEW)
        for (key, values) in self.item_fields.items():
            if values['can_set']:
                new_obj.set(key, self.get(key))
        for (key, values) in self.object_fields.items():
            if values['can_set']:
                new_obj.set(key, self.get(key))
        old_fp = open(old_path.encode('UTF8'), 'rb')
        new_obj.set('parent_id', new_parent_id)
        new_obj.set('filepointer', old_fp)
        if self.__itemdata__.has_key('copyto_replace_name'):
            new_obj.set('filename', self.get('copyto_replace_name'))
        newpath = new_obj.get_path()
        if os.path.exists(newpath.encode('UTF8')):
            new_obj.set('filename', _Utils.make_path("copy_%s" % os.path.split(newpath)[-1]))
        new_obj.save()
        if not old_fp.closed:
            old_fp.close()

        return new_obj

    def BeforeUpdateHook(self):
        pass

    def BeforeDeleteHook(self):
        pass
