# Copyright (C) 2003-2005 Inversion Software Ltd. <inversion@inversion.ee>
# http://www.conflux.ee/
#
# This file is part of Conflux Lite
#
# Conflux Lite is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2,
# or (at your option) any later version.
#
# Conflux Lite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Conflux Lite; if not, write to the Free Software
# Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
######################################################################

import _RootObject, _Utils
import _InitTypes

# _flags bits
EFOLDER_NONE = 0
EFOLDER_INBOX = 1
EFOLDER_SYSTEM = 2
EFOLDER_DELETED = 4
EFOLDER_DRAFTS = 8
EFOLDER_SENT = 16

class Efolder(_RootObject.RootObject):
    table_name = 'efolders'
    field_name = 'efolder_id'
    item_type_id = _InitTypes.ITEM_TYPE_EFOLDER
    is_container = 1
    object_fields = {
'efolder_id':	{ 'type': _Utils.sql_cast_int,	'simple_type': int,	'null': 0,	'can_set': 0,	'has_default': 1,       'can_insert': 0 },
'efolder_name':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 0,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'efolder_flags':{ 'type': _Utils.sql_cast_int,	'simple_type': int,	'null': 0,	'can_set': 1,	'has_default': 1,       'can_insert': 1 },
    }

    fti_keys = ['efolder_name']

    def get_name(self):
        if self.get('efolder_name') is not None:
            return self.get('efolder_name')
        elif self.get_id() is None:
            return self.REQUEST.TLS('efolderobj_notsaved')
        else:
            return self.REQUEST.TLS('efolder_noname') % self.get_id()

    def delete_hook(self): # delete all related efilters
        self.REQUEST.db.execute("""SELECT i.item_id FROM items i, email_filters f WHERE i.item_id=f.filter_id AND i.item_type_id=%d AND f.folder_id=%d""" %(_InitTypes.ITEM_TYPE_EFILTER, self.get_id()))
        res = self.REQUEST.db.dictfetchall()
        for efilter in res:
            obj = _Efilter.Efilter(self.REQUEST, _InitTypes.INIT_TYPE_ID, efilter['item_id'])
            obj.delete()

def create_default_folders(REQUEST):
    mbox = Efolder(REQUEST, _InitTypes.INIT_TYPE_NEW)
    mbox.set('parent_id', REQUEST.USER.get_id())
    mbox.set('is_private',1)
    mbox.set('efolder_name', 'Inbox')
    mbox.set('efolder_flags', EFOLDER_INBOX|EFOLDER_SYSTEM)
    mbox.save()

    mbox = Efolder(REQUEST, _InitTypes.INIT_TYPE_NEW)
    mbox.set('parent_id', REQUEST.USER.get_id())
    mbox.set('is_private',1)
    mbox.set('efolder_name', 'Sent Items')
    mbox.set('efolder_flags', EFOLDER_SYSTEM|EFOLDER_SENT)
    mbox.save()

    mbox = Efolder(REQUEST, _InitTypes.INIT_TYPE_NEW)
    mbox.set('parent_id', REQUEST.USER.get_id())
    mbox.set('is_private',1)
    mbox.set('efolder_name', 'Deleted Items')
    mbox.set('efolder_flags', EFOLDER_SYSTEM|EFOLDER_DELETED)
    mbox.save()

    mbox = Efolder(REQUEST, _InitTypes.INIT_TYPE_NEW)
    mbox.set('parent_id', REQUEST.USER.get_id())
    mbox.set('is_private',1)
    mbox.set('efolder_name', 'Drafts')
    mbox.set('efolder_flags', EFOLDER_SYSTEM|EFOLDER_DRAFTS)
    mbox.save()
