# Copyright (C) 2003-2005 Inversion Software Ltd. <inversion@inversion.ee>
# http://www.conflux.ee/
#
# This file is part of Conflux Lite
#
# Conflux Lite is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2,
# or (at your option) any later version.
#
# Conflux Lite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Conflux Lite; if not, write to the Free Software
# Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
######################################################################

import md5, base64
import _RootObject, _Utils
import _InitTypes

EMAIL_NONE = 0
EMAIL_READ = 1
EMAIL_ANSWERED = 2
EMAIL_ATTACH = 4
EMAIL_DRAFT = 8
EMAIL_PRIORITY = 16

class Email(_RootObject.RootObject):
    table_name = 'emails'
    field_name = 'email_id'
    item_type_id = _InitTypes.ITEM_TYPE_EMAIL
    is_container = 0
    object_fields = {
'email_id':	{ 'type': _Utils.sql_cast_int,	'simple_type': int,	'null': 0,	'can_set': 0,	'has_default': 1,       'can_insert': 0 },
'email_from':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 0,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'email_subject':{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 0,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'email_to':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 0,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'email_date':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 0,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'email_size':	{ 'type': _Utils.sql_cast_int,	'simple_type': int,	'null': 0,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'email_flags':	{ 'type': _Utils.sql_cast_int,	'simple_type': int,	'null': 0,	'can_set': 1,	'has_default': 1,       'can_insert': 1 },
'email_message_id':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'email_contents':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 0,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'email_contacts':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'email_from_addr':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
    }

    fti_keys = ['email_from','email_subject','email_to']

    def get_name(self):
        if self.get('email_subject') is not None:
            return self.get('email_subject')
        elif self.get_id() is None:
            return self.REQUEST.TLS('emailobj_notsaved')
        else:
            return self.REQUEST.TLS('emailobj_noname') % self.get_id()

    def AfterInsertHook(self):
        self.save_received()

    def AfterUpdateHook(self):
        if not self.__olddata__.has_key('email_contents') or not self.__itemdata__.has_key('email_contents') or self.__olddata__['email_contents'] <> self.__itemdata__['email_contents']:
            self.REQUEST.db.execute("""UPDATE items SET index_this_item=1 WHERE item_id=%d""" % self.get_id())

    def save_received(self):
        if self.get('email_pop3_uid') and self.get('email_message_id'):
            SQL = u"""INSERT INTO emails_received (parent_id, email_account, email_message_id, email_pop3_uid) VALUES (%d, %s, %s, %s)""" % \
                (self.REQUEST.USER.get_id(), _Utils.sql_cast_str(self.REQUEST, self.get('email_account')), _Utils.sql_cast_str(self.REQUEST, self.get('email_message_id')), _Utils.sql_cast_str(self.REQUEST, self.get('email_pop3_uid')))
        elif self.get('email_pop3_uid'):
            md5sum = md5.md5(base64.decodestring(self.get('email_contents'))).hexdigest()
            SQL = u"""INSERT INTO emails_received (parent_id, email_account, email_message_id, email_pop3_uid) VALUES (%d, %s, %s, %s)""" % \
                (self.REQUEST.USER.get_id(), _Utils.sql_cast_str(self.REQUEST, self.get('email_account')), _Utils.sql_cast_str(self.REQUEST, md5sum), _Utils.sql_cast_str(self.REQUEST, self.get('email_pop3_uid')))
        elif self.get('email_message_id'):
            SQL = u"""INSERT INTO emails_received (parent_id, email_account, email_message_id, email_pop3_uid) VALUES (%d, %s, %s, null)""" % \
                (self.REQUEST.USER.get_id(), _Utils.sql_cast_str(self.REQUEST, self.get('email_account')), _Utils.sql_cast_str(self.REQUEST, self.get('email_message_id')))
        else:
            md5sum = md5.md5(base64.decodestring(self.get('email_contents'))).hexdigest()
            SQL = u"""INSERT INTO emails_received (parent_id, email_account, email_message_id, email_pop3_uid) VALUES (%d, %s, %s, null)""" % \
                (self.REQUEST.USER.get_id(), _Utils.sql_cast_str(self.REQUEST, self.get('email_account')), _Utils.sql_cast_str(self.REQUEST, md5sum))

        if SQL:
            self.REQUEST.db.execute(SQL.encode('UTF8'))

    def get_size(self):
        if self.get('email_contents') is None:
            return 0
        else:
            return len(self.get('email_contents'))
