# Copyright (C) 2003-2005 Inversion Software Ltd. <inversion@inversion.ee>
# http://www.conflux.ee/
#
# This file is part of Conflux Lite
#
# Conflux Lite is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2,
# or (at your option) any later version.
#
# Conflux Lite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Conflux Lite; if not, write to the Free Software
# Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
######################################################################

"""

Core Text Indexer

All content that needs to be indexed should be filtered through
required filters to achieve plain text and then fed to TextIndexer.
"""

BLOCK_SIZE = 8192

import mimetypes, tempfile, popen2, os, unicodedata, cStringIO
from string import strip
import _Utils, _Conf, _Exceptions

tempfile.tempdir = _Conf.TEMP_DOCUMENT_PATH

def clean_garbage(character):
     if unicodedata.category(character)[0] in ['L','N']:
         return character
     else:
         return u" "

class TextIndexer:

    def __init__(self, REQUEST, fp, item_id, encoding=_Conf.ENCODING, ftitype='false'):
        self.REQUEST = REQUEST
        self.fp = fp
        self.item_id = item_id
        self.encoding = encoding
        self.ftitype = ftitype

    def index(self):
        FALLBACK_ENCODING = _Conf.FALLBACK_ENCODING
        words_dict = {}

        line = self.fp.readline()
        while line:
            try:
                line = "".join(map(clean_garbage, line.decode(self.encoding).lower()))
            except: # Let's try another encoding before failing.
                line = "".join(map(clean_garbage, line.decode(FALLBACK_ENCODING).lower()))
            words = map(strip, line.split(u" "))
            for word in words:
                wordlen = len(word)
                if wordlen < 3 or wordlen > 20:
                    continue
                elif wordlen == 3:
                    words_dict[word] = 1
                else:
                    for z in range(3, wordlen+1):
                        words_dict[word[:z]] = 1
            line = self.fp.readline()
        self.fp.seek(0)

        if words_dict:
            keys = words_dict.keys()
            for delta in range(0, len(keys),10):
                loop_words = keys[delta:delta+10]
                SQL = u"""SELECT word FROM fti_search WHERE (word = %s) AND fti_type=%s AND item_id=%d GROUP BY word""" % (" OR word=".join(map(lambda x: _Utils.sql_cast_str(self.REQUEST, x), loop_words)), self.ftitype, self.item_id)
                self.REQUEST.db.execute(SQL.encode('UTF8'))
                res = map(lambda x: x[0].decode('UTF8'), self.REQUEST.db.fetchall())
                for word in loop_words:
                    if word not in res:
                        SQL = u"""INSERT INTO fti_search (item_id, word, fti_type) VALUES (%d, %s, %s)""" % (self.item_id, _Utils.sql_cast_str(self.REQUEST, word), self.ftitype)
                        self.REQUEST.db.execute(SQL.encode('UTF8'))


indexers = {
'text/plain': TextIndexer,
}

def index_file(REQUEST, fp, filename, item_id, encoding=_Conf.ENCODING, ftitype='false'):
    ftype, fileencoding = mimetypes.guess_type(filename)
    if ftype is None or not indexers.has_key(ftype): # We don't know the type, thus we do not know whether we should index it.
        return

    indexer = indexers[ftype](REQUEST, fp, item_id, encoding, ftitype)
    indexer.index()

def delete_index(REQUEST, item_id, ftitype='false'):
    REQUEST.db.execute("""\
DELETE FROM fti_search
WHERE
  item_id=%d and fti_type=%s""" % (item_id, ftitype))
