# Copyright (C) 2003-2005 Inversion Software Ltd. <inversion@inversion.ee>
# http://www.conflux.ee/
#
# This file is part of Conflux Lite
#
# Conflux Lite is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2,
# or (at your option) any later version.
#
# Conflux Lite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Conflux Lite; if not, write to the Free Software
# Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
######################################################################

import _RootObject, _Utils
import _InitTypes, _Exceptions
import os

class Organization(_RootObject.RootObject):
    SQL_INSERT_GROUP = """\
INSERT INTO user_groups
(group_org_id, group_name)
VALUES
( %(group_org_id)d, %(group_name)s );
INSERT INTO access_rights
(item_id, treepos, group_id, right_id)
VALUES
( %(group_org_id)d, %(treepos)s, currval('user_groups_seq'), 2)"""

    SQL_SELECT_GROUP = """
SELECT
 group_id
FROM user_groups
WHERE
 group_org_id=%(organization_id)d"""

    SQL_INSERT_GROUP_MEMBER = """
INSERT into access_groups
( group_id, person_id )
VALUES
( %(group_id)d, %(person_id)d )"""

    SQL_DELETE_GROUP_MEMBER = """
DELETE FROM access_groups WHERE
group_id=%(group_id)d AND person_id=%(person_id)d"""

    table_name = 'organizations'
    field_name = 'organization_id'
    item_type_id = _InitTypes.ITEM_TYPE_ORGANIZATION
    is_container = 1
    object_fields = {
'organization_id':	{ 'type': _Utils.sql_cast_int,	'simple_type': int,	'null': 0,	'can_set': 0,	'has_default': 1,       'can_insert': 0 },
'organization_name':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 0,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'address':		{ 'type': _Utils.sql_cast_str_hidenull,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'city':			{ 'type': _Utils.sql_cast_str_hidenull,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'state':		{ 'type': _Utils.sql_cast_str_hidenull,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'zipcode':		{ 'type': _Utils.sql_cast_str_hidenull,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'country':		{ 'type': _Utils.sql_cast_str_hidenull,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'website':		{ 'type': _Utils.sql_cast_str_hidenull,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'email':		{ 'type': _Utils.sql_cast_str_hidenull,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'telephone':		{ 'type': _Utils.sql_cast_str_hidenull,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'fax':			{ 'type': _Utils.sql_cast_str_hidenull,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'superuser_id':		{ 'type': _Utils.sql_cast_int,	'simple_type': int,	'null': 1,	'can_set': 0,	'has_default': 0,       'can_insert': 1 },
    }

    fti_keys = ['organization_name','address','city','state','zipcode','website','email','telephone','fax']

    def get_name(self):
        if self.get('organization_name') is not None:
            return self.get('organization_name')
        elif self.get_id() is None:
            return self.REQUEST.TLS('orgobj_notsaved')
        else:
            return self.REQUEST.TLS('orgobj_noname') % self.get_id()

    def AfterInsertHook(self):
        ns = {'group_name':_Utils.sql_cast_str(self.REQUEST, 'Default group'), 'group_org_id': self.get_id()}
        ns['treepos'] = _Utils.sql_cast_str(self.REQUEST, self.get('treepos'))
        try:
            self.REQUEST.db.execute((self.SQL_INSERT_GROUP % ns).encode('utf8'))
        except:
            raise # FIXME
            raise _Exceptions.OrganizationError, self.REQUEST.TLS('orgobj_accgrperr')

        path = self.get_path()
        try:
            os.mkdir(path.encode('UTF8'), 0755)
        except:
            raise

    def AfterUpdateHook(self):
        self.rename_hook()

    def delete_hook(self):
        oldpath = self.get_path()
        self.set('oldpath', oldpath)
        os.rmdir(oldpath.encode('UTF8'))

    def undelete_hook(self):
        oldpath = self.get('oldpath')
        if oldpath is None:
            raise _Exceptions.OrganizationError, self.REQUEST.TLS('orgobj_undeletemissing')
        os.mkdir(oldpath.encode('UTF8'))

    def rename_hook(self):
        if not self.__olddata__:
            self.REQUEST.db.execute("SELECT i.parent_id, o.organization_name FROM items i, organizations o WHERE i.item_id=o.organization_id AND item_id = %d" % self.get_id())
            r = self.REQUEST.db.dictfetchall()[0]
            old_filename = r['organization_name'].decode('UTF8')
            old_parent_id = r['parent_id']
        else:
            old_parent_id = self.__olddata__['parent_id']
            old_filename = self.__olddata__['organization_name'].decode('UTF8')
        if self.get_parent_id() == old_parent_id and self.get('organization_name') == old_filename:
            return

        oldpath = os.path.join(_RootObject.dispatch_item(self.REQUEST, old_parent_id).get_path(), _Utils.make_path(old_filename))
        os.rename(oldpath.encode('UTF8'), self.get_path().encode('UTF8'))

    def copyto_hook(self):
        raise _Exceptions.OrganizationError, self.REQUEST.TLS('orgobj_copytohook')

###
# Non-standard extensions
###
    def load_group(self):
        ns = {'organization_id': int(self.get_id())}
        self.REQUEST.db.execute((self.SQL_SELECT_GROUP % ns).encode('UTF8'))
        dbresult = self.REQUEST.db.dictfetchall()
        if len(dbresult) == 0:
            self.set('group_id', None)
        else:
            self.set('group_id', dbresult[0]['group_id'])

    def add_person(self, person_id):
        self.load_group()
        if self.get('group_id') is None:
            raise _Exceptions.OrganizationError, self.REQUEST.TLS('orgobj_noaccgrp')
        ns = {'group_id':self.get('group_id'), 'person_id':person_id}
        self.REQUEST.db.execute((self.SQL_INSERT_GROUP_MEMBER % ns).encode('UTF8'))

    def delete_person(self, person_id):
        self.load_group()
        if self.get('group_id') is None:
            raise _Exceptions.OrganizationError, self.REQUEST.TLS('orgobj_noaccgrp')
        ns = {'group_id':self.get('group_id'), 'person_id':person_id}
        self.REQUEST.db.execute((self.SQL_DELETE_GROUP_MEMBER % ns).encode('UTF8'))
