# Copyright (C) 2003-2005 Inversion Software Ltd. <inversion@inversion.ee>
# http://www.conflux.ee/
#
# This file is part of Conflux Lite
#
# Conflux Lite is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2,
# or (at your option) any later version.
#
# Conflux Lite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Conflux Lite; if not, write to the Free Software
# Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
######################################################################

import _RootObject, _Utils
import _InitTypes
import _Organization

PERSON_TRANSLATOR = 1
PERSON_CLIENTSERVICE = 2

class Person(_RootObject.RootObject):
    table_name = 'persons'
    field_name = 'person_id'
    item_type_id = _InitTypes.ITEM_TYPE_PERSON
    is_container = 1
    object_fields = {
'person_id':	{ 'type': _Utils.sql_cast_int,	'simple_type': int,	'null': 0,	'can_set': 0,	'has_default': 1,       'can_insert': 0 },
'first_name':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 0,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'middle_name':	{ 'type': _Utils.sql_cast_str_hidenull,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'last_name':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 0,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'username':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 0,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'password':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 0,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'phone_work':	{ 'type': _Utils.sql_cast_str_hidenull,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'phone_home':	{ 'type': _Utils.sql_cast_str_hidenull,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'phone_cell':	{ 'type': _Utils.sql_cast_str_hidenull,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'phone_fax':	{ 'type': _Utils.sql_cast_str_hidenull,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'e_mail':	{ 'type': _Utils.sql_cast_str_hidenull,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'picture':	{ 'type': _Utils.sql_cast_str_hidenull,	'simple_type': unicode,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'email_notify':	{ 'type': _Utils.sql_cast_int,	'simple_type': int,	'null': 0,	'can_set': 1,	'has_default': 1,       'can_insert': 1 },
'person_flags':	{ 'type': _Utils.sql_cast_int,	'simple_type': int,	'null': 0,	'can_set': 1,	'has_default': 1,       'can_insert': 1 },
    }

    fti_keys = ['first_name','last_name','middle_name','username','phone_work','phone_cell','phone_home','phone_fax',
                'e_mail']

    def get_name(self):
        if self.get('first_name') is not None and self.get('last_name') is not None:
            return "%s %s" % ( self.get('first_name'), self.get('last_name') )
        elif self.get_id() is None:
            return self.REQUEST.TLS('personobj_notsaved')
        else:
            return self.REQUEST.TLS('personobj_noname') % self.get_id()

    def AfterInsertHook(self):
        org = _Organization.Organization(self.REQUEST, _InitTypes.INIT_TYPE_ID, self.get_parent_id())
        org.add_person(self.get_id())

    def move_to_org(self, org_id):
        old_org = _Organization.Organization(self.REQUEST, _InitTypes.INIT_TYPE_ID, self.get_parent_id())
        old_org.delete_person(self.get_id())
        if old_org.get('superuser_id') == self.get_id():
            SQL = """UPDATE organizations SET superuser_id=null WHERE organization_id=%d""" % old_org.get_id()
            self.REQUEST.db.execute(SQL)
        new_org = _Organization.Organization(self.REQUEST, _InitTypes.INIT_TYPE_ID, org_id)
        new_org.add_person(self.get_id())
        self.update_access()
        self.set('parent_id', org_id)
