# Copyright (C) 2003-2005 Inversion Software Ltd. <inversion@inversion.ee>
# http://www.conflux.ee/
#
# This file is part of Conflux Lite
#
# Conflux Lite is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2,
# or (at your option) any later version.
#
# Conflux Lite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Conflux Lite; if not, write to the Free Software
# Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
######################################################################

import os, time, cStringIO, mx.DateTime, tempfile

import _Utils, _Exceptions, _Indexer

import _InitTypes
import _Access
import _Conf
import _DBSpool_utils

LOG_INSERT = 1
LOG_UPDATE = 2
LOG_DELETE = 4

SCROLLBACK_UNDELETED = 1

class RootObject:
    SQL_SELECT = """\
SELECT
 i.*,
 o.*
FROM
 items i,
 %(table_name)s o
WHERE
 i.item_id=o.%(field_name)s AND
 i.item_id=%(param_item_id)d"""
    SQL_ITEM_INSERT = u"""\
INSERT INTO
items
( %(insert_keys)s )
VALUES
( %(insert_values)s )
"""
    SQL_OBJECT_INSERT = u"""\
INSERT INTO
%(table_name)s
( %(insert_keys)s )
VALUES
( %(insert_values)s )
"""
    SQL_ITEM_UPDATE = u"""\
UPDATE
 items
SET
 item_name=%(item_name)s,
 creator_id=%(creator_id)d,
 editor_id=%(editor_id)d,
 editor_ip=%(editor_ip)s,
 parent_id=%(parent_id)d,
 is_private=%(is_private)s,
 is_public=%(is_public)s,
 edit_time=now(),
 item_size=%(item_size)d
WHERE
 item_id=%(item_id)d"""
    SQL_OBJECT_UPDATE = u"""UPDATE
 %(table_name)s
SET
 %(fl)s
WHERE
 %(field_name)s=%(item_id)d"""

    SQL_SELECT_ITEM_CURRVAL = """SELECT item_id, treepos FROM items WHERE item_id=currval('item_id_seq')"""


    SQL_GET_TID = """SELECT nextval('scrollback_id_seq') AS conflux_tid"""
    SQL_DELETE_ITEM_BK = """\
INSERT INTO
 items_scrollback
( %(fl)s, deleter_id, conflux_tid )
( SELECT
 %(fl)s,
 %(deleter_id)d,
 %(conflux_tid)d
FROM
 items
WHERE
 item_id = %(item_id)d )"""
    SQL_SELECT_DELETED_TIME = """SELECT delete_time FROM items_scrollback WHERE item_id=%(item_id)d AND conflux_tid=%(conflux_tid)d"""
    SQL_DELETE_ITEM = """DELETE FROM items WHERE item_id=%(item_id)d"""
    SQL_DELETE_RIGHTS = """DELETE FROM access_rights WHERE item_id=%(item_id)d"""

    table_name = 'NaN' # Override!
    field_name = 'NaN' # Override!
    item_type_id = None # Override!
    delete_hook = None # Override if required.
    undelete_hook = None # Override if required.
    copyto_hook = None # Override if required
    
    is_container = 0 # Override if object is container (organization, folder)

    item_fields = {
'item_id':	{ 'type': _Utils.sql_cast_int,	'simple_type': int,	'null': 0,	'can_set': 0,	'has_default': 1,	'can_insert': 0	},
'parent_id':	{ 'type': _Utils.sql_cast_int,	'simple_type': int,	'null': 0,	'can_set': 1,	'has_default': 0,       'can_insert': 1	},
'treepos':	{ 'type': _Utils.sql_cast_bit,	'simple_type': unicode,	'null': 1,	'can_set': 0,	'has_default': 0,       'can_insert': 0	},
'item_name':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 0,	'can_set': 1,	'has_default': 1,       'can_insert': 1 }, # These five have dummy has_default, since they are overrided in _RootObject code.
'item_size':	{ 'type': _Utils.sql_cast_int,	'simple_type': int,	'null': 0,	'can_set': 0,	'has_default': 1,       'can_insert': 1 },
'creator_id':	{ 'type': _Utils.sql_cast_int,	'simple_type': int,	'null': 0,	'can_set': 0,	'has_default': 1,       'can_insert': 1 },
'creator_ip':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 0,	'can_set': 0,	'has_default': 1,       'can_insert': 1 },
'create_time':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 0,	'can_set': 0,	'has_default': 1,       'can_insert': 0 },
'editor_id':	{ 'type': _Utils.sql_cast_int,	'simple_type': int,	'null': 0,	'can_set': 0,	'has_default': 1,       'can_insert': 1 },
'editor_ip':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 0,	'can_set': 0,	'has_default': 1,       'can_insert': 1 }, # end dummy has_default
'edit_time':	{ 'type': _Utils.sql_cast_str,	'simple_type': unicode,	'null': 0,	'can_set': 0,	'has_default': 1,       'can_insert': 0 },
'item_type_id':	{ 'type': _Utils.sql_cast_int,	'simple_type': int,	'null': 0,	'can_set': 0,	'has_default': 0,       'can_insert': 1 },
'is_internal':	{ 'type': _Utils.sql_cast_bool,	'simple_type': int,	'null': 0,	'can_set': 0,	'has_default': 1,       'can_insert': 1 },
'is_public':	{ 'type': _Utils.sql_cast_bool,	'simple_type': int,	'null': 0,	'can_set': 1,	'has_default': 1,       'can_insert': 1 },
'is_private':	{ 'type': _Utils.sql_cast_bool,	'simple_type': int,	'null': 0,	'can_set': 1,	'has_default': 1,       'can_insert': 1 },
'owner_id':	{ 'type': _Utils.sql_cast_int,	'simple_type': int,	'null': 1,	'can_set': 1,	'has_default': 0,       'can_insert': 1 },
'index_this_item':	{ 'type': _Utils.sql_cast_int,	'simple_type': int,	'null': 0,	'can_set': 1,	'has_default': 1,       'can_insert': 1 }
    }
    object_fields = {} # Override!
    fti_keys = [] # Override

    def __init__(self, REQUEST, init_type, extra_data=None): # Call from subclass init!
        self.REQUEST = REQUEST
        self.__olddata__  = {}
        self.__itemdata__ = {}
        self._parent = None
        self.__logdata__ = None

        for (key, value) in self.item_fields.items():
            self.__itemdata__[key] = None
            self.__olddata__[key]  = None
        for (key, value) in self.object_fields.items():
            if key in self.__itemdata__.keys():
                raise _Exceptions.RootObjectError, self.REQUEST.TLS('rootobj_matchingkeys')
            self.__itemdata__[key] = None
            self.__olddata__[key]  = None

        if self.item_type_id is None:
            raise _Exceptions.RootObjectError, self.REQUEST.TLS('rootobj_noitemtype')
        else:
            self.__itemdata__['item_type_id'] = self.item_type_id
            self.__olddata__['item_type_id'] = self.item_type_id

        if init_type == _InitTypes.INIT_TYPE_ID:
            self.load_db(extra_data)
        elif init_type == _InitTypes.INIT_TYPE_NEW:
            self.__olddata__ = None
        elif init_type == _InitTypes.INIT_TYPE_REQ:
            self.__olddata__ = None
            self.load_req()
        else:
            raise _Exceptions.RootObjectError, self.REQUEST.TLS('rootobj_invalidinit')

    def set(self, key, value): # FIXME!!
        if self.item_fields.has_key(key) and not self.item_fields[key]['can_set']:
            raise _Exceptions.RootObjectError, self.REQUEST.TLS('rootobj_nosetaccess') % key
        self.__itemdata__[key] = value

    def get(self, key):
        if self.__itemdata__.has_key(key):
            value = self.__itemdata__[key]
        else:
            value = None
        if isinstance(value, str):
            value = value.decode('UTF8')
        return value

    def get_name(self):
        if self.__itemdata__.has_key('item_name') and self.__itemdata__['item_name'] is not None:
            return self.__itemdata__['item_name']
        elif self.__itemdata__.has_key('item_id'):
            return self.REQUEST.TLS('rootobj_noname') % self.__itemdata__['item_id']
        else:
            return self.REQUEST.TLS('rootobj_notsaved')

    def get_id(self):
        return self.get('item_id')

    def get_parent_id(self):
        return self.get('parent_id')

    def get_itemdata(self):
        return self.__itemdata__

    def load_req(self):
        itemkeys = self.item_fields.keys()
        objectkeys = self.object_fields.keys()

        for key in self.REQUEST.form.keys():
             if key in itemkeys:
                 try:
                     self.set(key, self.item_fields[key]['simple_type'](self.REQUEST.form[key].value.decode('UTF8')))
                 except _Exceptions.RootObjectError:
                     raise _Exceptions.RootObjectError, self.REQUEST.TLS('rootobj_nosetaccess') % key
                 except:
                     pass
             elif key in objectkeys:
                 try:
                     self.set(key, self.object_fields[key]['simple_type'](self.REQUEST.form[key].value.decode('UTF8')))
                 except _Exceptions.RootObjectError:
                     raise _Exceptions.RootObjectError, self.REQUEST.TLS('rootobj_nosetaccess') % key
                 except:
                     pass

    def load_db(self, item_id):
        self.__itemdata__['item_id'] = item_id
        self.canRead(fRaise=1)
        try:
            self.REQUEST.db.execute(self.sqlSelect())
        except:
            raise _Exceptions.RootObjectError, self.REQUEST.TLS('rootobj_selecterror')

        result = self.REQUEST.db.dictfetchall()[0]
        for key in self.__itemdata__.keys():
            if result.has_key(key):
                self.__itemdata__[key] = result[key]
                self.__olddata__[key] = result[key]

    def sqlInsert(self):
        if self.__itemdata__['item_id']:
            raise _Exceptions.RootObjectError, self.REQUEST.TLS('rootobj_primarykeyset')

        item_items = {}
        object_items = {}

        # check input
        for (key, item) in self.item_fields.items():
            # FIXME
            # Opportunity and client can change it's creator id
            if key in self.__itemdata__.keys() and not item['can_insert']:
                if not (key == 'creator_id' and self.item_type_id in [_InitTypes.ITEM_TYPE_CLIENT, _InitTypes.ITEM_TYPE_OPPORTUNITY]):
                    continue
            try:
                if self.__itemdata__.has_key(key) and (item['null'] or (not item['null'] and self.__itemdata__[key] is not None)):
                    item_items[key] = item['type'](self.REQUEST, self.__itemdata__[key])
            except:
                raise _Exceptions.RootObjectError, self.REQUEST.TLS('rootobj_typecasterr') % key
        for (key, item) in self.item_fields.items():
            if key not in item_items.keys() and not item['has_default'] and item['can_insert'] and not item['null']:
                raise _Exceptions.RootObjectError, self.REQUEST.TLS('rootobj_missingnodefault') % key
        for (key, item) in self.object_fields.items():
            if key in self.__itemdata__.keys() and not item['can_insert']:
                continue
            try:
                if self.__itemdata__.has_key(key) and (item['null'] or (not item['null'] and self.__itemdata__[key] is not None)):
                    object_items[key] = item['type'](self.REQUEST, self.__itemdata__[key])
            except:
                raise _Exceptions.RootObjectError, self.REQUEST.TLS('rootobj_typecasterr') % key
        for (key, item) in self.object_fields.items():
            if key not in object_items.keys() and not item['has_default'] and item['can_insert'] and not item['null']:
                raise _Exceptions.RootObjectError, self.REQUEST.TLS('rootobj_missingnodefault') % key

        item_items['item_size'] = self.get_size()
        if self.item_type_id in [_InitTypes.ITEM_TYPE_CLIENT, _InitTypes.ITEM_TYPE_OPPORTUNITY] and item_items.has_key('creator_id'):
            item_items['creator_id'] = self.item_fields['creator_id']['type'](self.REQUEST, item_items['creator_id'])
        else:
            item_items['creator_id'] = self.item_fields['creator_id']['type'](self.REQUEST, self.REQUEST.USER.get_id())
        item_items['editor_id'] = self.item_fields['editor_id']['type'](self.REQUEST, self.REQUEST.USER.get_id())
        item_items['creator_ip'] = self.item_fields['creator_ip']['type'](self.REQUEST, self.REQUEST.USER.get_host())
        item_items['editor_ip'] = self.item_fields['editor_ip']['type'](self.REQUEST, self.REQUEST.USER.get_host())
        item_items['item_name'] = self.item_fields['item_name']['type'](self.REQUEST, self.get_name())
        if len(item_items['item_name']) > 200:
            item_items['item_name'] = item_items['item_name'][:200] + '...' + "'"

        # INSERT
        QUERY = []

        ns = {}
        ns['insert_keys'] = ','.join(item_items.keys())
        ns['insert_values'] = ','.join(map(unicode, item_items.values()))
        QUERY.append(self.SQL_ITEM_INSERT % ns)

        ns['insert_keys'] = ','.join(object_items.keys())
        ns['insert_values'] = ','.join(map(unicode, object_items.values()))
        ns['table_name'] = self.table_name
        QUERY.append(self.SQL_OBJECT_INSERT % ns)

        return u";\n".join(QUERY).encode('UTF8')

    def sqlUpdate(self):
        if not self.__itemdata__.has_key('item_id') or not self.__itemdata__.has_key(self.field_name):
            raise _Exceptions.RootObjectError, self.REQUEST.TLS('rootobj_primarykeymissing')

        ns = {}
        ns['item_size'] = self.get_size()
        ns['item_name'] = self.item_fields['item_name']['type'](self.REQUEST, self.get_name())
        if len(ns['item_name']) > 200:
            ns['item_name'] = ns['item_name'][:200] + '...' + "'"
        ns['editor_id'] = self.item_fields['editor_id']['type'](self.REQUEST, self.REQUEST.USER.get_id())
        if self.item_type_id in [_InitTypes.ITEM_TYPE_CLIENT, _InitTypes.ITEM_TYPE_OPPORTUNITY] and self.__itemdata__.has_key('creator_id'):
            ns['creator_id'] = self.item_fields['creator_id']['type'](self.REQUEST, self.__itemdata__['creator_id'])
        elif self.get('creator_id') is None:
            REQUEST.db.execute("""SELECT creator_id FROM items WHERE item_id=%d""" % self.get_id())
            ns['creator_id'] = REQUEST.db.fetchone()[0]
        else:
            ns['creator_id'] = self.get('creator_id')
        ns['editor_ip'] = self.item_fields['editor_ip']['type'](self.REQUEST, self.REQUEST.USER.get_host())
        ns['item_id'] = self.item_fields['item_id']['type'](self.REQUEST, self.get_id())
        ns['parent_id'] = self.item_fields['parent_id']['type'](self.REQUEST, self.get_parent_id())
        ns['is_public'] = self.item_fields['is_public']['type'](self.REQUEST, self.get('is_public'))
        ns['is_private'] = self.item_fields['is_private']['type'](self.REQUEST, self.get('is_private'))

        QUERY = []
        QUERY.append(self.SQL_ITEM_UPDATE % ns)

        ns = {}
        fl = []
        for (key, item) in self.object_fields.items():
            if item['can_insert']:
                fl.append("%s=%s" % (key, self.object_fields[key]['type'](self.REQUEST, self.get(key))))
        fl = ",\n".join(fl)

        ns['item_id'] = self.get_id()
        ns['table_name'] = self.table_name
        ns['field_name'] = self.field_name
        ns['fl'] = fl

        QUERY.append(self.SQL_OBJECT_UPDATE % ns)

        return u";\n".join(QUERY).encode('UTF8')

    def log_data(self):
        """We do a two-step select/compare here to avoid any manual type conversions
           that we'd have to do"""
        if self.get_id() is None: # we only log changed items, not new
            return
        if self.item_type_id == _InitTypes.ITEM_TYPE_EMAIL:
            return
        if self.__logdata__ is None:
            # load old values from db
            keys = self.item_fields.keys() + self.object_fields.keys()
            dbvalues = {}
            self.REQUEST.db.execute(self.sqlSelect())
            result = self.REQUEST.db.dictfetchone()
            for key in keys:
                if result.has_key(key):
                    dbvalues[key] = result[key]
            self.__logdata__ = dbvalues
        else:
            LOG_SQL = u"""INSERT INTO
 item_log (
 item_id,
 batch_id,
 person_id,
 field_name,
 old_value,
 new_value,
 log_flags )
VALUES (
 %(item_id)s,
 %(batch_id)s,
 %(person_id)s,
 %(field_name)s,
 %(old_value)s,
 %(new_value)s,
 %(log_flags)d )"""
            # load old values from db
            keys = self.item_fields.keys() + self.object_fields.keys()
            dbvalues = {}
            self.REQUEST.db.execute(self.sqlSelect())
            result = self.REQUEST.db.dictfetchone()
            for key in keys:
                if result.has_key(key):
                    dbvalues[key] = result[key]
            mxclass = type(mx.DateTime.now())
            changed_fields = {}
            for key in keys:
                old_val = self.__logdata__[key]
                strip_quotes_1 = 0
                strip_quotes_2 = 0
                if isinstance(old_val, mxclass):
                    old_val = "mx.DateTime.DateTime(%d, %d, %d, %d, %d)" % (old_val.year,
                                                                            old_val.month,
                                                                            old_val.day,
                                                                            old_val.hour,
                                                                            old_val.minute)
                    strip_quotes_1 = 1
                elif isinstance(old_val, _DBSpool_utils.TimeParser) or \
                     ( hasattr(old_val, 's') and \
                       hasattr(old_val, 'delta') and \
                       hasattr(old_val, 'value')
                     ): # FIXME Hack for timeparser problem fix
                    old_val = "_DBSpool_utils.TimeParser('%s')" % old_val.s
                    strip_quotes_1 = 1
                elif self.object_fields.has_key(key) and self.object_fields[key]['simple_type'] is list:
                    old_val = _Utils.sql_parse_array(self.REQUEST, old_val)
                elif isinstance(old_val, str):
                    old_val = old_val.decode('UTF8')
                new_val = dbvalues[key]
                if isinstance(new_val, mxclass):
                    new_val = "mx.DateTime.DateTime(%d, %d, %d, %d, %d)" % (new_val.year,
                                                                            new_val.month,
                                                                            new_val.day,
                                                                            new_val.hour,
                                                                            new_val.minute)
                    strip_quotes_2 = 1
                elif isinstance(new_val, _DBSpool_utils.TimeParser) or \
                     ( hasattr(new_val, 's') and \
                       hasattr(new_val, 'delta') and \
                       hasattr(new_val, 'value')
                     ): # FIXME Hack for timeparser problem fix
                    new_val = "_DBSpool_utils.TimeParser('%s')" % new_val.s
                    strip_quotes_2 = 1
                elif self.object_fields.has_key(key) and self.object_fields[key]['simple_type'] is list:
                    new_val = _Utils.sql_parse_array(self.REQUEST, new_val)
                elif isinstance(new_val, str):
                    new_val = new_val.decode('UTF8')
                if old_val <> new_val:
                    changed_fields[key] = (old_val, new_val, strip_quotes_1, strip_quotes_2)
            if changed_fields:
                self.REQUEST.db.execute("""SELECT nextval('log_batch_seq') AS batch_id""")
                batch_id = self.REQUEST.db.fetchone()[0]
                ns = { 'item_id': self.get_id(),
                       'batch_id': batch_id,
                       'person_id': self.REQUEST.USER.get_id()
                     }
                for key in changed_fields.keys():
                    old_val = changed_fields[key][0]
                    new_val = changed_fields[key][1]
                    ns['field_name'] = _Utils.sql_quote(key)
                    if changed_fields[key][2]:
                        ns['old_value'] = _Utils.sql_quote(repr(old_val)[1:-1])
                    else:
                        ns['old_value'] = _Utils.sql_quote(repr(old_val))
                    if changed_fields[key][3]:
                        ns['new_value'] = _Utils.sql_quote(repr(new_val)[1:-1])
                    else:
                        ns['new_value'] = _Utils.sql_quote(repr(new_val))
                    ns['log_flags'] = LOG_UPDATE
                    sql = LOG_SQL % ns
                    self.REQUEST.db.execute(sql.encode('UTF8'))



            self.__logdata__ = None

    def log_delete(self):
        if self.item_type_id == _InitTypes.ITEM_TYPE_EMAIL:
            return
        if self.get_id() is not None: # we only log changed items, not new
            LOG_SQL = u"""INSERT INTO
 item_log (
 item_id,
 batch_id,
 person_id,
 field_name,
 old_value,
 new_value,
 log_flags )
VALUES (
 %(item_id)s,
 %(batch_id)s,
 %(person_id)s,
 %(field_name)s,
 %(old_value)s,
 %(new_value)s,
 %(log_flags)d )"""
            # load values from db
            keys = self.item_fields.keys() + self.object_fields.keys()
            dbvalues = {}
            self.REQUEST.db.execute(self.sqlSelect())
            result = self.REQUEST.db.dictfetchone()
            for key in keys:
                if result.has_key(key):
                    dbvalues[key] = result[key]
            mxclass = type(mx.DateTime.now())
            changed_fields = {}
            for key in keys:
                strip_quotes = 0
                new_val = dbvalues[key]
                if isinstance(new_val, mxclass):
                    new_val = "mx.DateTime.DateTime(%d, %d, %d, %d, %d)" % (new_val.year,
                                                                            new_val.month,
                                                                            new_val.day,
                                                                            new_val.hour,
                                                                            new_val.minute)
                    strip_quotes = 1
                elif isinstance(new_val, _DBSpool_utils.TimeParser) or \
                     ( hasattr(new_val, 's') and \
                       hasattr(new_val, 'delta') and \
                       hasattr(new_val, 'value')
                     ): # FIXME Hack for timeparser problem fix
                    new_val = "_DBSpool_utils.TimeParser('%s')" % new_val.s
                    strip_quotes = 1
                elif self.object_fields.has_key(key) and self.object_fields[key]['simple_type'] == list:
                    new_val = _Utils.sql_parse_array(self.REQUEST, new_val)
                elif isinstance(new_val, str):
                    new_val = new_val.decode('UTF8')
                changed_fields[key] = (new_val, strip_quotes)

            self.REQUEST.db.execute("""SELECT nextval('log_batch_seq') AS batch_id""")
            batch_id = self.REQUEST.db.fetchone()[0]
            ns = { 'item_id': self.get_id(),
                   'batch_id': batch_id,
                    'person_id': self.REQUEST.USER.get_id()
                 }
            for key in changed_fields.keys():
                new_val = changed_fields[key][0]
                ns['field_name'] = _Utils.sql_quote(key)
                if changed_fields[key][1]:
                    ns['old_value'] = _Utils.sql_quote(repr(new_val)[1:-1])
                else:
                    ns['old_value'] = _Utils.sql_quote(repr(new_val))
                ns['new_value'] = "'None'"
                ns['log_flags'] = LOG_DELETE
                sql = LOG_SQL % ns
                self.REQUEST.db.execute(sql.encode('UTF8'))

    def log_insert(self):
        if self.item_type_id == _InitTypes.ITEM_TYPE_EMAIL:
            return
        if self.get_id() is not None: # we only log changed items, not new
            LOG_SQL = u"""INSERT INTO
 item_log (
 item_id,
 batch_id,
 person_id,
 field_name,
 old_value,
 new_value,
 log_flags )
VALUES (
 %(item_id)s,
 %(batch_id)s,
 %(person_id)s,
 %(field_name)s,
 %(old_value)s,
 %(new_value)s,
 %(log_flags)d )"""
            # load values from db
            keys = self.item_fields.keys() + self.object_fields.keys()
            dbvalues = {}
            self.REQUEST.db.execute(self.sqlSelect())
            result = self.REQUEST.db.dictfetchone()
            for key in keys:
                if result.has_key(key):
                    dbvalues[key] = result[key]
            mxclass = type(mx.DateTime.now())
            changed_fields = {}
            for key in keys:
                strip_quotes = 0
                new_val = dbvalues[key]
                if isinstance(new_val, mxclass):
                    new_val = "mx.DateTime.DateTime(%d, %d, %d, %d, %d)" % (new_val.year,
                                                                            new_val.month,
                                                                            new_val.day,
                                                                            new_val.hour,
                                                                            new_val.minute)
                    strip_quotes = 1
                elif isinstance(new_val, _DBSpool_utils.TimeParser) or \
                     ( hasattr(new_val, 's') and \
                       hasattr(new_val, 'delta') and \
                       hasattr(new_val, 'value')
                     ): # FIXME Hack for timeparser problem fix
                    new_val = "_DBSpool_utils.TimeParser('%s')" % new_val.s
                    strip_quotes = 1
                elif self.object_fields.has_key(key) and self.object_fields[key]['simple_type'] == list:
                    new_val = _Utils.sql_parse_array(self.REQUEST, new_val)
                elif isinstance(new_val, str):
                    new_val = new_val.decode('UTF8')
                changed_fields[key] = (new_val, strip_quotes)

            self.REQUEST.db.execute("""SELECT nextval('log_batch_seq') AS batch_id""")
            batch_id = self.REQUEST.db.fetchone()[0]
            ns = { 'item_id': self.get_id(),
                   'batch_id': batch_id,
                    'person_id': self.REQUEST.USER.get_id()
                 }
            for key in changed_fields.keys():
                new_val = changed_fields[key][0]
                ns['field_name'] = _Utils.sql_quote(key)
                if changed_fields[key][1]:
                    ns['new_value'] = _Utils.sql_quote(repr(new_val)[1:-1])
                else:
                    ns['new_value'] = _Utils.sql_quote(repr(new_val))
                ns['old_value'] = "'None'"
                ns['log_flags'] = LOG_INSERT
                sql = LOG_SQL % ns
                self.REQUEST.db.execute(sql.encode('UTF8'))

    def save(self):
        # Check for quota
        old_parent = None
        old_size = None
        if self.item_type_id <> _InitTypes.ITEM_TYPE_QUOTA:
            self.REQUEST.su_by_id(_Conf.SUPERUSER)
            topquota = self.get_top_parent().get_quota()
            if topquota and topquota.get('quota_used') > (topquota.get('quota_hard_size')*1024):
                raise RuntimeError, self.REQUEST.TLS('quota_exceeded')
            self.REQUEST.unsu()
        if self.get_id() is None:
            self.canInsert(fRaise=1)
            if self.__itemdata__['parent_id'] is None:
                raise _Exceptions.RootObjectError, self.REQUEST.TLS('rootobj_parentmissing')
            self.BeforeInsertHook()
            try:
                self.REQUEST.db.execute(self.sqlInsert())
            except _Exceptions.RootObjectError, message:
                raise _Exceptions.RootObjectError, message

            self.REQUEST.db.execute(self.SQL_SELECT_ITEM_CURRVAL)
            dbres = self.REQUEST.db.dictfetchall()
            item_id = dbres[0]['item_id']
            treepos = dbres[0]['treepos']
            self.__itemdata__['item_id'] = item_id
            self.__itemdata__['treepos'] = treepos
            self.__itemdata__[self.field_name] = item_id
            self.AfterInsertHook()
            self.log_insert()
        else:
            self.canUpdate(fRaise=1)
            self.REQUEST.db.execute("""SELECT item_size, parent_id FROM items WHERE item_id=%d""" % self.get_id())
            old_size, old_parent = self.REQUEST.db.fetchone()
            if self.__itemdata__['parent_id'] is None:
                raise _Exceptions.RootObjectError, self.REQUEST.TLS('rootobj_parentmissing')
            if self.__itemdata__['item_id'] is None and self.__itemdata__[self.field_name] is None:
                raise _Exceptions.RootObjectError, self.REQUEST.TLS('rootobj_primarykeymissing')
            elif self.__itemdata__['item_id'] is None:
                self.__itemdata__['item_id'] = self.__itemdata__[self.field_name]

            update_access = 0
            if not self.__olddata__:
                self.REQUEST.db.execute("""SELECT parent_id FROM items WHERE item_id=%d""" % self.get_id())
                if self.__itemdata__['parent_id'] <> self.REQUEST.db.dictfetchone()['parent_id']:
                    update_access = 1

            self.log_data()
            self.BeforeUpdateHook()
            try:
                self.REQUEST.db.execute(self.sqlUpdate())
            except _Exceptions.RootObjectError, message:
                raise _Exceptions.RootObjectError, message
            self.AfterUpdateHook()
            if update_access or (self.__olddata__ and self.__itemdata__ and self.__olddata__['parent_id'] <> self.__itemdata__['parent_id']):
                self.update_access()
            self.log_data()
        # Update metadata indexes
        ftifp = cStringIO.StringIO()
        for key in self.fti_keys:
            val = self.get(key)
            if val and isinstance(val, unicode):
                ftifp.write(val.encode('UTF8') + ' ')
            elif val and isinstance(val, int):
                ftifp.write(str(val) + ' ')
            elif val and isinstance(val, float):
                ftifp.write(str(val) + ' ')
        ftifp.seek(0)
        _Indexer.delete_index(self.REQUEST, self.get_id(), ftitype='true')
        _Indexer.index_file(self.REQUEST, ftifp, 'a.txt', self.get_id(), ftitype='true')
        ftifp.close()
        del ftifp

        # Update quota
        if self.item_type_id <> _InitTypes.ITEM_TYPE_QUOTA:
            self.REQUEST.su_by_id(_Conf.SUPERUSER)
            topquota = self.get_top_parent().get_quota()
            if topquota:
                topquota.set('quota_used', topquota.get('quota_used') + self.get_size())
                topquota.save()
            if old_parent is not None:
                topquota = dispatch_item(self.REQUEST, old_parent).get_top_parent().get_quota()
                if topquota:
                    topquota.set('quota_used', topquota.get('quota_used') - old_size)
                    topquota.save()
            self.REQUEST.unsu()

    def sqlSelect(self):
        if not self.__itemdata__.has_key('item_id'):
            raise _Exceptions.RootObjectError, self.REQUEST.TLS('rootobj_selectnoprimary')
        ns = {}
        ns['table_name'] = self.table_name
        ns['field_name'] = self.field_name
        ns['param_item_id'] = self.__itemdata__['item_id']

        return (self.SQL_SELECT % ns)

    def canRead(self, fRaise=0): #FIXME
        if _Access.get_item_right(self.REQUEST, self.get_id(), self.REQUEST.USER.get_id()) < _Access.ACCESS_READ:
            if fRaise:
                raise _Exceptions.AccessError, self.REQUEST.TLS('rootobj_noreadaccess')
            else: return 0
        return 1

    def canInsert(self, fRaise=0, parent_id=None): #FIXME
        if parent_id is None:
            parent_id = self.get_parent_id()

        if self.REQUEST.USER.get_id() == self.get_top_parent().get('superuser_id'):
            req_right = _Access.ACCESS_NONE
        elif self.item_type_id in [_InitTypes.ITEM_TYPE_ORGANIZATION, _InitTypes.ITEM_TYPE_PERSON]:
            req_right = _Access.ACCESS_MANAGE
        else:
            req_right = _Access.ACCESS_WRITE

        if parent_id is None or (_Access.get_item_right(self.REQUEST, parent_id, self.REQUEST.USER.get_id()) < req_right):
            if fRaise:
                raise _Exceptions.AccessError, self.REQUEST.TLS('rootobj_noinsertaccess')
            else: return 0
        return 1

    def canUpdate(self, fRaise=0): #FIXME
        if self.get_id() == _Conf.ROOTOBJECT:
            if fRaise:
                raise _Exceptions.AccessError, self.REQUEST.TLS('rootobj_noupdateaccess')
            else: return 0

        if self.REQUEST.USER.get_id() == self.get_top_parent().get('superuser_id'):
            req_right = _Access.ACCESS_NONE
        elif self.item_type_id == _InitTypes.ITEM_TYPE_ORGANIZATION:
            req_right = _Access.ACCESS_MANAGE
        elif self.item_type_id == _InitTypes.ITEM_TYPE_PERSON and self.get_id() <> self.REQUEST.USER.get_id():
            req_right = _Access.ACCESS_MANAGE
        else:
            req_right = _Access.ACCESS_WRITE

        if _Access.get_item_right(self.REQUEST, self.get_parent_id(), self.REQUEST.USER.get_id()) < req_right:
            if fRaise:
                raise _Exceptions.AccessError, self.REQUEST.TLS('rootobj_noupdateaccess')
            else: return 0

        if self.__olddata__ is not None and self.__olddata__.has_key('parent_id') and \
           self.get_parent_id() <> self.__olddata__['parent_id'] and \
            _Access.get_item_right(self.REQUEST, self.__olddata__['parent_id'], self.REQUEST.USER.get_id()) < req_right:
            if fRaise:
                raise _Exceptions.AccessError, self.REQUEST.TLS('rootobj_noupdateaccess')
            else: return 0

        self.REQUEST.su_by_id(_Conf.SUPERUSER)
        if self.get_parent().get('treepos')[:len(self.get('treepos'))] == self.get('treepos'):
            raise _Exceptions.RootObjectError, self.REQUEST.TLS('rootobj_recursiveupdate')
        self.REQUEST.unsu()
        return 1

    def canDelete(self, fRaise=0): #FIXME
        if self.get_id() == _Conf.ROOTOBJECT:
            if fRaise:
                raise _Exceptions.AccessError, self.REQUEST.TLS('rootobj_noupdateaccess')
            else: return 0
        if self.REQUEST.USER.get_id() == self.get_top_parent().get('superuser_id'):
            req_right = _Access.ACCESS_NONE
        elif self.item_type_id in [_InitTypes.ITEM_TYPE_ORGANIZATION, _InitTypes.ITEM_TYPE_PERSON]:
            req_right = _Access.ACCESS_MANAGE
        else:
            req_right = _Access.ACCESS_WRITE
        if _Access.get_item_right(self.REQUEST, self.get_id(), self.REQUEST.USER.get_id()) < req_right:
            if fRaise:
                raise _Exceptions.AccessError, self.REQUEST.TLS('rootobj_nodeleteaccess')
            else: return 0
        return 1

    def get_path(self):
        if self.REQUEST.USER.get_id() == _Conf.SUPERUSER:
            return self._get_path()
        self.REQUEST.su_by_id(_Conf.SUPERUSER)
        path = self._get_path()
        self.REQUEST.unsu()
        return path

    def _get_path(self):
        if self.get_id() == _Conf.ROOTOBJECT:
            return _Conf.BASE_DOCUMENT_PATH
        elif self.get_parent_id() == _Conf.ROOTOBJECT:
            return os.path.join(_Conf.BASE_DOCUMENT_PATH, _Utils.make_path(self.get_name()))
        else:
            return os.path.join(self.get_parent()._get_path(), _Utils.make_path(self.get_name()))

    def get_virtual_path(self):
        if self.REQUEST.USER.get_id() == _Conf.SUPERUSER:
            return self._get_virtual_path()
        self.REQUEST.su_by_id(_Conf.SUPERUSER)
        path = self._get_virtual_path()
        self.REQUEST.unsu()
        return path

    def _get_virtual_path(self):
        if self.get_parent_id() == _Conf.ROOTOBJECT:
            return "/%s" % self.get_name()
        else:
            return os.path.join(self.get_parent()._get_virtual_path(), self.get_name())

    def get_parent(self): # FIXME
        if self._parent is None:
            self._parent = dispatch_item(self.REQUEST, self.get_parent_id())
        return self._parent

    def get_top_parent(self):
        if self.REQUEST.USER.get_id() == _Conf.SUPERUSER:
            return self._get_top_parent()
        self.REQUEST.su_by_id(_Conf.SUPERUSER)
        parent = self._get_top_parent()
        self.REQUEST.unsu()
        return parent

    def _get_top_parent(self):
        if self.get_parent_id() == _Conf.ROOTOBJECT:
            return self
        else:
            return self.get_parent()._get_top_parent()

    def update_access(self): # This functions updates the treepos path and accesses. Should be called when parent_id has changed
#        return # treepos still needs to update all the children, but we'll let the trigger do all the work for us

        tp = _Utils.sql_cast_bit(self.REQUEST, self.get('treepos')).encode('UTF8')
        self.REQUEST.db.execute("""SELECT item_id FROM items WHERE treepos > %s and treepos < (%s || B'1') ORDER BY treepos ASC""" % (tp, tp))
        children = self.REQUEST.db.dictfetchall()
        for child in children:
            self.REQUEST.db.execute("""UPDATE items set creator_id=creator_id WHERE item_id=%d""" % child['item_id'])


    def delete(self):
        self.canDelete(fRaise=1)
        self.BeforeDeleteHook()
        self.REQUEST.db.execute("""SELECT count(item_id) AS num_items FROM items WHERE parent_id=%d""" % _Utils.sql_cast_int(self.REQUEST, self.get_id()))
        children = self.REQUEST.db.dictfetchall()[0]['num_items']
        if children:
            self.REQUEST.db.execute(self.SQL_GET_TID)
            conflux_tid = self.REQUEST.db.dictfetchall()[0]['conflux_tid']
            treepos = _Utils.sql_cast_bit(self.REQUEST, self.get('treepos')).encode('UTF8')
            self.REQUEST.db.execute("""
SELECT
 item_id,
 treepos
FROM
 items
LEFT JOIN documents ON documents.document_id=items.item_id
WHERE
 treepos >= %s and treepos < (%s || B'1') AND
 documents.final_version_id is NULL
ORDER BY treepos DESC""" % (treepos, treepos))
            children = self.REQUEST.db.dictfetchall()

            deleted = []
            try:
                for child in children:
                    item = dispatch_item(self.REQUEST, child['item_id'])
                    item.full_delete(tid=conflux_tid)
                    deleted.append(item)
            except:
                    deleted.reverse()
                    for item in deleted:
                        if item.undelete_hook:
                            item.undelete_hook()
                    raise
        else:
            self.full_delete()
        _Indexer.delete_index(self.REQUEST, self.get_id(), ftitype='true')
        _Indexer.delete_index(self.REQUEST, self.get_id(), ftitype='false')
        self.AfterDeleteHook()

    def full_delete(self, tid=None): # DO NOT CALL DIRECTLY.
        ns = {}
        if tid is None:
            self.REQUEST.db.execute(self.SQL_GET_TID)
            ns['conflux_tid'] = self.REQUEST.db.dictfetchall()[0]['conflux_tid']
        else:
            ns['conflux_tid'] = tid

        keys = ['item_id', 'parent_id', 'item_name', 'item_type_id']
        ns['fl'] = ", ".join(keys)
        ns['deleter_id'] = self.REQUEST.USER.get_id()
        ns['item_id'] = self.get_id()
        self.log_delete()
        self.REQUEST.db.execute(self.SQL_DELETE_ITEM_BK % ns)
        self.REQUEST.db.execute(self.SQL_DELETE_ITEM % ns)
        self.REQUEST.db.execute(self.SQL_DELETE_RIGHTS % ns)

        self.REQUEST.db.execute(self.SQL_SELECT_DELETED_TIME % ns)
        self.set('delete_time', self.REQUEST.db.dictfetchall()[0]['delete_time'])

        if self.delete_hook: # If we have a funky object, let it clean up the non-db things in the end
            self.delete_hook()
        # Update quota
        if self.item_type_id <> _InitTypes.ITEM_TYPE_QUOTA:
            self.REQUEST.su_by_id(_Conf.SUPERUSER)
            topquota = self.get_top_parent().get_quota()
            if topquota:
                topquota.set('quota_used', topquota.get('quota_used') - self.get_size())
                topquota.save()
            self.REQUEST.unsu()
        
    def copyto(self, new_parent_id, _toplevel=1, depth=-1): # NEVER EVER use _toplevel as a call argument, this is an internal variable
        # Check if we have insert rights to the new parent
        self.canInsert(parent_id=new_parent_id, fRaise=1)
        if dispatch_item(self.REQUEST, new_parent_id).get('treepos')[:len(self.get('treepos'))] == self.get('treepos'):
            raise _Exceptions.RootObjectError, self.REQUEST.TLS('rootobj_recursiveupdate')
        # Copy item itself
        new_parent = self.full_copyto(new_parent_id)
        new_parent_id = new_parent.get_id()

        # Tell children to copy to new item, if depth is acceptable
        if depth <> 0:
            self.REQUEST.db.execute(("""SELECT item_id, treepos FROM items WHERE parent_id=%d""" % _Utils.sql_cast_int(self.REQUEST, self.get_id())).encode('UTF8'))
            children = self.REQUEST.db.dictfetchall()
            try:
                for child in children:
                    item = dispatch_item(self.REQUEST, child['item_id'])
                    item.copyto(new_parent_id, _toplevel=0, depth=depth-1)
            except:
                if _toplevel:
                    new_parent.delete()
                raise
        return new_parent_id

    def full_copyto(self, new_parent_id): # DO NOT CALL DIRECTLY.
        new_obj = self.__class__(self.REQUEST, _InitTypes.INIT_TYPE_NEW)
        for (key, values) in self.item_fields.items():
            if values['can_set']:
                new_obj.set(key, self.get(key))
        for (key, values) in self.object_fields.items():
            if values['can_set']:
                new_obj.set(key, self.get(key))
        new_obj.set('parent_id', new_parent_id)
        if new_obj.copyto_hook: new_obj.copyto_hook() # This is supposed to check if stuff exists and then alter name
        new_obj.save()
        return new_obj

    def make_backup_dir(self):
        now = time.gmtime()
        folder = os.path.join(_Conf.BACKUP_DOCUMENT_PATH, os.path.join(os.path.join(str(now[0]), str(now[1])), str(now[2])))
        if not os.path.exists(folder):
            os.makedirs(folder)
        return folder

    def xml_tree(self, depth, types, fp=None, indent=1): # htmlquote
        # Start file
        is_toplevel = 0
        if fp is None:
            is_toplevel = 1
            fp = tempfile.TemporaryFile()
            fp.write("""<?xml version="1.0" encoding="UTF-8"?>\n<ConfluxRoot>\n""")

        # XML-ify self
        keys = self.item_fields.keys() + self.object_fields.keys()
        keys.sort()
        mxclass = type(mx.DateTime.now())
        fp.write("  "*indent + "  <ConfluxNode>\n""")
        for key in keys:
            if key in ['treepos']: # These fields have no value in XML
                continue
            val = self.get(key)
            if isinstance(val, mxclass):
                val = "%04d%02d%02dT%02d:%02d" % (val.year,
                                                      val.month,
                                                      val.day,
                                                      val.hour,
                                                      val.minute)
            elif isinstance(val, _DBSpool_utils.TimeParser) or \
                     ( hasattr(val, 's') and \
                       hasattr(val, 'delta') and \
                       hasattr(val, 'value')
                     ): # FIXME Hack for timeparser problem fix
                if val.delta is not None:
                    val = "DateTimeDelta(%d, %d, %d)" % (val.delta.day, val.delta.hour, val.delta.minute)
                else:
                    val = None
            elif isinstance(val, unicode):
                val = _Utils.html_quote(val)
            if val is None:
                val = ''
            fp.write("  "*(indent+1))
            fp.write( (u"""<%s>%s</%s>\n""" % (key, val, key)).encode('UTF8') )

        # Children
        if depth <> 0:
            SQL = u"""SELECT item_id FROM items WHERE parent_id=%d AND item_id <> %d""" % (self.get_id(), _Conf.ROOTOBJECT)
            if types:
                SQL += u" AND item_type_id in (%s)" % ",".join(map(str,types))
            self.REQUEST.db.execute(SQL.encode('UTF8'))
            children = self.REQUEST.db.fetchall()
            if children:
                fp.write("  "*(indent+1) + "<NodeChildren>\n")
            for child in children:
                child_obj = dispatch_item(self.REQUEST, child[0])
                child_obj.xml_tree(depth-1, types, fp, indent+2)
            if children:
                fp.write("  "*(indent+1) + "</NodeChildren>\n")

        fp.write("  "*indent + "  </ConfluxNode>\n""")

        # End file
        if is_toplevel:
            fp.write("""</ConfluxRoot>\n""")
            fp.seek(0)
            return fp

    def log_undo(self, batch_id):
        item_id = self.get_id()
        LOG_SQL = u"""SELECT
 l.field_name,
 l.new_value
FROM
 item_log l
WHERE
 l.item_id = %(item_id)d AND
 l.batch_id = (
  SELECT
   coalesce(max(l2.batch_id), -1)
  FROM
   item_log l2
  WHERE
   l2.item_id=%(item_id)d AND
   l2.field_name=l.field_name AND
   l2.batch_id <= %(batch_id)d
)
""" % locals()

        self.REQUEST.db.execute(self.sqlSelect())
        current_data = self.REQUEST.db.dictfetchone()
        self.REQUEST.db.execute(LOG_SQL.encode('UTF8'))
        log_data = self.REQUEST.db.fetchall()
        for row in log_data:
            current_data[row[0]] = eval(row[1])
        for key in current_data.keys():
            self.__itemdata__[key] = current_data[key]

    def get_quota(self):
        if self.get_id() is None:
            return
        if self.item_type_id <> _InitTypes.ITEM_TYPE_ORGANIZATION:
            raise RuntimeError, "Quotas can only be set on organizations." # FIXME
        if self.get_parent_id() <> _Conf.ROOTOBJECT:
            raise RuntimeError, "Quotas can only be set on top-level organizations." # FIXME
        self.REQUEST.db.execute("""SELECT item_id FROM items WHERE parent_id=%d AND item_type_id=%d""" % (self.get_id(), _InitTypes.ITEM_TYPE_QUOTA))
        res = self.REQUEST.db.fetchone()
        if not res:
            return None
        else:
            return dispatch_item(self.REQUEST, res[0])


    ###
    # Hooks to be overrided by children
    ###

    def BeforeInsertHook(self):
        pass
    def AfterInsertHook(self):
        pass
    def BeforeUpdateHook(self):
        pass
    def AfterUpdateHook(self):
        pass
    def BeforeDeleteHook(self):
        pass
    def AfterDeleteHook(self):
        pass

    def get_size(self):
        return 0

###
# General methods
###

def get_classtype(item_type_id):
    if item_type_id == _InitTypes.ITEM_TYPE_PERSON:
        import _Person
        classtype = _Person.Person
    elif item_type_id == _InitTypes.ITEM_TYPE_ORGANIZATION:
        import _Organization
        classtype = _Organization.Organization
    elif item_type_id == _InitTypes.ITEM_TYPE_FOLDER:
        import _Folder
        classtype = _Folder.Folder
    elif item_type_id == _InitTypes.ITEM_TYPE_DOCUMENT:
        import _Document
        classtype = _Document.Document
    elif item_type_id == _InitTypes.ITEM_TYPE_EVENT:
        import _Event
        classtype = _Event.Event
    elif item_type_id == _InitTypes.ITEM_TYPE_TODO:
        import _Todo
        classtype = _Todo.Todo
    elif item_type_id == _InitTypes.ITEM_TYPE_CONTACT:
        import _Contact
        classtype = _Contact.Contact
    elif item_type_id == _InitTypes.ITEM_TYPE_EFOLDER:
        import _Efolder
        classtype = _Efolder.Efolder
    elif item_type_id == _InitTypes.ITEM_TYPE_EMAIL:
        import _Email
        classtype = _Email.Email
    elif item_type_id == _InitTypes.ITEM_TYPE_EACCOUNT:
        import _Eaccount
        classtype = _Eaccount.Eaccount
    elif item_type_id == _InitTypes.ITEM_TYPE_DISCUSSION:
        import _Discussion
        classtype = _Discussion.Discussion
    elif item_type_id == _InitTypes.ITEM_TYPE_USERNOTE:
        import _Usernote
        classtype = _Usernote.Usernote
    elif item_type_id == _InitTypes.ITEM_TYPE_DISCCATEGORY:
        import _DiscCategory
        classtype = _DiscCategory.DiscCategory
    elif item_type_id == _InitTypes.ITEM_TYPE_RESOURCE:
        import _Resource
        classtype = _Resource.Resource
    elif item_type_id == _InitTypes.ITEM_TYPE_BOOKMARK:
        import _Bookmark
        classtype = _Bookmark.Bookmark
    elif item_type_id == _InitTypes.ITEM_TYPE_WORKFLOW:
        import _Workflow
        classtype = _Workflow.Workflow
    elif item_type_id == _InitTypes.ITEM_TYPE_EFILTER:
        import _Efilter
        classtype = _Efilter.Efilter
    elif item_type_id == _InitTypes.ITEM_TYPE_CLIENT:
        import _Client
        classtype = _Client.Client
    elif item_type_id == _InitTypes.ITEM_TYPE_OPPORTUNITY:
        import _Opportunity
        classtype = _Opportunity.Opportunity
    elif item_type_id == _InitTypes.ITEM_TYPE_PRODUCT:
        import _Product
        classtype = _Product.Product
    elif item_type_id == _InitTypes.ITEM_TYPE_QUOTA:
        import _Quota
        classtype = _Quota.Quota
    elif item_type_id == _InitTypes.ITEM_TYPE_CONTACTGROUP:
        import _ContactGroup
        classtype = _ContactGroup.ContactGroup
    elif item_type_id == _InitTypes.ITEM_TYPE_VIRTFOLDER:
        import _VirtFolder
        classtype = _VirtFolder.VirtFolder
    elif item_type_id == _InitTypes.ITEM_TYPE_MESSAGE:
        import _Message
        classtype = _Message.Message
    else:
        raise _Exceptions.RootObjectError, REQUEST.TLS('rootobj_dispatchnotype')

    return classtype

def dispatch_item(REQUEST, item_id):
    REQUEST.db.execute("SELECT item_type_id FROM items WHERE item_id=%d" % item_id)
    try:
        item_type_id = REQUEST.db.dictfetchall()[0]['item_type_id']
    except IndexError:
        raise _Exceptions.RootObjectError, "Dispatch called with invalid id %s." % item_id

    return get_classtype(item_type_id)(REQUEST, _InitTypes.INIT_TYPE_ID, item_id)

def log_undelete(REQUEST, item_id, save=0):
    LOG_SQL = u"""SELECT
 l.field_name,
 l.old_value,
 l.log_flags
FROM
 item_log l
WHERE
 l.item_id = %(item_id)d AND
 l.batch_id = ( SELECT max(l2.batch_id) FROM item_log l2 WHERE l2.item_id = %(item_id)d )
""" % locals()
    SCROLLBACK_SQL = u"""SELECT
*
FROM
 items_scrollback
WHERE
 item_id = %(item_id)d AND
 scrollback_flags=0
""" % locals()
    ITEM_SQL = u"""SELECT
item_id
FROM
 items
WHERE
 item_id = %(item_id)d
""" % locals()

    REQUEST.db.execute(ITEM_SQL.encode('UTF8'))
    if REQUEST.db.fetchone():
        raise RuntimeError, 'This item is not deleted!'

    REQUEST.db.execute(SCROLLBACK_SQL.encode('UTF8'))
    item = REQUEST.db.dictfetchone()
    if not item:
        raise RuntimeError, 'No such deleted item found in items_scrollback.'

    REQUEST.db.execute("""SELECT item_id FROM items WHERE item_id=%d""" % item['parent_id'])
    if not REQUEST.db.fetchone():
        raise RuntimeError, 'The parent object does not exist! Please restore that before trying to restore it\'s children. (%s)' % item['parent_id']

    REQUEST.db.execute(LOG_SQL.encode('UTF8'))
    log_data = REQUEST.db.fetchall()
    if not log_data or (log_data[0][2] & LOG_DELETE) == 0:
        raise RuntimeError, 'Last log entries found were not marked as delete-log!'


    item_obj = get_classtype(item['item_type_id'])(REQUEST, _InitTypes.INIT_TYPE_NEW)

    for row in log_data:
        item_obj.__itemdata__[row[0]] = eval(row[1])

    if not save:
        return item_obj
    else:
        scrollback_tid = item['conflux_tid']
        REQUEST.db.execute("""UPDATE items_scrollback SET scrollback_flags=%d WHERE item_id=%d and conflux_tid=%d""" % (SCROLLBACK_UNDELETED, item_id, item['conflux_tid']))
        item_items = {}
        object_items = {}

        for key in item_obj.__itemdata__.keys():
            if item_obj.item_fields.has_key(key):
                item = item_obj.item_fields[key]
                try:
                    if (item['null'] or (not item['null'] and item_obj.__itemdata__[key] is not None)):
                        item_items[key] = item['type'](REQUEST, item_obj.__itemdata__[key])
                except:
                    raise _Exceptions.RootObjectError, REQUEST.TLS('rootobj_typecasterr') % key
            elif item_obj.object_fields.has_key(key):
                item = item_obj.object_fields[key]
                try:
                    if (item['null'] or (not item['null'] and item_obj.__itemdata__[key] is not None)):
                        object_items[key] = item['type'](REQUEST, item_obj.__itemdata__[key])
                except:
                    raise _Exceptions.RootObjectError, REQUEST.TLS('rootobj_typecasterr') % key

        for (key, item) in item_obj.item_fields.items():
            if key not in item_items.keys() and not item['has_default'] and item['can_insert'] and not item['null']:
                raise _Exceptions.RootObjectError, REQUEST.TLS('rootobj_missingnodefault') % key
        for (key, item) in item_obj.object_fields.items():
            if key not in object_items.keys() and not item['has_default'] and item['can_insert'] and not item['null']:
                raise _Exceptions.RootObjectError, REQUEST.TLS('rootobj_missingnodefault') % key

        # INSERT
        QUERY = []

        ns = {}
        ns['insert_keys'] = ','.join(item_items.keys())
        ns['insert_values'] = ','.join(map(unicode, item_items.values()))
        QUERY.append(item_obj.SQL_ITEM_INSERT % ns)

        ns['insert_keys'] = ','.join(object_items.keys())
        ns['insert_values'] = ','.join(map(unicode, object_items.values()))
        ns['table_name'] = item_obj.table_name
        QUERY.append(item_obj.SQL_OBJECT_INSERT % ns)

        QUERY = u";\n".join(QUERY).encode('UTF8')
        try:
            REQUEST.db.execute(QUERY)
        except _Exceptions.RootObjectError, message:
            raise _Exceptions.RootObjectError, message

        item_obj.__itemdata__ = {}
        item_obj.__olddata__ = {}
        for (key, value) in item_obj.item_fields.items():
            item_obj.__itemdata__[key] = None
            item_obj.__olddata__[key]  = None
        for (key, value) in item_obj.object_fields.items():
            if key in item_obj.__itemdata__.keys():
                raise _Exceptions.RootObjectError, REQUEST.TLS('rootobj_matchingkeys')
            item_obj.__itemdata__[key] = None
            item_obj.__olddata__[key]  = None
        item_obj.load_db(item_id)
        item_obj.log_insert()
        if item_obj.undelete_hook: # documents, folders, orgs for now
            ns = {'item_id':item_id, 'conflux_tid': scrollback_tid}
            item_obj.REQUEST.db.execute(item_obj.SQL_SELECT_DELETED_TIME % ns)
            item_obj.set('delete_time', item_obj.REQUEST.db.dictfetchall()[0]['delete_time'])
            item_obj.set('oldpath', item_obj.get_path())
            item_obj.undelete_hook()
        return item_obj

