# Copyright (C) 2003-2005 Inversion Software Ltd. <inversion@inversion.ee>
# http://www.conflux.ee/
#
# This file is part of Conflux Lite
#
# Conflux Lite is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2,
# or (at your option) any later version.
#
# Conflux Lite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Conflux Lite; if not, write to the Free Software
# Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
######################################################################

class UserGroup:
    SQL_SELECT = """\
SELECT
 ug.*,
 o.*,
 o2.organization_name AS top_org_name
FROM
 user_groups ug
LEFT JOIN organizations o ON ug.group_org_id = o.organization_id
LEFT JOIN organizations o2 ON ug.top_org_id = o2.organization_id
WHERE ug.group_id=%(group_id)s"""

    SQL_SELECT_PERSONS = """\
SELECT
 ag.person_id,
 ag.group_id,
 p.first_name,
 p.last_name,
 p.middle_name,
 p.username
FROM
 access_groups ag
LEFT JOIN persons p ON ag.person_id=p.person_id
WHERE
 ag.group_id=%(group_id)s"""

    SQL_SELECT_RIGHTS = """\
SELECT
 ar.*,
 i.item_name
FROM
 access_rights ar
LEFT JOIN items i ON ar.item_id=i.item_id
WHERE
 ar.group_id=%(group_id)s"""

    SQL_ADD_PERSON = """\
INSERT INTO
 access_groups (
group_id,
person_id )
VALUES (
%(group_id)d,
%(person_id)d )"""

    SQL_DEL_PERSON = """\
DELETE FROM
 access_groups
WHERE
 group_id=%(group_id)d AND
 person_id=%(person_id)d"""

    def __init__(self, REQUEST, group_id):
        self.group_id = group_id
        self.REQUEST = REQUEST
        self.load_db()

    def load_db(self): # FIXME error management
        group_id = self.group_id
        self.REQUEST.db.execute(self.SQL_SELECT % locals())
        self.data = self.REQUEST.db.dictfetchall()[0]
        self.persons = None
        self.rights = None

    def load_persons(self):
        group_id = self.group_id
        self.REQUEST.db.execute(self.SQL_SELECT_PERSONS % locals())
        self.persons = self.REQUEST.db.dictfetchall()

    def load_rights(self):
        group_id = self.group_id
        self.REQUEST.db.execute(self.SQL_SELECT_RIGHTS % locals())
        self.rights = self.REQUEST.db.dictfetchall()

    def add_person(self, person_id):
        group_id = self.group_id
        self.REQUEST.db.execute(self.SQL_ADD_PERSON % locals())
        self.load_persons()

    def del_person(self, person_id):
        group_id = self.group_id
        self.REQUEST.db.execute(self.SQL_DEL_PERSON % locals())
        self.load_persons()

