# Copyright (C) 2003-2005 Inversion Software Ltd. <inversion@inversion.ee>
# http://www.conflux.ee/
#
# This file is part of Conflux Lite
#
# Conflux Lite is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2,
# or (at your option) any later version.
#
# Conflux Lite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Conflux Lite; if not, write to the Free Software
# Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
######################################################################

import _Exceptions
from cUtils import new_session_key, get_salt, init_random
import _Conf
import time, mx.DateTime, datetime
import sgmllib
import xml.dom.minidom
import _DBSpool_utils

###
# Randomize
###

INITIALIZED = 0
if not INITIALIZED:
    init_random()
INITIALIZED = 1

###
# Class for html stripping
###

class StrippingParser(sgmllib.SGMLParser):
    "From Python CookBook"

    # These are the HTML tags that we will leave intact
    # valid_tags = ('b', 'a', 'i', 'br', 'p')

    from htmlentitydefs import entitydefs # replace entitydefs from sgmllib
    
    def __init__(self):
        sgmllib.SGMLParser.__init__(self)
        self.result = ""
        self.endTagList = [] 
        
    def handle_data(self, data):
        if data:
            self.result = self.result + data

    def handle_charref(self, name):
        self.result = "%s&#%s;" % (self.result, name)
        
    def handle_entityref(self, name):
        if self.entitydefs.has_key(name): 
            x = ';'
        else:
            # this breaks unstandard entities that end with ';'
            x = ''
        self.result = "%s&%s%s" % (self.result, name, x)
    
    def unknown_starttag(self, tag, attrs):
        """ Delete all tags except for legal ones """
        if tag in self.valid_tags:       
            self.result = self.result + '<' + tag
            for k, v in attrs:
                if k[0:2].lower() != 'on' and v[0:10].lower() != 'javascript':
                    self.result = '%s %s="%s"' % (self.result, k, v)
            endTag = '</%s>' % tag
            self.endTagList.insert(0,endTag)    
            self.result = self.result + '>'
                
    def unknown_endtag(self, tag):
        if tag in self.valid_tags:
            self.result = "%s</%s>" % (self.result, tag)
            remTag = '</%s>' % tag
            self.endTagList.remove(remTag)

    def cleanup(self):
        """ Append missing closing tags """
        for j in range(len(self.endTagList)):
                self.result = self.result + self.endTagList[j]    
        
def strip_html(s, valid_tags=('b', 'a', 'i', 'br', 'p')):
    """ Strip illegal HTML tags from string s """
    try:
        parser = StrippingParser()
        parser.valid_tags = valid_tags
        parser.feed(s)
        parser.close()
        parser.cleanup()
        return parser.result
    except: # FIXME, should do something smarter here
        return s

def strip_all_html(s):
    try:
        parser = StrippingParser()
        parser.valid_tags = ()
        parser.feed(s)
        parser.close()
        parser.cleanup()
        return parser.result
    except: # FIXME, should do something smarter here
        raise



###
# Utility functions
###


def sql_quote(text):
    if isinstance(text, unicode):
        return u"'%s'" % text.replace("\\", "\\\\").replace("'", "''")
    elif isinstance(text, str):
        return "'%s'" % text.replace("\\", "\\\\").replace("'", "''")
    else:
#        raise RuntimeError, str(text)
        return "'%s'" % str(text).replace("\\", "\\\\").replace("'", "''")

def sql_quote_simple(text):
    return text.replace("\\", "\\\\").replace("'", "''")

def crypt(word, salt):
    import crypt
    return crypt.crypt(word, salt)

###
# SQL
###

def sql_cast_bool(REQUEST, var):
    if var is None: return 'null'
    try:
        if isinstance(var, int):
            if var == 0:
                return u'false'
            elif var == 1:
                return u'true'
            else:
                raise _Exceptions.InvalidVariable, REQUEST.TLS('utils_invalidboolean')
        elif var.lower() in ['true', 'false']:
            return var.lower()
        else:
            raise _Exceptions.InvalidVariable, REQUEST.TLS('utils_invalidboolean')
    except:
        raise _Exceptions.InvalidVariable, REQUEST.TLS('utils_invalidboolean')

def sql_cast_int(REQUEST, var):
    if var is None: return u'null'
    if isinstance(var, int):
        return var
    elif isinstance(var, str) or isinstance(var, unicode):
        try:
            return int(var)
        except:
            raise _Exceptions.InvalidVariable, REQUEST.TLS('utils_invalidint')
    else:
        raise _Exceptions.InvalidVariable, REQUEST.TLS('utils_invalidint')

def sql_cast_long(REQUEST, var):
    if var is None: return u'null'
    if isinstance(var, int) or isinstance(var, long):
        return var
    elif isinstance(var, str) or isinstance(var, unicode):
        try:
            return long(var)
        except:
            raise _Exceptions.InvalidVariable, REQUEST.TLS('utils_invalidint')
    else:
        raise _Exceptions.InvalidVariable, REQUEST.TLS('utils_invalidint')

def sql_cast_float(REQUEST, var):
    if var is None: return u'null'
    if isinstance(var, float):
        return var
    elif isinstance(var, str) or isinstance(var, unicode):
        if not var.strip(): return u'null'
        try:
            return float(var)
        except:
            raise _Exceptions.InvalidVariable, REQUEST.TLS('utils_invalidint')
    else:
        raise _Exceptions.InvalidVariable, REQUEST.TLS('utils_invalidint')

def sql_cast_array(REQUEST, var):
    if var is None: return u'null'
    if isinstance(var, list):
        return "'{%s}'::int[]" % ",".join(map(str, var))
    elif isinstance(var, str) or isinstance(var, unicode) and var[0] == '{' and var[-1] == '}':
        return "'%s'::int[]" % var
    else:
        raise _Exceptions.InvalidVariable, REQUEST.TLS('utils_invalidarray')

def sql_cast_str(REQUEST, var):
    if var is None: return 'null'
    return sql_quote(var)

def sql_cast_str_hidenull(REQUEST, var):
    if var is None: var = ''
    return sql_quote(var)

def sql_cast_bit(REQUEST, var):
    if var is None: return 'null'
    return "B%s" % sql_quote(var)

def sql_cast_interval(REQUEST, var):
    try:
        if var is None or var == 'null' or var is 'None': return 'null'
        if hasattr(var, 'value') and (var.value is None or var.value == 'None'): return 'null'
        return sql_cast_str(REQUEST, var)
    except:
        raise

def tuple2isostr(dt):
    return u"%04d-%02d-%02d %02d:%02d" % (dt[0], dt[1], dt[2], dt[3], dt[4])

def isostr2tuple(dt):
    date_s, time_s = dt.split(' ')
    year, month, day = map(int, date_s.split('-'))
    hour, minute, rest = time_s.split(':')
    hour = int(hour)
    minute = int(minute)
    return (year, month, day, hour, minute, 0, 0, 0, 0)

def datetime2tuple(dt):
    return dt.tuple()[:-1] + (time.localtime()[8],)

def tuple2datetime(dt):
    return mx.DateTime.DateTime(dt[0], dt[1], dt[2], dt[3], dt[4], dt[5])

def shorten_string(input, length):
    if len(input) <= length:
        return input
    else:
        return input[:length-3] + ".."

def make_path(path):
    return path.replace(' ','_').replace('/', '_').replace('\\', '_').replace('%', '_')

def str_size(size):
    if size >= 1073741824:
        return "%.2fGB" % (size/1073741824.0)
    elif size >= 1048576:
        return "%.1fMB" % (size/1048576.0)
    elif size > 1024:
        return "%.0fKB" % (size/1024.0)
    else:
        return "%dB" % size

def sql_parse_array(REQUEST, var):
    if var is None: return u'null'
    if (isinstance(var, unicode) or isinstance(var, str)) and var[0] == '{' and var[-1] == '}':
        if len(var) == 2:
            return []
        return map(int, var[1:-1].split(','))
    else:
        raise _Exceptions.InvalidVariable, REQUEST.TLS('utils_invalidarray')

def newline2br(var):
    return var.replace("\r","").replace("\n", "<BR>")

def html_quote(var):
    var = var.replace("&", "&amp;")
    var = var.replace("<", "&lt;")
    var = var.replace(">", "&gt;")
    var = var.replace('"', "&quot;")
    var = var.replace("'", "&#39;")
    return var

def html_unquote(var):
    var = var.replace("&lt;", "<")
    var = var.replace("&gt;", ">")
    var = var.replace("&quot;", '"')
    var = var.replace("&#39;", "'")
    var = var.replace("&amp;", "&")
    return var

def html_quote_show(var):
    if not isinstance(var, str) and not isinstance(var, unicode):
      return var
    var = var.replace("<", "&lt;")
    var = var.replace(">", "&gt;")
    return var

def html_quote_edit(var):
    if not isinstance(var, str) and not isinstance(var, unicode):
      return var
    if var is None:
      return ""
    var = var.replace('"', "&quot;")
    var = var.replace("'", "&#39;")
    return var

def make_setting_selection(REQUEST, object, fieldname, settingname, is_multiple=None, read_only=0, selected="", style="textfield1"):
    if object is None and read_only == 0:
        try:
            values = REQUEST.get_setting(settingname)
            if is_multiple:
                value_output = [u"""<SELECT class="%s" name="%s" multiple size="4">""" % (style, fieldname)]
            else:
                value_output = [u"""<SELECT class="%s" name="%s">""" % (style, fieldname)]
                value_output.append(u"""<OPTION value="">&nbsp;&nbsp;&nbsp;&nbsp;""")
            for value in values:
                if value==selected:
                    value_output.append(u"""<OPTION SELECTED value="%s">%s""" % (value, value))
                else:
                    value_output.append(u"""<OPTION value="%s">%s""" % (value, value))
            value_output.append(u"""</SELECT>""")
            values = "\n".join(value_output)
            return values
        except:
            values = u"""<SELECT class="%s" name="%s">\n<OPTION value="">&nbsp;&nbsp;&nbsp;&nbsp;\n</SELECT>""" % (style, fieldname)
    elif object is None:
        return u""

    try:
        if is_multiple:
            values = REQUEST.get_setting(settingname)
            obj_values = map(lambda x: x.replace(u'|',u''), object.get(fieldname).split(u'|'))
            for obj_value in obj_values:
                if obj_value and obj_value not in values:
                    values.append(obj_value)

            if read_only:
                value_output = []
            else:
                value_output = [u"""<SELECT class="%s" name="%s" multiple size="5">""" % (style, fieldname)]
            for value in values:
                if not read_only:
                    if value in obj_values:
                        value_output.append(u"""<OPTION value="%s" SELECTED>%s""" % (value, value))
                    else:
                        value_output.append("""<OPTION value="%s">%s""" % (value, value))
                elif read_only and value in obj_values:
                    value_output.append(value)
        else:
            values = REQUEST.get_setting(settingname)
            obj_value = object.get(fieldname)
            if obj_value and obj_value not in values:
                values.append(obj_value)

            if read_only:
                value_output = []
            else:
                value_output = [u"""<SELECT class="%s" name="%s">""" % (style, fieldname)]
                value_output.append(u"""<OPTION value="">&nbsp;&nbsp;&nbsp;&nbsp;""")
            for value in values:
                if not read_only:
                    if value == obj_value:
                        value_output.append(u"""<OPTION value="%s" SELECTED>%s""" % (value, value))
                    else:
                        value_output.append("""<OPTION value="%s">%s""" % (value, value))
                elif read_only and value == obj_value:
                    value_output.append(value)
                    
        if read_only:
            values = "<BR>".join(value_output)
        else:
            value_output.append(u"""</SELECT>""")
            values = "\n".join(value_output)
    except:
        if object and object.get(fieldname) and read_only:
            values = object.get(fieldname)
            if not values:
                values = u''
            elif values and values.find('|') <> -1:
                values = "<BR>".join(values[1:-1].split('|'))
        elif read_only:
            values = u""
        elif object and object.get(fieldname):
            values = u"""<SELECT class="%s" name="%s">\n<OPTION value="">&nbsp;&nbsp;&nbsp;&nbsp;\n<OPTION value="%s">%s\n</SELECT>""" % (style, fieldname, object.get(fieldname), object.get(fieldname))
        else:
            values = u"""<SELECT class="%s" name="%s">\n<OPTION value="">&nbsp;&nbsp;&nbsp;&nbsp;\n</SELECT>""" % (style, fieldname)
    return values

def split_text(value, maxval=12):
    spacecnt = 0
    output = []
    for charval in value:
        spacecnt += 1
        output.append(charval)
        if charval.isspace():
            spacecnt = 0
        elif spacecnt >= maxval:
            spacecnt = 1
            output.append("- ")
    return "".join(output)

def split_text_plain(value, maxval=70):
    value = value.replace('\r','')
    output = []
    thisrow = ""
    thisword= ""
    for charval in value:
        if charval =='\n':
            if len(thisrow+thisword+charval) > maxval:
                output.append(thisrow + '\n')
                output.append(thisword+ '\n')
            else:
                output.append(thisrow+thisword+'\n')
            thisrow = ""
            thisword = ""
        elif charval.isspace():
            if len(thisrow+thisword+charval) > maxval:
                output.append(thisrow + '\n')
                thisrow = thisword + charval
                thisword = ""
            else:
                thisrow = thisrow + thisword + charval
                thisword = ""
        else:
            thisword += charval
    if len(thisrow+thisword+charval) > maxval:
        output.append(thisrow + '\n')
        output.append(thisword+ '\n')
    else:
        output.append(thisrow+thisword+'\n')
    return output


def get_epoz_field(REQUEST, fieldname, data, style=""):
    EPOZ_JS_PATH = _Conf.BASE_EPOZ_URL
    EPOZ_IMG_PATH = _Conf.BASE_EPOZ_IMG_URL

    js_data = data.replace('\r','').replace('\n','\\n').replace("'","\\'")

    return u"""\
<iframe id="EpozIFrame" name="EpozIFrame" style="position: absolute; visibility: hidden; width: 0px; height: 0px;"></iframe>
<script language="JavaScript" type="text/javascript" src="%(EPOZ_JS_PATH)sepoz_lang_en.js"></script>
<script language="JavaScript" type="text/javascript" src="%(EPOZ_JS_PATH)sepoz_script_widget.js"></script>
<script language="JavaScript" type="text/javascript" src="%(EPOZ_JS_PATH)sepoz_script_detect.js"></script>
<script language="JavaScript" type="text/javascript" src="%(EPOZ_JS_PATH)sepoz_script_main.js"></script>
<script language="JavaScript" type="text/javascript">
<!--
InitEpoz('%(fieldname)s','%(js_data)s','%(EPOZ_IMG_PATH)s','FIXME','%(style)s','background-color: #EFEFEF; border: 1px solid #A0A0A0; cursor: pointer; cursor: hand; margin-right: 1px; margin-bottom: 1px;');
//-->
</script>
<noscript><textarea name="%(fieldname)s" style="width: 560px; height: 250px; border: 1px solid #000000;">%(data)s</textarea></noscript>
""" % locals()

def mx2dt(date):
    if not isinstance(date, datetime.datetime):
        return datetime.datetime(date.year, date.month, date.day, date.hour, date.minute, int(date.second))
    return date

def dt2mx(date):
    now = mx.DateTime.now()
    if not isinstance(date, type(now)):
        return mx.DateTime.DateTime(date.year, date.month, date.day, date.hour, date.minute, int(date.second))
    return date

def utc2local_mx(REQUEST, date):
    if not REQUEST.tz:
        return date
    else:
        date_utc = mx2dt(date)
        offset = datetime.timedelta(minutes=_Conf.TIMEZONES_D[REQUEST.tz][4])
        date = date_utc + offset
        return dt2mx(date)

def local2utc_mx(REQUEST, date):
    if not REQUEST.tz:
        return date
    else:
        date_utc = mx2dt(date)
        offset = datetime.timedelta(minutes=_Conf.TIMEZONES_D[REQUEST.tz][4])
        date = date_utc - offset
        return dt2mx(date)

def dom2dict(domtree):
    tree = {'text':[]}
    for node in domtree.childNodes:
        if node.nodeName == '#text':
            tree['text'].append(node.nodeValue)
        elif node.nodeName:
            try:
                tree[node.nodeName].append(dom2dict(node))
            except:
                tree[node.nodeName] = [dom2dict(node)]
    return tree

def xml2dict(xmlstr):
    domtree = xml.dom.minidom.parseString(xmlstr)
    tree = dom2dict(domtree)
    domtree.unlink()
    return tree

def utc_now_mx():
    d = time.gmtime()
    return mx.DateTime.DateTime(d[0], d[1], d[2], d[3], d[4], d[5])

def parse_csv(data):
    """
  Input: unicode string
  Output: list of lists (rows and columns)
    """
    data = data.replace('\r', '')
    COLS = 18
    data_len = len(data)
    result = []
    line = []
    col = ""

    in_tag = 0

    i = 0 # char in text
    j = 0 # col in line
    while 1:
        if i >= data_len:
            break
        if in_tag and data[i] == u'"' and (i+1) < data_len and data[i+1] == u'"':
            col += u'"'
            i += 2
            continue
        elif not in_tag and data[i] == u',':
            line.append(col)
            col = ""
            i += 1
            j += 1
# Should never happen
#            if j == COLS:
#                result.append(line)
#                line = []
#                j = 0
            continue
        elif not in_tag and data[i] == u'\n':
            line.append(col)
            col = ""
            i += 1
            result.append(line)
            line = []
            j = 0
            continue
        elif in_tag and data[i] == u'"':
            i += 1
            in_tag = 0
            continue
        elif not in_tag and data[i] == u'"':
            i += 1
            in_tag = 1
            continue
        else:
            col += data[i]
            i += 1
            continue
    if col:
        line.append(col)
    if line:
        result.append(line)
    return result

def get_module_access(REQUEST):
    try:
        access_str = REQUEST._settings.get_setting(REQUEST, 'module_access') or _Conf.MODULE_DEFAULT
    except:
        access_str = _Conf.MODULE_DEFAULT

    if len(access_str) < len(_Conf.MENU):
        access_str = access_str + "1"*(_Conf.MODULES_TOTAL_COUNT-len(access_str))

    access_str = map(lambda x:x, access_str)
    for i in range(len(_Conf.MENU)):
        if not _Conf.MENU[i][_Conf.MENU_ENABLED]:
            access_str[_Conf.MENU[i][2]] = "0"
    access_str = "".join(access_str)
    return access_str


def get_module_access_org(REQUEST, org_id):
    try:
        access_str = REQUEST._settings.get_setting_org_any(REQUEST, org_id, 'module_access') or _Conf.MODULE_DEFAULT
    except:
        access_str = _Conf.MODULE_DEFAULT

    if len(access_str) < len(_Conf.MENU):
        access_str = access_str + "1"*(len(_Conf.MENU)-len(access_str))

    access_str = map(lambda x:x, access_str)
    for i in range(len(_Conf.MENU)):
        if not _Conf.MENU[i][_Conf.MENU_ENABLED]:
            access_str[_Conf.MENU[i][2]] = "0"
    access_str = "".join(access_str)
    return access_str


def date2text(REQUEST, utc_date):
    usernow  = utc2local_mx(REQUEST, utc_now_mx())
    userdate = utc2local_mx(REQUEST, utc_date)
    if usernow.year==userdate.year and usernow.month == userdate.month and usernow.day == userdate.day:
        date = REQUEST.iso2usertime(REQUEST, utc_date)
    elif usernow.year==userdate.year:
        date = REQUEST.date2usertext(REQUEST, userdate.day, userdate.month)
    else:
        date = REQUEST.iso2userdatetime(REQUEST, utc_date).split(" ")[0]
    return date

def dateformat2jscalformat(REQUEST):
    dateformat = REQUEST._translator.dateformat
    if dateformat=="estonian":
        return "%d.%m.%Y"
    elif dateformat=="en_US" or dateformat=="en_US_12":
        return "%m/%d/%Y"
    else:
        return "%Y-%m-%d"

def lang2jscalformat(REQUEST):
    lang = REQUEST._translator.language
    if lang == "estonian":
        return "ee"
    elif lang == "pt_PT":
        return "pt"
    else:
        return "en"
