# Copyright (C) 2003-2005 Inversion Software Ltd. <inversion@inversion.ee>
# http://www.conflux.ee/
#
# This file is part of Conflux Lite
#
# Conflux Lite is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2,
# or (at your option) any later version.
#
# Conflux Lite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Conflux Lite; if not, write to the Free Software
# Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
######################################################################

import time, calendar, mx.DateTime, md5, os, sys, socket
import _Exceptions, _Access, _InitTypes, _Utils, _Event, _Conf

def localsort(a, b):
    if a['start_stamp'] < b['start_stamp']: return -1
    elif a['start_stamp'] > b['start_stamp']: return 1
    else:
        if a['end_stamp'] < b['end_stamp']: return -1
        elif a['end_stamp'] > b['end_stamp']: return 1
        else: return 0

class Day:
    def __init__(self, day, month, year):
        self.events = [
        (7,00,[]),
        (8,00,[]),
        (9,00,[]),
        (10,00,[]),
        (11,00,[]),
        (12,00,[]),
        (13,00,[]),
        (14,00,[]),
        (15,00,[]),
        (16,00,[]),
        (17,00,[]),
        (18,00,[]),
        (19,00,[]),
        (20,00,[]),
        (21,00,[]),
        (22,00,[])]

        self.name = None
        self.day = day
        self.month = month
        self.year = year

    def addevent(self, event, dts):
        for time in self.events:
            thistime = (self.year, self.month, self.day, time[0],time[1], 0, 0, 0, 0)
            try:
                for x in range(4):
                    if x == 3 and ((dts[x] > 22 and thistime[x] == 22) or (dts[x] < 7 and thistime[x] == 7)):
                        break
                    elif dts[x] <> thistime[x]:
                        raise RuntimeError, 'Please continue'
            except RuntimeError:
                continue

            if event not in time[2]:
                time[2].append(event)

    def sort(self):
        for time in self.events:
            time[2].sort(localsort)

class MonthHandler:
    def __init__(self, REQUEST, year=None, month=None, day=None, show_users=None, show_resources=None):
        self.mapped_days = {}
        self.show_users = show_users
        self.show_resources = show_resources
        if year is None or month is None or day is None:
            current_stamp = _Utils.utc2local_mx(REQUEST, _Utils.utc_now_mx())
            self.year = current_stamp.year
            self.month = current_stamp.month
            self.day = current_stamp.day
        else:
            self.year = year
            self.month = month
            self.day = day
        self.calendar = calendar.monthcalendar(self.year, self.month)
        if self.month == 1:
            self.prev = calendar.monthcalendar(self.year-1, 12)
            self.next = calendar.monthcalendar(self.year, self.month+1)
        elif self.month == 12:
            self.prev = calendar.monthcalendar(self.year, self.month-1)
            self.next = calendar.monthcalendar(self.year+1, 1)
        elif self.month > 1 and self.month < 12:
            self.prev = calendar.monthcalendar(self.year, self.month-1)
            self.next = calendar.monthcalendar(self.year, self.month+1)
        
        self.events_calendar = []
        ndx = 0
        for week in self.calendar:
            self.events_calendar.append([])
            for i in week:
                self.events_calendar[ndx].append(Day(i, self.month, self.year))
            ndx = ndx + 1

        found = 0
        for week in self.events_calendar:
            for i in range(len(week)):
                if week[i].day == 0 and found:
                    if self.month == 12:
                        week[i] = Day(self.next[0][i], 1, self.year+1)
                    else:
                        week[i] = Day(self.next[0][i], self.month+1, self.year)
                elif week[i].day == 0:
                    if self.month == 1:
                        week[i] = Day(self.prev[-1][i], 12, self.year-1)
                    else:
                        week[i] = Day(self.prev[-1][i], self.month-1, self.year)
                else:
                    found = 1
                if week[i].day in [11,12,13]:
                    week[i].name = "%d%s" % (week[i].day, REQUEST.TLS('day_end_th'))
                elif (week[i].day) % 10 == 1:
                    week[i].name = "%d%s" % (week[i].day, REQUEST.TLS('day_end_st'))
                elif (week[i].day) % 10 == 2:
                    week[i].name = "%d%s" % (week[i].day, REQUEST.TLS('day_end_nd'))
                elif (week[i].day) % 10 == 3:
                    week[i].name = "%d%s" % (week[i].day, REQUEST.TLS('day_end_rd'))
                else:
                    week[i].name = "%d%s" % (week[i].day, REQUEST.TLS('day_end_th'))
        for week in self.events_calendar:
            for day in week:
                self.mapped_days[(day.year, day.month, day.day)] = day

    def load(self, REQUEST):
        for week in self.events_calendar:
            for day in week:
                dayhandler = DayHandler(REQUEST, day.year, day.month, day.day, self.show_users, self.show_resources)
                dayhandler.load(REQUEST)
                day.events = dayhandler.active_week[0].events

class WeekHandler:
    def __init__(self, REQUEST, year=None, month=None, day=None, show_users=None, show_resources=None):
        self.mapped_days = {}
        self.show_users = show_users
        self.show_resources = show_resources
        if year is None or month is None or day is None:
            current_stamp = _Utils.utc2local_mx(REQUEST, _Utils.utc_now_mx())
            self.year = current_stamp.year
            self.month = current_stamp.month
            self.day = current_stamp.day
        else:
            self.year = year
            self.month = month
            self.day = day
        self.calendar = calendar.monthcalendar(self.year, self.month)
        if self.month == 1:
            self.prev = calendar.monthcalendar(self.year-1, 12)
            self.next = calendar.monthcalendar(self.year, self.month+1)
        elif self.month == 12:
            self.prev = calendar.monthcalendar(self.year, self.month-1)
            self.next = calendar.monthcalendar(self.year+1, 1)
        elif self.month > 1 and self.month < 12:
            self.prev = calendar.monthcalendar(self.year, self.month-1)
            self.next = calendar.monthcalendar(self.year, self.month+1)
        
        self.active_week = []
        for week in self.calendar:
            if self.day in week:
                for i in week:
                    self.active_week.append(Day(i, self.month, self.year))

        if not self.active_week:
            raise _Exceptions.CalendarError, "Provided calendar did not include a week with today."

        found = 0
        for i in range(len(self.active_week)):
            if self.active_week[i].day == 0 and found:
                if self.month == 12:
                    self.active_week[i] = Day(self.next[0][i], 1, self.year+1)
                else:
                    self.active_week[i] = Day(self.next[0][i], self.month+1, self.year)
            elif self.active_week[i].day == 0:
                if self.month == 1:
                    self.active_week[i] = Day(self.prev[-1][i], 12, self.year-1)
                else:
                    self.active_week[i] = Day(self.prev[-1][i], self.month-1, self.year)
            else:
                found = 1
            if self.active_week[i].day in [11,12,13]:
                self.active_week[i].name = "%d%s" % (self.active_week[i].day, REQUEST.TLS('day_end_th'))
            elif (self.active_week[i].day) % 10 == 1:
                self.active_week[i].name = "%d%s" % (self.active_week[i].day, REQUEST.TLS('day_end_st'))
            elif (self.active_week[i].day) % 10 == 2:
                self.active_week[i].name = "%d%s" % (self.active_week[i].day, REQUEST.TLS('day_end_nd'))
            elif (self.active_week[i].day) % 10 == 3:
                self.active_week[i].name = "%d%s" % (self.active_week[i].day, REQUEST.TLS('day_end_rd'))
            else:
                self.active_week[i].name = "%d%s" % (self.active_week[i].day, REQUEST.TLS('day_end_th'))

        for day in self.active_week:
            self.mapped_days[(day.year, day.month, day.day)] = day

    def load(self, REQUEST):
        for day in self.active_week:
            dayhandler = DayHandler(REQUEST, day.year, day.month, day.day, self.show_users, self.show_resources)
            dayhandler.load(REQUEST)
            day.events = dayhandler.active_week[0].events

class DayHandler:
    def __init__(self, REQUEST, year=None, month=None, day=None, show_users=None, show_resources=None):
        self.mapped_days = {}
        self.show_users = show_users
        self.show_resources = show_resources
        if year is None or month is None or day is None:
            current_stamp = _Utils.utc2local_mx(REQUEST, _Utils.utc_now_mx())
            self.year = current_stamp.year
            self.month = current_stamp.month
            self.day = current_stamp.day
        else:
            self.year = year
            self.month = month
            self.day = day
        self.calendar = calendar.monthcalendar(self.year, self.month)
        if self.month == 1:
            self.prev = calendar.monthcalendar(self.year-1, 12)
            self.next = calendar.monthcalendar(self.year, self.month+1)
        elif self.month == 12:
            self.prev = calendar.monthcalendar(self.year, self.month-1)
            self.next = calendar.monthcalendar(self.year+1, 1)
        else:
            self.prev = calendar.monthcalendar(self.year, self.month-1)
            self.next = calendar.monthcalendar(self.year, self.month+1)
        
        self.active_week = []
        if self.day <= calendar.monthrange(self.year, self.month)[1]:
            self.active_week.append(Day(self.day, self.month, self.year))
        else:
            raise _Exceptions.CalendarError, "Invalid date provided." # FIXME

        i = 0
        if self.active_week[i].day in [11,12,13]:
            self.active_week[i].name = "%d%s" % (self.active_week[i].day, REQUEST.TLS('day_end_th'))
        elif (self.active_week[i].day) % 10 == 1:
            self.active_week[i].name = "%d%s" % (self.active_week[i].day, REQUEST.TLS('day_end_st'))
        elif (self.active_week[i].day) % 10 == 2:
            self.active_week[i].name = "%d%s" % (self.active_week[i].day, REQUEST.TLS('day_end_nd'))
        elif (self.active_week[i].day) % 10 == 3:
            self.active_week[i].name = "%d%s" % (self.active_week[i].day, REQUEST.TLS('day_end_rd'))
        else:
            self.active_week[i].name = "%d%s" % (self.active_week[i].day, REQUEST.TLS('day_end_th'))

        for day in self.active_week:
            self.mapped_days[(day.year, day.month, day.day)] = day

    def load(self, REQUEST, all_users = 0):
        start = (self.active_week[0].year, self.active_week[0].month, self.active_week[0].day, 0, 0, 0, 0, 0, time.localtime()[8])
        events = self.get_calendar_events(REQUEST, start, 0, self.show_users, self.show_resources, all_users)

        for event in events:
            set = _Utils.datetime2tuple(event['start_stamp'])
            if set[0] < self.active_week[0].year or (set[0] == self.active_week[0].year and set[1] < self.active_week[0].month) or (set[0] == self.active_week[0].year and set[1] == self.active_week[0].month and set[2] < self.active_week[0].day): # If event started earlier, make it look like it started on the morning
                event['start_stamp'] = mx.DateTime.DateTime(self.active_week[0].year, self.active_week[0].month, self.active_week[0].day, 7)
                set = _Utils.datetime2tuple(event['start_stamp'])
            self.mapped_days[(set[0], set[1], set[2])].addevent(event, set)
        for day in self.active_week:
            day.sort()

    def get_calendar_events(self, REQUEST, start, days, show_users, show_resources, all_users = 0):
        "Requires datetime tuples."
        start = (start[0], start[1], start[2], 0, 0, 0, start[6], start[7], start[8])
        start_dt = _Utils.tuple2datetime(start)
        start_dt = _Utils.local2utc_mx(REQUEST, start_dt)
        start = _Utils.datetime2tuple(start_dt)
        end_dt = _Utils.tuple2datetime(start) + mx.DateTime.DateTimeDelta(1)
        end = _Utils.datetime2tuple(end_dt)
        end = _Utils.sql_cast_str(REQUEST, _Utils.tuple2isostr(end))
        start = _Utils.sql_cast_str(REQUEST, _Utils.tuple2isostr(start))
        right_param = _Access.select_param(REQUEST.USER.get_id(), _Access.ACCESS_READ, [_InitTypes.ITEM_TYPE_EVENT])
        if not all_users:
            if show_users: # Show involved people
                users_limit = []
                for userid in show_users:
                    users_limit.append("""i.creator_id = %d""" % userid)
                    if REQUEST.db.version < 740:
                        users_limit.append("""e.related_persons @ '{%d}'::int[]""" % userid)
                        REQUEST.db.execute("""SELECT group_id FROM calendar_groups WHERE members @ '{%d}'::int[]""" % userid)
                    else:
                        users_limit.append("""%d=ANY(e.related_persons)""" % userid)
                        REQUEST.db.execute("""SELECT group_id FROM calendar_groups WHERE %d=ANY(members)""" % userid)
                    groups = REQUEST.db.dictfetchall()
                    if REQUEST.db.version < 740:
                        for i in groups:
                            users_limit.append("""e.related_groups @ '{%d}'::int[]""" % i['group_id'])
                    else:
                        for i in groups:
                            users_limit.append("""%d=ANY(e.related_groups)""" % i['group_id'])
                users_limit = "\n     (%s)" % " OR ".join(users_limit)
            else: # show only personal stuff
                if REQUEST.db.version < 740:
                    REQUEST.db.execute("""SELECT group_id FROM calendar_groups WHERE members @ '{%d}'::int[]""" % REQUEST.USER.get_id())
                else:
                    REQUEST.db.execute("""SELECT group_id FROM calendar_groups WHERE %d=ANY(members)""" % REQUEST.USER.get_id())
                groups = REQUEST.db.dictfetchall()
                if groups:
                    g = []
                    if REQUEST.db.version < 740:
                        for i in groups:
                            g.append("""e.related_groups @ '{%d}'::int[]""" % i['group_id'])
                    else:
                        for i in groups:
                            g.append("""%d=ANY(e.related_groups)""" % i['group_id'])
                    groups = " OR %s" % " OR ".join(g)
                else:
                    groups = ""
                if REQUEST.db.version < 740:
                    users_limit = """\n     (i.creator_id = %d OR e.related_persons @ '{%d}'::int[]%s)""" % (REQUEST.USER.get_id(), REQUEST.USER.get_id(), groups)
                else:
                    users_limit = """\n     (i.creator_id = %d OR %d=ANY(e.related_persons)%s)""" % (REQUEST.USER.get_id(), REQUEST.USER.get_id(), groups)
            if show_resources:
                res_limit = []
                for resid in show_resources:
                    res_limit.append("""r.resource_id=%d""" % resid)
                res_limit = "\n     (%s)" % " OR ".join(res_limit)

            if show_resources:
                limit_query = """(%s OR
 (SELECT r.event_id FROM event_resources r WHERE r.event_id=e.event_id AND %s LIMIT 1) is not null)
AND
""" % (users_limit, res_limit)
            else:
                limit_query = users_limit + " AND"
        else:
            limit_query = ""

        sql_query = """\
    SELECT
     e.*,
     i.*,
     e.start_stamp AS real_start_stamp,
     e.end_stamp AS real_end_stamp,
     p.first_name || ' ' || p.last_name as creator_name,
     p.e_mail,
     p.phone_cell
    FROM
     events e,
     items i,
     persons p
    WHERE
     i.item_id=e.event_id AND%s
     i.creator_id=p.person_id AND
     e.start_stamp <= e.end_stamp AND
     ((e.start_stamp >= %s AND
      e.start_stamp < %s) OR
     (e.end_stamp >= %s AND
      e.end_stamp < %s) OR
     (e.start_stamp < %s AND
      e.end_stamp > %s) AND
      repeat_interval is null)""" % (limit_query, start, end, start, end, start, end)
        if right_param:
            right_param = " AND\n %s" % right_param
            sql_query = sql_query + right_param
        REQUEST.db.execute(sql_query.encode('utf8'))
        results = REQUEST.db.dictfetchall()

        results2 = []
        for result in results:
            for key in ['start_stamp','end_stamp','real_start_stamp','real_end_stamp']:
                result[key] = _Utils.utc2local_mx(REQUEST, result[key])  
            results2.append(result)

        return results2

class DayHandlerUTC:
    def __init__(self, REQUEST, user_id, year=None, month=None, day=None, show_users=None, show_resources=None):
        self.mapped_days = {}
        self.user_id = user_id
        self.show_users = show_users
        self.show_resources = show_resources
        if year is None or month is None or day is None:
            current_stamp = _Utils.utc_now_mx()
            self.year = current_stamp.year
            self.month = current_stamp.month
            self.day = current_stamp.day
        else:
            self.year = year
            self.month = month
            self.day = day
        self.calendar = calendar.monthcalendar(self.year, self.month)
        if self.month == 1:
            self.prev = calendar.monthcalendar(self.year-1, 12)
            self.next = calendar.monthcalendar(self.year, self.month+1)
        elif self.month == 12:
            self.prev = calendar.monthcalendar(self.year, self.month-1)
            self.next = calendar.monthcalendar(self.year+1, 1)
        else:
            self.prev = calendar.monthcalendar(self.year, self.month-1)
            self.next = calendar.monthcalendar(self.year, self.month+1)
        
        self.active_week = []
        if self.day <= calendar.monthrange(self.year, self.month)[1]:
            self.active_week.append(Day(self.day, self.month, self.year))
        else:
            raise _Exceptions.CalendarError, "Invalid date provided." # FIXME

        i = 0
        if self.active_week[i].day in [11,12,13]:
            self.active_week[i].name = "%d%s" % (self.active_week[i].day, REQUEST.TLS('day_end_th'))
        elif (self.active_week[i].day) % 10 == 1:
            self.active_week[i].name = "%d%s" % (self.active_week[i].day, REQUEST.TLS('day_end_st'))
        elif (self.active_week[i].day) % 10 == 2:
            self.active_week[i].name = "%d%s" % (self.active_week[i].day, REQUEST.TLS('day_end_nd'))
        elif (self.active_week[i].day) % 10 == 3:
            self.active_week[i].name = "%d%s" % (self.active_week[i].day, REQUEST.TLS('day_end_rd'))
        else:
            self.active_week[i].name = "%d%s" % (self.active_week[i].day, REQUEST.TLS('day_end_th'))

        for day in self.active_week:
            self.mapped_days[(day.year, day.month, day.day)] = day

    def load(self, REQUEST, all_users = 0):
        start = (self.active_week[0].year, self.active_week[0].month, self.active_week[0].day, 0, 0, 0, 0, 0, time.localtime()[8])
        events = self.get_calendar_events(REQUEST, start, 0, self.show_users, self.show_resources, all_users)

        for event in events:
            set = _Utils.datetime2tuple(event['start_stamp'])
            if set[0] < self.active_week[0].year or (set[0] == self.active_week[0].year and set[1] < self.active_week[0].month) or (set[0] == self.active_week[0].year and set[1] == self.active_week[0].month and set[2] < self.active_week[0].day): # If event started earlier, make it look like it started on the morning
                event['start_stamp'] = mx.DateTime.DateTime(self.active_week[0].year, self.active_week[0].month, self.active_week[0].day, 7)
                set = _Utils.datetime2tuple(event['start_stamp'])
            self.mapped_days[(set[0], set[1], set[2])].addevent(event, set)
        for day in self.active_week:
            day.sort()

    def get_calendar_events(self, REQUEST, start, days, show_users, show_resources, all_users = 0):
        "Requires datetime tuples."
        start = (start[0], start[1], start[2], 0, 0, 0, start[6], start[7], start[8])
        start_dt = _Utils.tuple2datetime(start)
        start_dt = _Utils.local2utc_mx(REQUEST, start_dt)
        start = _Utils.datetime2tuple(start_dt)
        end_dt = _Utils.tuple2datetime(start) + mx.DateTime.DateTimeDelta(1)
        end = _Utils.datetime2tuple(end_dt)
        end = _Utils.sql_cast_str(REQUEST, _Utils.tuple2isostr(end))
        start = _Utils.sql_cast_str(REQUEST, _Utils.tuple2isostr(start))
        right_param = _Access.select_param(self.user_id, _Access.ACCESS_READ, [_InitTypes.ITEM_TYPE_EVENT])
        if not all_users:
            if show_users: # Show involved people
                users_limit = []
                for userid in show_users:
                    users_limit.append("""i.creator_id = %d""" % userid)
                    if REQUEST.db.version < 740:
                        users_limit.append("""e.related_persons @ '{%d}'::int[]""" % userid)
                        REQUEST.db.execute("""SELECT group_id FROM calendar_groups WHERE members @ '{%d}'::int[]""" % userid)
                    else:
                        users_limit.append("""%d=ANY(e.related_persons)""" % userid)
                        REQUEST.db.execute("""SELECT group_id FROM calendar_groups WHERE %d=ANY(members)""" % userid)
                    groups = REQUEST.db.dictfetchall()
                    if REQUEST.db.version < 740:
                        for i in groups:
                            users_limit.append("""e.related_groups @ '{%d}'::int[]""" % i['group_id'])
                    else:
                        for i in groups:
                            users_limit.append("""%d=ANY(e.related_groups)""" % i['group_id'])
                users_limit = "\n     (%s)" % " OR ".join(users_limit)
            else: # show only personal stuff
                if REQUEST.db.version < 740:
                    REQUEST.db.execute("""SELECT group_id FROM calendar_groups WHERE members @ '{%d}'::int[]""" % self.user_id)
                else:
                    REQUEST.db.execute("""SELECT group_id FROM calendar_groups WHERE %d=ANY(members)""" % self.user_id)
                groups = REQUEST.db.dictfetchall()
                if groups:
                    g = []
                    if REQUEST.db.version < 740:
                        for i in groups:
                            g.append("""e.related_groups @ '{%d}'::int[]""" % i['group_id'])
                    else:
                        for i in groups:
                            g.append("""%d=ANY(e.related_groups)""" % i['group_id'])
                    groups = " OR %s" % " OR ".join(g)
                else:
                    groups = ""
                if REQUEST.db.version < 740:
                    users_limit = """\n     (i.creator_id = %d OR e.related_persons @ '{%d}'::int[]%s)""" % (self.user_id, self.user_id, groups)
                else:
                    users_limit = """\n     (i.creator_id = %d OR %d=ANY(e.related_persons)%s)""" % (self.user_id, self.user_id, groups)
            if show_resources:
                res_limit = []
                for resid in show_resources:
                    res_limit.append("""r.resource_id=%d""" % resid)
                res_limit = "\n     (%s)" % " OR ".join(res_limit)

            if show_resources:
                limit_query = """(%s OR
 (SELECT r.event_id FROM event_resources r WHERE r.event_id=e.event_id AND %s LIMIT 1) is not null)
AND
""" % (users_limit, res_limit)
            else:
                limit_query = users_limit + " AND"
        else:
            limit_query = ""

        sql_query = """\
    SELECT
     e.*,
     i.*,
     e.start_stamp AS real_start_stamp,
     e.end_stamp AS real_end_stamp,
     p.first_name || ' ' || p.last_name as creator_name,
     p.e_mail,
     p.phone_cell
    FROM
     events e,
     items i,
     persons p
    WHERE
     i.item_id=e.event_id AND%s
     i.creator_id=p.person_id AND
     e.start_stamp <= e.end_stamp AND
     ((e.start_stamp >= %s AND
      e.start_stamp < %s) OR
     (e.end_stamp >= %s AND
      e.end_stamp < %s) OR
     (e.start_stamp < %s AND
      e.end_stamp > %s) AND
      repeat_interval is null)""" % (limit_query, start, end, start, end, start, end)
        if right_param:
            right_param = " AND\n %s" % right_param
            sql_query = sql_query + right_param
        REQUEST.db.execute(sql_query.encode('utf8'))
        results = REQUEST.db.dictfetchall()

        results2 = []
        for result in results:
            for key in ['start_stamp','end_stamp','real_start_stamp','real_end_stamp']:
                result[key] = _Utils.utc2local_mx(REQUEST, result[key])  
            results2.append(result)

        return results2
