# Copyright (C) 2003-2005 Inversion Software Ltd. <inversion@inversion.ee>
# http://www.conflux.ee/
#
# This file is part of Conflux Lite
#
# Conflux Lite is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2,
# or (at your option) any later version.
#
# Conflux Lite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Conflux Lite; if not, write to the Free Software
# Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
######################################################################

import email.MIMEText, email.Header, smtplib
from _Conf import SMTPSERVER
import _ConfSMS

# glob defines
DEBUG        = 0

def get_type(address, codes = _ConfSMS.CODES):
    for a in codes.keys():
        for op in codes[a]:
            if ((a + op[0][1:]) == address[:6]):
                return op[1]
    return _ConfSMS.TYPE_NONE

def get_full_address(address, codes = _ConfSMS.CODES):
    long_addr = address[1:]
    short_addr = "0" + address[4:]
    for a in codes.keys():
        for op in codes[a]:
            if ((a + op[0][1:]) == address[:6]):
                return op[2] % { 'long_num': long_addr, 'short_num': short_addr }
    return None

def get_full_ops(codes = _ConfSMS.CODES):
    result = []
    for a in codes.keys():
        for op in codes[a]:
            result.append(a + op[0][1:]) 
    return result

def sub_parse_address(address):
    if not address:
        return None

    codes = _ConfSMS.CODES.keys()
    results = get_full_ops()

    results_noplus = []
    for a in results:
        results_noplus.append(a[1:])

    results_oponly = []
    for a in results:
        results_oponly.append("0" + a[4:])

    if address[:6] in results:
        return address
    elif address[:5] in results_noplus:
        return ("+" + address)
    elif address[:3] in results_oponly:
        default_countrycode = _ConfSMS.CODES.keys()[0]
        return (default_countrycode + address[1:])

def parse_address(address):
    "Checks address (string) for type and if sms, then network. Returns an int and a parsed address."

    address = address.strip()
    address = address.replace(' ', '')
    address = address.replace('-', '')
    address = sub_parse_address(address)
    if not address:
        return (_ConfSMS.TYPE_NONE, None)
    else:
        return (get_type(address), get_full_address(address))

def send_sms(addresses = [], subject = '', body = '', username=_ConfSMS.SENDER_EMAIL):
    "Checks the address list for emails/sms's and sends them out with body as the message."
    error_list = []

    if type([]) <> type(addresses):
        # we have a serious case of string here?
        if type('') == type(addresses):
            addresses = [addresses]
        else:
            error_list.append( {'error':'Error with arguments', 'reason':'Somebody gave me a non-list non-string argument for address!'})
            return error_list

    for address in addresses:
        (type_thing, address2) = parse_address(address)
        if type_thing == _ConfSMS.TYPE_NONE:
            if address2:
                error_list.append( {'error':'Mailing error', 'reason':'Non-supported address type or gsm network for %s.' % address } )
        else:
            err_sms_list = sms_email(address2, subject, body, username)
            if err_sms_list <> []:
                error_list = error_list + err_sms_list

    return error_list

def sms_email(address, subject='', body='', username=_ConfSMS.SENDER_EMAIL):
    if address.find('@') == -1:
        return [ { 'error':'Error with sending message', 'reason':'Invalid e-mail address %s.' % address}]

    err_list = []

    username = '%s@%s' % (username, SMTPSERVER)

    try:
        message = email.MIMEText.MIMEText(body)
        del message['Content-Type']
        message['Content-Type'] = 'text/plain; charset="utf-8"'
        message['From'] = email.Header.Header(username)
        message['To'] = email.Header.Header(address)
        message['Subject'] = email.Header.Header(subject)

        server = smtplib.SMTP(SMTPSERVER)
        errors = server.sendmail(message['from'], address, message.as_string())
        server.quit()
    except:
        err_list.append(address)

    return err_list

#if __name__=="__main__":
#    import sys
#    print send_sms(sys.argv[1:], 'test', 'blablabla\ndisko', 'ruutmees')
