# Copyright (C) 2003-2005 Inversion Software Ltd. <inversion@inversion.ee>
# http://www.conflux.ee/
#
# This file is part of Conflux Lite
#
# Conflux Lite is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2,
# or (at your option) any later version.
#
# Conflux Lite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Conflux Lite; if not, write to the Free Software
# Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
######################################################################

import _Conf, _InitTypes, standard_html, _RootObject, _Exceptions, os, urllib, _Utils

UNLINKABLE_TYPES = [_InitTypes.ITEM_TYPE_PERSON, _InitTypes.ITEM_TYPE_ORGANIZATION, _InitTypes.ITEM_TYPE_DISCCATEGORY, _InitTypes.ITEM_TYPE_EFOLDER, _InitTypes.ITEM_TYPE_EACCOUNT, _InitTypes.ITEM_TYPE_RESOURCE, _InitTypes.ITEM_TYPE_FOLDER, _InitTypes.ITEM_TYPE_EMAIL, _InitTypes.ITEM_TYPE_BOOKMARK, _InitTypes.ITEM_TYPE_USERNOTE, _InitTypes.ITEM_TYPE_DISCUSSION, _InitTypes.ITEM_TYPE_EFILTER, _InitTypes.ITEM_TYPE_QUOTA, _InitTypes.ITEM_TYPE_CONTACTGROUP, _InitTypes.ITEM_TYPE_VIRTFOLDER, _InitTypes.ITEM_TYPE_MESSAGE, _InitTypes.ITEM_TYPE_DUSJOB, _InitTypes.ITEM_TYPE_DUSSERVICE, _InitTypes.ITEM_TYPE_DUSTRANSLATION]

REL_DUS_ORIG = 1
REL_DUS_RESULT = 2

def add_relation(REQUEST, left_id, right_id, relation_flags=0):
    if left_id == right_id:
        return
    SQL = """SELECT * FROM item_rels WHERE relation_flags=%(relation_flags)d AND (left_item_id=%(left_id)d AND right_item_id=%(right_id)d) OR (left_item_id=%(right_id)d and right_item_id=%(left_id)d)""" % locals()
    REQUEST.db.execute(SQL)
    if REQUEST.db.dictfetchone():
       return
    SQL = """INSERT INTO item_rels ( left_item_id, right_item_id, relation_flags ) VALUES ( %(left_id)d, %(right_id)d, %(relation_flags)d )""" % locals()
    REQUEST.db.execute(SQL)

def get_relations(REQUEST, item, relation_flags=0):
    SQL = """\
SELECT
 ir.*,
CASE WHEN i.item_id=ir.right_item_id THEN i.item_name ELSE i2.item_name END AS right_item_name,
CASE WHEN i.item_id=ir.right_item_id THEN i2.item_name ELSE i.item_name END AS left_item_name,
CASE WHEN i.item_id=ir.right_item_id THEN i.item_type_id ELSE i2.item_type_id END AS right_type_id,
CASE WHEN i.item_id=ir.right_item_id THEN i2.item_type_id ELSE i.item_type_id END AS left_type_id,
CASE WHEN i.item_id=ir.right_item_id THEN i.edit_time ELSE i2.edit_time END AS right_edit_time,
CASE WHEN i.item_id=ir.right_item_id THEN i2.edit_time ELSE i.edit_time END AS left_edit_time
FROM
 item_rels ir,
 items i,
 items i2
WHERE
 ir.relation_flags=%d AND
 (i.item_id=ir.left_item_id AND i2.item_id=ir.right_item_id AND ir.left_item_id = %d)
  OR
 (i.item_id=ir.right_item_id AND i2.item_id=ir.left_item_id AND ir.right_item_id = %d)
""" % (relation_flags, item.get_id(), item.get_id())
    REQUEST.db.execute(SQL)
    return REQUEST.db.dictfetchall()

def get_relations_by_id(REQUEST, item_id, relation_flags=0):
    SQL = """\
SELECT
 ir.*,
CASE WHEN i.item_id=ir.right_item_id THEN i.item_name ELSE i2.item_name END AS right_item_name,
CASE WHEN i.item_id=ir.right_item_id THEN i2.item_name ELSE i.item_name END AS left_item_name,
CASE WHEN i.item_id=ir.right_item_id THEN i.item_type_id ELSE i2.item_type_id END AS right_type_id,
CASE WHEN i.item_id=ir.right_item_id THEN i2.item_type_id ELSE i.item_type_id END AS left_type_id,
CASE WHEN i.item_id=ir.right_item_id THEN i.edit_time ELSE i2.edit_time END AS right_edit_time,
CASE WHEN i.item_id=ir.right_item_id THEN i2.edit_time ELSE i.edit_time END AS left_edit_time
FROM
 item_rels ir,
 items i,
 items i2
WHERE
 ir.relation_flags=%d AND
 (i.item_id=ir.left_item_id AND i2.item_id=ir.right_item_id AND ir.left_item_id = %d)
  OR
 (i.item_id=ir.right_item_id AND i2.item_id=ir.left_item_id AND ir.right_item_id = %d)
""" % (relation_flags, item_id, item_id)
    REQUEST.db.execute(SQL)
    return REQUEST.db.dictfetchall()

def relations_count(REQUEST, item, relation_flags=0):
   relations = get_relations(REQUEST, item, relation_flags)
   return len(relations)

def show_relations(REQUEST, item, relation_flags=0):
    output = []
    if(REQUEST.form.has_key('sel_id') and REQUEST.form['sel_id'].value.strip()):
        del_id_1 = int(REQUEST.form['rel_id1'].value)
        del_id_2 = int(REQUEST.form['sel_id'].value)
        SQL = "DELETE FROM item_rels WHERE relation_flags=%d AND (left_item_id = %d AND right_item_id = %d) OR (left_item_id = %d AND right_item_id = %d)" % (relation_flags, del_id_1, del_id_2, del_id_2, del_id_1)
        REQUEST.db.execute(SQL)
    
    SQL = """SELECT count(*) AS count_all FROM item_rels ir, items i, items i2 WHERE ir.relation_flags=%d AND (i.item_id=ir.left_item_id AND i2.item_id=ir.right_item_id AND ir.left_item_id = %d) OR (i.item_id=ir.right_item_id AND i2.item_id=ir.left_item_id AND ir.right_item_id = %d)""" % (relation_flags, item.get_id(), item.get_id())
    REQUEST.db.execute(SQL)
    count_all = REQUEST.db.dictfetchone()['count_all']

    relations = get_relations(REQUEST, item)
    relations_dict = {}
    for relation in relations:
        try:
            if item.get_id() == relation['left_item_id']:
                relations_dict[relation['right_type_id']].append(relation)
            else:
                relations_dict[relation['left_type_id']].append(relation)
        except KeyError:
            if item.get_id() == relation['left_item_id']:
                relations_dict[relation['right_type_id']] = []
                relations_dict[relation['right_type_id']].append(relation)
            else:
                relations_dict[relation['left_type_id']] = []
                relations_dict[relation['left_type_id']].append(relation)

    SQL = """SELECT * FROM item_types ORDER BY item_type_id"""
    REQUEST.db.execute(SQL)
    item_types = REQUEST.db.dictfetchall()
    tls_add_relation = REQUEST.TLS('tls_add_relation')
    tls_name = REQUEST.TLS('tls_name')
    tls_owner = REQUEST.TLS('client_owner')
    tls_edit = REQUEST.TLS('tls_edittime')
    tls_create = REQUEST.TLS('tls_createtime')
    BASE_ICON_URL = _Conf.BASE_ICON_URL
    item_id = item.get_id()
    tls_relations = REQUEST.TLS('tls_relations')
    tls_remove_rels = REQUEST.TLS('tls_remove_rels')
    tls_add = REQUEST.TLS('tls_add')
    tls_choose = REQUEST.TLS('email_choose')
    tls_action = REQUEST.TLS('email_action')
    tls_short_del = REQUEST.TLS('email_short_del')
    tls_short_edit = REQUEST.TLS('admin_edit')
    tls_norecords = REQUEST.TLS('tls_norecords')

    access_str = _Utils.get_module_access(REQUEST)


    types_menus = {_InitTypes.ITEM_TYPE_DOCUMENT: _Conf.MODULES_DOCUMENTS,
                   _InitTypes.ITEM_TYPE_FOLDER: _Conf.MODULES_DOCUMENTS,
                   _InitTypes.ITEM_TYPE_EVENT: _Conf.MODULES_CALENDAR,
                   _InitTypes.ITEM_TYPE_TODO: _Conf.MODULES_TODOS,
                   _InitTypes.ITEM_TYPE_WORKFLOW: _Conf.MODULES_WORKFLOWS,
                   _InitTypes.ITEM_TYPE_CONTACT: _Conf.MODULES_CONTACTS,
                   _InitTypes.ITEM_TYPE_DISCUSSION: _Conf.MODULES_DISCUSSION,
                   _InitTypes.ITEM_TYPE_EMAIL: _Conf.MODULES_EMAIL,
                   _InitTypes.ITEM_TYPE_USERNOTE: _Conf.MODULES_TODOS,
                   _InitTypes.ITEM_TYPE_OPPORTUNITY: _Conf.MODULES_OPPORTUNITIES,
                   _InitTypes.ITEM_TYPE_PRODUCT: _Conf.MODULES_PRODUCTS,
                   _InitTypes.ITEM_TYPE_CONTACT: _Conf.MODULES_CONTACTS,
                   _InitTypes.ITEM_TYPE_BOOKMARK: _Conf.MODULES_DESKTOP,
                   _InitTypes.ITEM_TYPE_CLIENT: _Conf.MODULES_CLIENTS,
                   _InitTypes.ITEM_TYPE_MESSAGE: _Conf.MODULES_MESSAGES,
    }
    
    output.append(u"""
<TABLE width="100%%" cellpadding="0" cellspacing="0" border="0">
<FORM name="relf" method="post">
<input type="hidden" name="rel_id1" value="%(item_id)s">
<TR>
  <TD class="td_listing1"><img src="%(BASE_ICON_URL)sspacer.gif" width=10 height=20 border=0 alt=""></TD>
  <TD class="td_listing1b" width="100%%">%(tls_relations)s</TD>
</TR>
</TABLE>
<BR>
<TABLE border="0" width="100%%" cellpadding="0" cellspacing="0">""" % locals())
    
    link_dict = {_InitTypes.ITEM_TYPE_DOCUMENT: "<a class=\"td_listing\" href=\"" + _Conf.BASE_GENERAL_URL + 'info.py?item_id=%(item_id)d' + "\">",
                 _InitTypes.ITEM_TYPE_EVENT: "<a class=\"td_listing\" href=\"" + _Conf.BASE_CALENDAR_URL + 'editevent.py?event_id=%(item_id)d' + "\">",
                 _InitTypes.ITEM_TYPE_TODO: "<a class=\"td_listing\" href=\"" + _Conf.BASE_TODO_URL + 'todoedit.py?todo_id=%(item_id)d' + "\">",
                 _InitTypes.ITEM_TYPE_WORKFLOW: "<a class=\"td_listing\" href=\"" + _Conf.BASE_WORKFLOWS_URL + 'workflowedit.py?workflow_id=%(item_id)d' + "\">",
                 _InitTypes.ITEM_TYPE_CONTACT: "<a class=\"td_listing\" href=\"" + _Conf.BASE_CONTACT_URL + 'contactedit.py?contact_id=%(item_id)d' + "\">",
                 _InitTypes.ITEM_TYPE_CLIENT: "<a class=\"td_listing\" href=\"" + _Conf.BASE_CLIENT_URL + 'clientedit.py?client_id=%(item_id)d' + "\">",
                 _InitTypes.ITEM_TYPE_DISCUSSION: "<a class=\"td_listing\" href=\"" + _Conf.BASE_DISCUSSION_URL + 'discread.py?discussion_id=%(item_id)d' + "\">",
                 _InitTypes.ITEM_TYPE_OPPORTUNITY: "<a class=\"td_listing\" href=\"" + _Conf.BASE_OPPORTUNITIES_URL + 'oppedit.py?opp_id=%(item_id)d' + "\">",
                 _InitTypes.ITEM_TYPE_PRODUCT: "<a class=\"td_listing\" href=\"" + _Conf.BASE_PRODUCTS_URL + 'editproduct.py?product_id=%(item_id)d' + "\">",
                }
    
    edit_dict = {_InitTypes.ITEM_TYPE_DOCUMENT: "<a class=\"td_listing\" href=\"" + _Conf.BASE_GENERAL_URL + 'info.py?item_id=%(item_id)d' + "\">",
                 _InitTypes.ITEM_TYPE_EVENT: "<a class=\"td_listing\" href=\"" + _Conf.BASE_CALENDAR_URL + 'editevent.py?event_id=%(item_id)d' + "\">",
                 _InitTypes.ITEM_TYPE_TODO: "<a class=\"td_listing\" href=\"" + _Conf.BASE_TODO_URL + 'todoedit.py?todo_id=%(item_id)d' + "\">",
                 _InitTypes.ITEM_TYPE_WORKFLOW: "<a class=\"td_listing\" href=\"" + _Conf.BASE_WORKFLOWS_URL + 'workflowedit.py?workflow_id=%(item_id)d' + "\">",
                 _InitTypes.ITEM_TYPE_CONTACT: "<a class=\"td_listing\" href=\"" + _Conf.BASE_CONTACT_URL + 'contactedit.py?contact_id=%(item_id)d' + "\">",
                 _InitTypes.ITEM_TYPE_CLIENT: "<a class=\"td_listing\" href=\"" + _Conf.BASE_CLIENT_URL + 'clientedit.py?client_id=%(item_id)d' + "\">",
                 _InitTypes.ITEM_TYPE_DISCUSSION: "<a class=\"td_listing\" href=\"" + _Conf.BASE_DISCUSSION_URL + 'discread.py?discussion_id=%(item_id)d' + "\">",
                 _InitTypes.ITEM_TYPE_OPPORTUNITY: "<a class=\"td_listing\" href=\"" + _Conf.BASE_OPPORTUNITIES_URL + 'oppedit.py?opp_id=%(item_id)d' + "\">",
                 _InitTypes.ITEM_TYPE_PRODUCT: "<a class=\"td_listing\" href=\"" + _Conf.BASE_PRODUCTS_URL + 'editproduct.py?product_id=%(item_id)d' + "\">",
                }
    add_dict = {_InitTypes.ITEM_TYPE_DOCUMENT: _Conf.BASE_DOCUMENT_URL + "docadd.py",
                 _InitTypes.ITEM_TYPE_EVENT: _Conf.BASE_CALENDAR_URL + "addevent.py",
                 _InitTypes.ITEM_TYPE_TODO: _Conf.BASE_TODO_URL + "todoadd.py",
                 _InitTypes.ITEM_TYPE_WORKFLOW: _Conf.BASE_WORKFLOWS_URL + "workflowadd.py",
                 _InitTypes.ITEM_TYPE_CONTACT: _Conf.BASE_CONTACT_URL + "contactadd.py",
                 _InitTypes.ITEM_TYPE_CLIENT: _Conf.BASE_CLIENT_URL + "clientadd.py",
                 _InitTypes.ITEM_TYPE_DISCUSSION: _Conf.BASE_DISCUSSION_URL + "discadd.py",
                 _InitTypes.ITEM_TYPE_OPPORTUNITY: _Conf.BASE_OPPORTUNITIES_URL + "oppadd.py",
                 _InitTypes.ITEM_TYPE_PRODUCT: _Conf.BASE_PRODUCTS_URL + "addproduct.py",
                 _InitTypes.ITEM_TYPE_MESSAGE: _Conf.BASE_MESSAGES_URL + "messaddin.py",
               }
    for item_type in item_types:
        if item_type['item_type_id'] not in UNLINKABLE_TYPES and access_str[types_menus[item_type['item_type_id']]]=="1":
            item_type['item_id'] = item_id
            item_type['BASE_ICON_URL'] = _Conf.BASE_ICON_URL
            item_type['item_type_name'] = REQUEST.TLS("type_" + item_type['item_type_name'])
            item_type['tls_add'] = REQUEST.TLS('tls_add')
            item_type['tls_choose'] = REQUEST.TLS('email_choose')
            item_type['item_choose'] = """onclick="open_window('%sselectrel.py?item_id=%d&item_type_id=%d');" """ % (_Conf.BASE_GENERAL_URL, item_id, item_type['item_type_id'])
            item_type['returl']=urllib.quote(REQUEST.mod_req.uri)+urllib.quote(REQUEST.get_formstring(['sel_id','rnd','o','rel_id1']))
            item_type['item_add'] = add_dict[item_type['item_type_id']]
            output.append(u"""
<TR>
  <TD class="td_listing0"><img src="%(BASE_ICON_URL)sspacer.gif" width=10 height=1 border=0 alt=""></TD>
  <TD width="100%%" colspan="6">
    <TABLE border=0 width="100%%" cellpadding="0" cellspacing="0">
      <TR>
        <TD><img src="%(BASE_ICON_URL)sspacer.gif" width="2" height="1" border=0 alt=""></TD>
        <TD><img src="%(BASE_ICON_URL)sspacer.gif" width="100" height="1" border=0 alt=""></TD>
        <TD><img src="%(BASE_ICON_URL)sspacer.gif" width="2" height="1" border=0 alt=""></TD>
        <TD><img src="%(BASE_ICON_URL)sspacer.gif" width="2" height="1" border=0 alt=""></TD>
        <TD width="100%%"><img src="%(BASE_ICON_URL)sspacer.gif" width="2" height="1" border=0 alt=""></TD>
      </TR>
      <TR>
        <TD style="background-image: url('%(BASE_ICON_URL)sdokunurk1.gif')"><img src="%(BASE_ICON_URL)sspacer.gif" width="2" height="20" border=0 alt=""></TD>
        <TD class="th_listing"><b>%(item_type_name)s</b></TD>
        <TD style="background-image: url('%(BASE_ICON_URL)sdokunurk2.gif')"><img src="%(BASE_ICON_URL)sspacer.gif" width="2" height="20" border=0 alt=""></TD>
        <TD nowrap>&nbsp;&nbsp;&nbsp;&nbsp;<input type="button" class="input6" value=" %(tls_add)s " onclick="document.location='%(item_add)s?rel_id=%(item_id)s&returl=%(returl)s'"> <input type="submit" class="input6" value=" %(tls_choose)s " %(item_choose)s></TD>
        <TD width="100%%"><img src="%(BASE_ICON_URL)sspacer.gif" width="1" height="1" border=0 alt=""></TD>
      </TR>
    </TABLE>
  <TD class="td_listing0"><img src="%(BASE_ICON_URL)sspacer.gif" width=10 height=1 border=0 alt=""></TD>
</TR>""" % item_type)
        else:
            continue
        if relations_dict.has_key(item_type['item_type_id']) and relations_dict[item_type['item_type_id']]:
            output.append(u"""
<TR>
  <TD class="td_listing1b"><img src="%(BASE_ICON_URL)sspacer.gif" width=10 height=1 border=0 alt=""></TD>
  <TD class="td_listing1b">&nbsp;</TD>
  <TD class="td_listing1b">%(tls_name)s</TD>
  <TD class="td_listing1b">%(tls_owner)s</TD>
  <TD class="td_listing1b">%(tls_create)s</TD>
  <TD class="td_listing1b">%(tls_edit)s</TD>
  <TD class="td_listing1b" align="center">%(tls_action)s</TD>
  <TD class="td_listing1b"><img src="%(BASE_ICON_URL)sspacer.gif" width=10 height=1 border=0 alt=""></TD>
</TR>""" % locals())
            index = 0
            for itemlist in relations_dict[item_type['item_type_id']]:
                REQUEST.su_by_id(_Conf.SUPERUSER)
                try:
                    if item.get_id() == itemlist['left_item_id']:
                        obj = _RootObject.dispatch_item(REQUEST, itemlist['right_item_id'])
                    else:
                        obj = _RootObject.dispatch_item(REQUEST, itemlist['left_item_id'])
                except _Exceptions.RootObjectError:
                    REQUEST.unsu()
                    continue
                REQUEST.unsu()
                obj_name = _Utils.html_quote_show(obj.get_name())
                obj_id = obj.get_id()
                SQL="""select item_type_icon from item_types where item_type_id = %d""" %(obj.get('item_type_id'))
                REQUEST.db.execute(SQL)
                item_info = REQUEST.db.dictfetchall()
                obj_icon = item_info[0]['item_type_icon']
                if obj.get('item_type_id') == _InitTypes.ITEM_TYPE_DOCUMENT:
                    file_ext = os.path.splitext(obj.get('filename') or '')[-1]
                    if(_Conf.MIME_TYPES.has_key(file_ext)):
                        obj_icon =  _Conf.MIME_TYPES[file_ext]
                    else:
                        obj_icon =  _Conf.MIME_TYPES["unknown_ext"]
                REQUEST.su_by_id(_Conf.SUPERUSER)
                try:
                    obj_creator = _RootObject.dispatch_item(REQUEST, obj.get('creator_id')).get_name()
                except _Exceptions.RootObjectError:
                    obj_creator = u"N/A"
                REQUEST.unsu()
                obj_edit = REQUEST.iso2userdatetime(REQUEST, obj.get('edit_time'))
                obj_create = REQUEST.iso2userdatetime(REQUEST, obj.get('create_time'))
                items = {'item_id':obj_id, 'BASE_CALENDAR_URL':_Conf.BASE_CALENDAR_URL}
                if item.get('item_type_id') <> _InitTypes.ITEM_TYPE_DOCUMENT and obj.get('item_type_id') == _InitTypes.ITEM_TYPE_DOCUMENT:
                    link = _Conf.BASE_DOCUMENT_URL + "docframe.py?doc_url=" + urllib.quote(_Conf.BASE_GENERAL_URL + 'info.py?item_id=%(item_id)d' % items)
                    view_link = u"""<a class="td_listing" href="%(link)s">""" % locals()
                    edit_link = view_link
                elif obj.get('item_type_id') == _InitTypes.ITEM_TYPE_DOCUMENT:
                    link = _Conf.BASE_GENERAL_URL + 'info.py?item_id=%(item_id)d' % items
                    view_link = u"""<a class="td_listing" href="%(link)s">""" % locals()
                    edit_link = view_link
                else:
                    try:
                        view_link = link_dict[obj.get('item_type_id')] % items
                    except:
                        view_link = '#'
                    try:
                        edit_link = edit_dict[obj.get('item_type_id')] % items
                    except:
                        link = '#'
                if index%2==0:
                  td_class = "0"
                else:
                  td_class = "1"
                index = index + 1
                output.append(u"""
<TR>
  <TD class="td_listing%(td_class)s"><img src="%(BASE_ICON_URL)sspacer.gif" width=10 height=1 border=0 alt=""></TD>
  <TD class="td_listing_rel%(td_class)s"><IMG src="%(BASE_ICON_URL)s%(obj_icon)s" border="0" alt=""></TD>
  <TD class="td_listing%(td_class)s" width="45%%">%(view_link)s%(obj_name)s</A></TD>
  <TD class="td_listing%(td_class)s" width="30%%">%(obj_creator)s</TD>
  <TD class="td_listing%(td_class)s" width="10%%" nowrap>%(obj_create)s</TD>
  <TD class="td_listing%(td_class)s" width="10%%" nowrap>%(obj_edit)s</TD>
  <TD class="td_listing%(td_class)s" width="5%%" nowrap>%(edit_link)s%(tls_short_edit)s</A> | <A onclick="document.forms['relf'].sel_id.value='%(obj_id)s';document.forms['relf'].submit();" href="#" class="td_listing">%(tls_short_del)s</A></TD>
  <TD class="td_listing%(td_class)s"><img src="%(BASE_ICON_URL)sspacer.gif" width=10 height=1 border=0 alt=""></TD>
</TR>""" % locals())
            output.append(u"""
<TR>
  <TD colspan="8"><img src="%(BASE_ICON_URL)sspacer.gif" width=1 height=20 border=0 alt=""></TD>
</TR>
""" % locals())
        else:
            output.append(u"""
<TR>
  <TD class="td_listing1b"><img src="%(BASE_ICON_URL)sspacer.gif" width=10 height=1 border=0 alt=""></TD>
  <TD class="td_listing1" colspan="6">%(tls_norecords)s</TD>
  <TD class="td_listing1b"><img src="%(BASE_ICON_URL)sspacer.gif" width=10 height=1 border=0 alt=""></TD>
</TR>
<TR>
  <TD colspan="8"><img src="%(BASE_ICON_URL)sspacer.gif" width=1 height=20 border=0 alt=""></TD>
</TR>
"""  % locals())
    output.append(u"""
</TABLE>
<input type="hidden" name="sel_id" value="">
</FORM>""")
    output.append(standard_html.window_script(target='relwin',height=550, width=600))
    return "\n".join(output)
