DBNAME="confluxlite"
USER="conflux"
SUSER="postgres"
PORT=5432

def update_paths(cur, parent_id):
    cur.execute("""SELECT item_id FROM items WHERE parent_id=%d and item_id <> 0""" % parent_id)
    res = cur.fetchall()
    for row in res:
        cur.execute("""UPDATE items SET is_private=is_private WHERE item_id=%d""" % res[0])
    for row in res:
        update_paths(cur, res[0])

import psycopg
con = psycopg.connect('dbname=%s user=%s port=%d' % (DBNAME, USER, PORT))
cur = con.cursor()
sucon = psycopg.connect('dbname=%s user=%s port=%d' % (DBNAME, SUSER, PORT))
sucur = sucon.cursor()

sucur.execute("""CREATE FUNCTION plpgsql_call_handler() RETURNS language_handler
    AS '$libdir/plpgsql', 'plpgsql_call_handler'
    LANGUAGE c""")
sucur.execute("""CREATE TRUSTED PROCEDURAL LANGUAGE plpgsql HANDLER plpgsql_call_handler""")
sucur.execute("""DROP FUNCTION rebuild_itempath(integer)""")
sucur.execute("""DROP FUNCTION get_nextpath(integer)""")
sucur.execute("""DROP FUNCTION items_path_trig() CASCADE""")
sucon.commit()
sucur.close()
sucon.close()

cur.execute("""ALTER TABLE items ADD treepos varbit""")
cur.execute("""UPDATE items SET treepos=0::bit(32)""")
cur.execute("""CREATE FUNCTION items_treepos_trig() RETURNS "trigger"
    AS '
  DECLARE
    parent_rec RECORD;
  BEGIN
    SELECT INTO parent_rec treepos FROM items WHERE item_id = NEW.parent_id;
    NEW.treepos := parent_rec.treepos || "bit"(NEW.item_id);
    RETURN NEW;
  END;
'
    LANGUAGE plpgsql""")
cur.execute("""CREATE TRIGGER items_treepos_trig
    BEFORE INSERT OR UPDATE ON items
    FOR EACH ROW
    EXECUTE PROCEDURE items_treepos_trig()""")
con.commit()
update_paths(cur, 0)
cur.execute("""ALTER TABLE items ALTER treepos SET NOT NULL""")
cur.execute("""CREATE UNIQUE INDEX items_treepos_ndx ON items (treepos)""")

cur.execute("""ALTER TABLE settings ADD treepos varbit""")
cur.execute("""UPDATE settings SET treepos=(SELECT treepos FROM items WHERE item_path=settings.item_path)""")
cur.execute("""DROP INDEX settings_ndx""")
cur.execute("""ALTER TABLE settings DROP constraint "$1" """)
cur.execute("""CREATE UNIQUE INDEX settings_ndx ON settings USING btree (setting_key, treepos)""")
cur.execute("""ALTER TABLE ONLY settings ADD CONSTRAINT "$1" FOREIGN KEY (treepos) REFERENCES items(treepos) ON UPDATE CASCADE ON DELETE CASCADE""")
cur.execute("""ALTER TABLE settings DROP item_path""")
cur.execute("""DROP INDEX access_rights_path_ndx""")
cur.execute("""ALTER TABLE access_rights ADD treepos varbit""")
cur.execute("""UPDATE access_rights SET treepos=(SELECT treepos FROM items WHERE item_path=access_rights.item_path)""")
cur.execute("""ALTER TABLE access_rights ALTER treepos SET NOT NULL""")
cur.execute("""ALTER TABLE access_rights DROP item_path""")
cur.execute("""CREATE INDEX access_rights_treepos_ndx ON access_rights USING btree (treepos)""")
cur.execute("""ALTER TABLE ONLY access_rights ADD CONSTRAINT "$3" FOREIGN KEY (treepos) REFERENCES items(treepos) ON UPDATE CASCADE ON DELETE CASCADE""")
cur.execute("""DROP TABLE items_scrollback""")
cur.execute("""DROP INDEX items_path_ndx""")
cur.execute("""ALTER TABLE items DROP item_path""")
cur.execute("""ALTER TABLE bookmarks ADD is_category int4""")
cur.execute("""ALTER TABLE bookmarks ALTER bookmark_description DROP NOT NULL""")
cur.execute("""ALTER TABLE contacts ADD e_mail2 text""")
cur.execute("""ALTER TABLE contacts ADD e_mail3 text""")
cur.execute("""ALTER TABLE discussions ADD is_front_page boolean""")
cur.execute("""UPDATE discussions SET is_front_page=false""")
cur.execute("""ALTER TABLE discussions ALTER is_front_page SET NOT NULL""")
cur.execute("""ALTER TABLE discussions ALTER is_front_page SET DEFAULT false""")
cur.execute("""ALTER TABLE documents ADD locker_id int4""")
cur.execute("""ALTER TABLE documents ADD lock_due timestamptz""")
cur.execute("""ALTER TABLE emails ADD email_from_addr text""")
cur.execute("""CREATE INDEX emails_addr_ndx ON emails USING btree (email_from_addr)""")
cur.execute("""ALTER TABLE events ADD last_reminder timestamptz""")
cur.execute("""ALTER TABLE events ADD related_groups int4[]""")
cur.execute("""UPDATE events SET related_groups='{}'""")
cur.execute("""ALTER TABLE events ALTER related_groups SET NOT NULL""")
cur.execute("""ALTER TABLE events ALTER related_groups SET DEFAULT '{}'::int4[]""")
cur.execute("""ALTER TABLE events ADD event_flags int4""")
cur.execute("""UPDATE events SET event_flags=0""")
cur.execute("""ALTER TABLE events ALTER event_flags SET NOT NULL""")
cur.execute("""ALTER TABLE events ALTER event_flags SET DEFAULT 0""")
cur.execute("""DROP INDEX events_related_ndx""")
cur.execute("""ALTER TABLE items DROP index_this_item""")
cur.execute("""ALTER TABLE items ADD owner_id int4""")
cur.execute("""ALTER TABLE items ADD index_this_item int4""")
cur.execute("""ALTER TABLE items ALTER index_this_item SET DEFAULT 0""")
cur.execute("""UPDATE items SET index_this_item=0""")
cur.execute("""DROP INDEX uniq_persons_username_ndx""")
cur.execute("""ALTER TABLE persons ADD picture text""")
cur.execute("""ALTER TABLE persons ADD email_notify int4""")
cur.execute("""ALTER TABLE persons ADD is_deleted int4""")
cur.execute("""ALTER TABLE persons ADD person_flags int4""")
cur.execute("""UPDATE persons SET email_notify=0""")
cur.execute("""UPDATE persons SET person_flags=0""")
cur.execute("""UPDATE persons SET is_deleted=0""")
cur.execute("""ALTER TABLE persons ALTER email_notify SET NOT NULL""")
cur.execute("""ALTER TABLE persons ALTER email_notify SET DEFAULT 0""")
cur.execute("""ALTER TABLE persons ALTER person_flags SET DEFAULT 0""")
cur.execute("""ALTER TABLE persons ALTER person_flags SET NOT NULL""")
cur.execute("""ALTER TABLE persons ALTER is_deleted SET NOT NULL""")
cur.execute("""ALTER TABLE persons ALTER is_deleted SET DEFAULT 0""")
cur.execute("""CREATE INDEX persons_deleted_ndx ON persons USING btree (is_deleted)""")
cur.execute("""ALTER TABLE systemnotes ADD tmp date""")
cur.execute("""UPDATE systemnotes SET tmp=note_stamp::date""")
cur.execute("""ALTER TABLE systemnotes DROP note_stamp""")
cur.execute("""ALTER TABLE systemnotes RENAME tmp TO note_stamp""")
cur.execute("""ALTER TABLE systemnotes ALTER note_stamp SET DEFAULT now()""")
cur.execute("""DROP INDEX todos_related_ndx""")
cur.execute("""ALTER TABLE todos DROP start_time""")
cur.execute("""ALTER TABLE todos DROP due_time""")
cur.execute("""ALTER TABLE todos ADD workflow_id int4""")
cur.execute("""ALTER TABLE todos ADD workflow_sequence_id int4""")
cur.execute("""ALTER TABLE todos ADD start_time date""")
cur.execute("""ALTER TABLE todos ADD due_time date""")
cur.execute("""ALTER TABLE user_groups ADD top_org_id int4""")
cur.execute("""CREATE INDEX usergroups_toporgid_ndx ON user_groups USING btree (top_org_id)""")
cur.execute("""ALTER TABLE ONLY user_groups ADD CONSTRAINT "$2" FOREIGN KEY (top_org_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE""")
cur.execute("""ALTER TABLE usernotes ADD note_color int4""")
cur.execute("""UPDATE usernotes SET note_color=0""")
cur.execute("""ALTER TABLE usernotes ALTER note_color SET NOT NULL""")
cur.execute("""ALTER TABLE usernotes ALTER note_color SET DEFAULT 0""")
cur.execute("""CREATE TABLE resources (
    resource_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    resource_name text NOT NULL
);
ALTER TABLE ONLY resources
    ADD CONSTRAINT resources_pkey PRIMARY KEY (resource_id);
ALTER TABLE ONLY resources
    ADD CONSTRAINT "$1" FOREIGN KEY (resource_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;
CREATE TABLE calendar_groups (
    parent_id integer NOT NULL,
    group_name text NOT NULL,
    members integer[] DEFAULT '{}'::integer[] NOT NULL,
    group_id integer DEFAULT nextval('calendar_groups_seq'::text) NOT NULL
);
CREATE SEQUENCE calendar_groups_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
CREATE UNIQUE INDEX cal_groups_uniq ON calendar_groups USING btree (parent_id, group_name);
CREATE TABLE clients (
    client_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    client_name text NOT NULL,
    client_nr text,
    telephone text,
    fax text,
    e_mail text,
    registry_code text,
    client_type text,
    client_field text,  
    client_rating integer,
    client_ownership text,
    homepage text,
    vat_nr text,
    country text, 
    mail_address text,
    legal_address text,
    description text,
    bank_reqs text
);
ALTER TABLE ONLY clients
    ADD CONSTRAINT clients_pkey PRIMARY KEY (client_id);
ALTER TABLE ONLY clients
    ADD CONSTRAINT "$1" FOREIGN KEY (client_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;
CREATE TABLE contact_groups (
    group_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    group_name text NOT NULL,
    group_description text,
    contacts integer[] DEFAULT '{}'::integer[] NOT NULL
);
ALTER TABLE ONLY contact_groups
    ADD CONSTRAINT contact_groups_pkey PRIMARY KEY (group_id);
ALTER TABLE ONLY contact_groups
    ADD CONSTRAINT "$1" FOREIGN KEY (group_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;
CREATE TABLE dictionaries (
    person_id integer NOT NULL,
    "language" text NOT NULL,
    word text NOT NULL
);
ALTER TABLE ONLY dictionaries
    ADD CONSTRAINT dictionaries_pkey PRIMARY KEY (person_id, "language", word);
CREATE TABLE event_resources (
    event_id integer,
    resource_id integer
);
CREATE INDEX evnt_res_ev_ndx ON event_resources USING btree (event_id);   
CREATE INDEX evnt_res_res_ndx ON event_resources USING btree (resource_id);
ALTER TABLE ONLY event_resources
    ADD CONSTRAINT "$1" FOREIGN KEY (event_id) REFERENCES events(event_id) ON UPDATE RESTRICT ON DELETE CASCADE;
ALTER TABLE ONLY event_resources
    ADD CONSTRAINT "$2" FOREIGN KEY (resource_id) REFERENCES resources(resource_id) ON UPDATE RESTRICT ON DELETE CASCADE;
CREATE TABLE fti_search (
    word text NOT NULL,
    item_id integer NOT NULL,
    fti_type boolean DEFAULT false NOT NULL
);
CREATE INDEX fti_item_ndx ON fti_search USING btree (item_id, fti_type);
CREATE INDEX fti_word_ndx ON fti_search USING btree (word, fti_type);
CREATE TABLE im (
    msg_id integer DEFAULT nextval('im_seq'::text) NOT NULL,
    source_id integer,
    dest_id integer,
    message text,
    date timestamp with time zone
);
CREATE SEQUENCE im_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
ALTER TABLE ONLY im
    ADD CONSTRAINT im_pkey PRIMARY KEY (msg_id);
CREATE TABLE item_log (
    item_id integer NOT NULL,
    batch_id integer NOT NULL,
    person_id integer NOT NULL,
    change_time timestamp with time zone DEFAULT now() NOT NULL,
    field_name text NOT NULL,
    old_value text NOT NULL,
    new_value text NOT NULL,
    log_flags integer DEFAULT 0 NOT NULL
);
CREATE INDEX item_log_item_ndx ON item_log USING btree (item_id);
CREATE INDEX item_log_person_ndx ON item_log USING btree (person_id);
CREATE INDEX item_log_field_ndx ON item_log USING btree (field_name);
CREATE TABLE item_rels (
    left_item_id integer,
    right_item_id integer,
    relation_flags integer DEFAULT 0 NOT NULL
);
CREATE INDEX item_rels_left ON item_rels USING btree (left_item_id);
CREATE INDEX item_rels_right ON item_rels USING btree (right_item_id);
ALTER TABLE ONLY item_rels
    ADD CONSTRAINT "$1" FOREIGN KEY (left_item_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;
ALTER TABLE ONLY item_rels
    ADD CONSTRAINT "$2" FOREIGN KEY (right_item_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;
CREATE TABLE items_scrollback (
    item_id integer NOT NULL,
    parent_id integer NOT NULL,
    item_name text NOT NULL,
    item_type_id integer NOT NULL,
    deleter_id integer NOT NULL,
    delete_time timestamp with time zone DEFAULT now() NOT NULL,
    conflux_tid integer NOT NULL,
    scrollback_flags integer DEFAULT 0 NOT NULL
);
CREATE UNIQUE INDEX scrollback_itemid_ndx ON items_scrollback USING btree (conflux_tid, item_id);
CREATE INDEX scrollback_item_ndx ON items_scrollback USING btree (item_id);
CREATE TABLE products (
    product_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    product_name text NOT NULL,
    product_group text,
    product_unit text,
    product_price numeric(10,2),
    description text
);
ALTER TABLE ONLY products
    ADD CONSTRAINT products_pkey PRIMARY KEY (product_id);
ALTER TABLE ONLY products
    ADD CONSTRAINT "$1" FOREIGN KEY (product_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;
CREATE TABLE opp_products (
    opp_id integer NOT NULL,
    product_id integer NOT NULL,
    amount numeric(10,2),
    price numeric(10,2),
    key_requirement text
);
CREATE INDEX opp_product_opp_ndx ON opp_products USING btree (opp_id);
CREATE INDEX opp_product_prod_ndx ON opp_products USING btree (product_id);
ALTER TABLE ONLY opp_products
    ADD CONSTRAINT "$1" FOREIGN KEY (opp_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;
ALTER TABLE ONLY opp_products
    ADD CONSTRAINT "$2" FOREIGN KEY (product_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;
CREATE TABLE opportunities (
    opp_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    opp_name text NOT NULL,
    client_id integer NOT NULL,
    opp_stage integer NOT NULL,
    opp_next_stage text,
    description text,
    req_descr text,
    amount numeric(10,2),
    close_date date NOT NULL,
    opp_lost text,
    lead_source text,
    client_type text
);
CREATE INDEX opp_client_ndx ON opportunities USING btree (client_id);   
ALTER TABLE ONLY opportunities
    ADD CONSTRAINT opportunities_pkey PRIMARY KEY (opp_id);
ALTER TABLE ONLY opportunities
    ADD CONSTRAINT "$1" FOREIGN KEY (opp_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;
ALTER TABLE ONLY opportunities
    ADD CONSTRAINT "$2" FOREIGN KEY (client_id) REFERENCES clients(client_id) ON UPDATE RESTRICT ON DELETE RESTRICT;
CREATE TABLE quotas (
    quota_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    quota_soft_size integer NOT NULL,
    quota_hard_size integer NOT NULL,
    quota_used bigint DEFAULT 0 NOT NULL
);
ALTER TABLE ONLY quotas
    ADD CONSTRAINT quotas_pkey PRIMARY KEY (quota_id);
ALTER TABLE ONLY quotas
    ADD CONSTRAINT "$1" FOREIGN KEY (quota_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;
CREATE TABLE workflows (
    workflow_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    workflow_title text NOT NULL,
    workflow_description text,
    workflow_flags integer DEFAULT 0 NOT NULL,
    related_persons integer[] DEFAULT '{}'::integer[] NOT NULL,
    percent_complete integer DEFAULT 0 NOT NULL,
    workflow_start_time date,
    workflow_due_time date
);
ALTER TABLE ONLY workflows
    ADD CONSTRAINT workflows_pkey PRIMARY KEY (workflow_id);
ALTER TABLE ONLY workflows
    ADD CONSTRAINT "$1" FOREIGN KEY (workflow_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;
CREATE TABLE messages (
    message_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    message_source integer NOT NULL,
    sender_org_id integer,
    sender_org_name text,
    sender_nr text,
    receiver_org_id integer,
    receiver_org_name text,
    receiver_person_id integer,
    receiver_person_name text,
    responsible_person integer,
    message_contents text,
    message_type text,
    delivery_method text,
    priority text,
    date_dispatch timestamp with time zone,
    date_arrival timestamp with time zone,
    date_deadline timestamp with time zone,
    date_answered timestamp with time zone,
    message_title text NOT NULL
);
ALTER TABLE ONLY messages
    ADD CONSTRAINT messages_pkey PRIMARY KEY (message_id);
ALTER TABLE ONLY messages
    ADD CONSTRAINT "$1" FOREIGN KEY (message_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;
CREATE TABLE virt_folders (
    virt_folder_id integer DEFAULT currval('item_id_seq'::text) NOT NULL,
    folder_name character varying(128),
    comments text
);
ALTER TABLE ONLY virt_folders
    ADD CONSTRAINT virt_folders_pkey PRIMARY KEY (virt_folder_id);
ALTER TABLE ONLY virt_folders
    ADD CONSTRAINT "$1" FOREIGN KEY (virt_folder_id) REFERENCES items(item_id) ON UPDATE RESTRICT ON DELETE CASCADE;
CREATE SEQUENCE log_batch_seq
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
INSERT INTO item_types VALUES (16, 'Workflows', 'workflow.gif');
INSERT INTO item_types VALUES (18, 'Client', 'client.gif');
INSERT INTO item_types VALUES (19, 'Opportunity', 'opportunity.gif');
INSERT INTO item_types VALUES (20, 'Product', 'product.gif');
INSERT INTO item_types VALUES (21, 'Quota', 'quota.gif');
INSERT INTO item_types VALUES (23, 'Contact groups', 'contactgroups.gif');
INSERT INTO item_types VALUES (24, 'Virtual folders', 'fld.gif');
INSERT INTO item_types VALUES (25, 'Messages', 'message.gif');
INSERT INTO item_types VALUES (26, 'Translation', 'translation.gif');
INSERT INTO item_types VALUES (27, 'Service', 'services.gif');
INSERT INTO item_types VALUES (28, 'Translation', 'document.gif');""")

con.commit()
cur.close()
con.close()
